@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -207,13 +207,13 @@ class clock ?(start = true) ?(sync = `Auto) id =
       in
       begin
         match (self_sync, new_val) with
-        | None, false | Some true, false ->
-            log#important "Delegating synchronisation to CPU clock";
-            t0 <- time ();
-            ticks <- 0L
-        | None, true | Some false, true ->
-            log#important "Delegating synchronisation to active sources"
-        | _ -> ()
+          | None, false | Some true, false ->
+              log#important "Delegating synchronisation to CPU clock";
+              t0 <- time ();
+              ticks <- 0L
+          | None, true | Some false, true ->
+              log#important "Delegating synchronisation to active sources"
+          | _ -> ()
       end;
       self_sync <- Some new_val;
       new_val
@@ -352,7 +352,7 @@ class clock ?(start = true) ?(sync = `Auto) id =
       fun () ->
         let to_start =
           if to_start <> [] then
-            log#info "Starting %d sources..." (List.length to_start);
+            log#info "Starting %d source(s)..." (List.length to_start);
           List.map
             (fun (s : active_source) ->
               try
@@ -468,7 +468,7 @@ let gc_alarm =
   fun () ->
     let nb_clocks = Clocks.count clocks in
     if nb_clocks <> !last_displayed then (
-      log#info "Currently %d clocks allocated." nb_clocks;
+      log#info "Currently %d clock(s) allocated." nb_clocks;
       last_displayed := nb_clocks)
 
 let () = ignore (Gc.create_alarm gc_alarm)
