@@ -33,6 +33,11 @@ val dynlink : bool
 (** Where to look for private executables. *)
 val bin_dir : string
 
+module CamomileCharEncoding : CamomileLibrary.CharEncoding.Interface
+
+(** Where to find the camomile runtime files. *)
+val camomile_dir : string
+
 (** Standard path. *)
 val path : string list
 
@@ -51,7 +56,6 @@ val requests_max_id : int
 (** Magic mime detection *)
 val file_mime : (string -> string) option
 
-val data_mime : (?len:int -> string -> string) option
 val requests_table_size : int
 
 (** Configured directories. Typically /var/(run|log)/liquidsoap. *)
@@ -73,6 +77,3 @@ val libs_versions : string
 
 (** File watch utility. *)
 val file_watcher : File_watcher.watch ref
-
-(** JSON parser. *)
-module JSON : module type of JSON
