@@ -45,11 +45,7 @@ module Samplerate = struct
   (** A converter takes a conversion ratio (output samplerate / input
       samplerate), an audio buffer, and returns a resampled buffer. *)
   type converter =
-    float ->
-    Frame_content.Audio.data ->
-    int ->
-    int ->
-    Frame_content.Audio.data * int * int
+    float -> Content.Audio.data -> int -> int -> Content.Audio.data * int * int
 
   type converter_plug = int -> converter
   type t = { channels : int; converter : converter }
@@ -64,7 +60,7 @@ module Samplerate = struct
     Dtools.Conf.list
       ~p:(samplerate_conf#plug "converters")
       "Preferred samplerate converters"
-      ~d:["ffmpeg"; "libsamplerate"; "native"]
+      ~d:["libsamplerate"; "ffmpeg"; "native"]
       ~comments:
         [
           "Preferred samplerate converter. The native converter is always \
@@ -111,13 +107,11 @@ module Channel_layout = struct
   exception Invalid_data
 
   type layout = [ `Mono | `Stereo | `Five_point_one ]
-
-  type converter =
-    layout -> layout -> Frame_content.Audio.data -> Frame_content.Audio.data
+  type converter = layout -> layout -> Content.Audio.data -> Content.Audio.data
 
   type t = {
     src : layout;
-    converter : Frame_content.Audio.data -> Frame_content.Audio.data;
+    converter : Content.Audio.data -> Content.Audio.data;
   }
 
   let channel_layout_conf =
