@@ -30,11 +30,7 @@ module Samplerate : sig
   exception Invalid_data
 
   type converter =
-    float ->
-    Frame_content.Audio.data ->
-    int ->
-    int ->
-    Frame_content.Audio.data * int * int
+    float -> Content.Audio.data -> int -> int -> Content.Audio.data * int * int
 
   type converter_plug = int -> converter
   type t
@@ -51,10 +47,10 @@ module Samplerate : sig
   val resample :
     t ->
     float ->
-    Frame_content.Audio.data ->
+    Content.Audio.data ->
     int ->
     int ->
-    Frame_content.Audio.data * int * int
+    Content.Audio.data * int * int
 end
 
 module Channel_layout : sig
@@ -62,10 +58,7 @@ module Channel_layout : sig
   exception Invalid_data
 
   type layout = [ `Mono | `Stereo | `Five_point_one ]
-
-  type converter =
-    layout -> layout -> Frame_content.Audio.data -> Frame_content.Audio.data
-
+  type converter = layout -> layout -> Content.Audio.data -> Content.Audio.data
   type t
 
   val channels_of_layout : layout -> int
@@ -79,5 +72,5 @@ module Channel_layout : sig
   (** [convert converter data]: converts input data to the destination layout.
       Raises [Invalid_data] if input layout does not match the layout passed
       as [create]. *)
-  val convert : t -> Frame_content.Audio.data -> Frame_content.Audio.data
+  val convert : t -> Content.Audio.data -> Content.Audio.data
 end
