@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -56,5 +56,4 @@ val find_converter :
   Image.Generic.t ->
   unit
 
-val scaler :
-  unit -> ?proportional:bool -> Video.Image.t -> Video.Image.t -> unit
+val scaler : unit -> Video.Image.t -> Video.Image.t -> unit
