@@ -51,8 +51,8 @@ type fps = Decoder_utils.fps = { num : int; den : int }
     - Implicit content drop *)
 type buffer = {
   generator : G.t;
-  put_pcm : ?pts:Int64.t -> samplerate:int -> Frame_content.Audio.data -> unit;
-  put_yuva420p : ?pts:Int64.t -> fps:fps -> Frame_content.Video.data -> unit;
+  put_pcm : ?pts:Int64.t -> samplerate:int -> Content.Audio.data -> unit;
+  put_yuva420p : ?pts:Int64.t -> fps:fps -> Content.Video.data -> unit;
 }
 
 type decoder = {
@@ -86,7 +86,7 @@ type decoder_specs = {
    * so a file with mime-type foo/bar; bla
    * matches mime-type foo/bar. Furthermore,
    * for streams, a stream with mime foo/bar<whatever>
-   * matches mime-type foo/bar. 
+   * matches mime-type foo/bar.
    * None means accept all mime-types. *)
   mime_types : unit -> string list option;
   (* None means no decodable content for that file. *)
