@@ -24,10 +24,10 @@ open Mm
 
 type samplerate_converter =
   samplerate:int ->
-  Frame_content.Audio.data ->
+  Content.Audio.data ->
   int ->
   int ->
-  Frame_content.Audio.data * int * int
+  Content.Audio.data * int * int
 
 let samplerate_converter () =
   let state = ref None in
@@ -43,7 +43,7 @@ let samplerate_converter () =
           state := Some converter;
           Audio_converter.Samplerate.resample converter ratio audio_buf 0 len
 
-type wav_converter = bytes -> int -> int -> Frame_content.Audio.data
+type wav_converter = bytes -> int -> int -> Content.Audio.data
 
 let from_iff ~format ~channels ~samplesize =
   let sample_bytes = samplesize / 8 in
@@ -70,7 +70,7 @@ let from_iff ~format ~channels ~samplesize =
     to_audio src 0 dst 0 sample_len;
     dst
 
-type channels_converter = Frame_content.Audio.data -> Frame_content.Audio.data
+type channels_converter = Content.Audio.data -> Content.Audio.data
 
 let channels_converter dst =
   let converter = ref None in
@@ -85,30 +85,9 @@ let channels_converter dst =
           converter := Some (c, _src);
           Audio_converter.Channel_layout.convert c data
 
-let video_scale () =
-  let dst_width = Lazy.force Frame.video_width in
-  let dst_height = Lazy.force Frame.video_height in
-  let video_scale = Video_converter.scaler () ~proportional:true in
-  fun img ->
-    let src_width = Video.Image.width img in
-    let src_height = Video.Image.height img in
-    if (src_width, src_height) = (dst_width, dst_height) then img
-    else (
-      let scl_width, scl_height =
-        if dst_height * src_width < dst_width * src_height then
-          (src_width * dst_height / src_height, dst_height)
-        else (dst_width, src_height * dst_width / src_width)
-      in
-      let img2 = Video.Image.create scl_width scl_height in
-      video_scale img img2;
-      if (scl_width, scl_height) = (dst_width, dst_height) then img2
-      else (
-        let img3 = Video.Image.create dst_width dst_height in
-        Video.Image.blank img3;
-        let x = (dst_width - scl_width) / 2 in
-        let y = (dst_height - scl_height) / 2 in
-        Image.YUV420.add img2 ~x ~y img3;
-        img3))
+let video_scale ~width ~height () =
+  let scaler = Video_converter.scaler () in
+  Video.Canvas.Image.resize ~scaler width height
 
 type fps = { num : int; den : int }
 
@@ -144,8 +123,8 @@ let video_resample ~in_freq ~out_freq =
 
 let video_resample () =
   let state = ref None in
-  let exec resampler data = resampler data 0 (Array.length data) in
-  fun ~in_freq ~out_freq data ->
+  let exec resampler data = resampler data 0 (Video.Canvas.length data) in
+  fun ~in_freq ~out_freq (data : Content.Video.data) : Content.Video.data ->
     if in_freq = out_freq then data
     else (
       match !state with
