@@ -26,14 +26,14 @@ open Mm
 
 type samplerate_converter =
   samplerate:int ->
-  Frame_content.Audio.data ->
+  Content.Audio.data ->
   int ->
   int ->
-  Frame_content.Audio.data * int * int
+  Content.Audio.data * int * int
 
 val samplerate_converter : unit -> samplerate_converter
 
-type wav_converter = bytes -> int -> int -> Frame_content.Audio.data
+type wav_converter = bytes -> int -> int -> Content.Audio.data
 
 (** samplesize is in bits.
     Formats: unsigned 8 bit (u8) or
@@ -41,12 +41,17 @@ type wav_converter = bytes -> int -> int -> Frame_content.Audio.data
 val from_iff :
   format:Wav_aiff.format -> channels:int -> samplesize:int -> wav_converter
 
-type channels_converter = Frame_content.Audio.data -> Frame_content.Audio.data
+type channels_converter = Content.Audio.data -> Content.Audio.data
 
 val channels_converter :
   Audio_converter.Channel_layout.layout -> channels_converter
 
-val video_scale : unit -> Video.Image.t -> Video.Image.t
+val video_scale :
+  width:int ->
+  height:int ->
+  unit ->
+  Video.Canvas.Image.t ->
+  Video.Canvas.Image.t
 
 type fps = { num : int; den : int }
 
@@ -54,5 +59,5 @@ val video_resample :
   unit ->
   in_freq:fps ->
   out_freq:fps ->
-  Frame_content.Video.data ->
-  Frame_content.Video.data
+  Content.Video.data ->
+  Content.Video.data
