@@ -63,10 +63,9 @@ let mk_audio_decoder ~stream_idx ~format container =
   let idx, stream, params = Av.find_best_audio_stream container in
   Ffmpeg_decoder_common.set_audio_stream_decoder stream;
   ignore
-    (Frame_content.merge format
-       Ffmpeg_copy_content.(Audio.lift_params (Some params)));
+    (Content.merge format Ffmpeg_copy_content.(Audio.lift_params (Some params)));
   let stream_time_base = Av.get_time_base stream in
-  let lift_data = Ffmpeg_copy_content.Audio.lift_data in
+  let lift_data data = Ffmpeg_copy_content.Audio.lift_data data in
   ( idx,
     stream,
     mk_decoder ~stream_idx ~lift_data ~stream_time_base ~put_data:G.put_audio
@@ -76,10 +75,9 @@ let mk_video_decoder ~stream_idx ~format container =
   let idx, stream, params = Av.find_best_video_stream container in
   Ffmpeg_decoder_common.set_video_stream_decoder stream;
   ignore
-    (Frame_content.merge format
-       Ffmpeg_copy_content.(Video.lift_params (Some params)));
+    (Content.merge format Ffmpeg_copy_content.(Video.lift_params (Some params)));
   let stream_time_base = Av.get_time_base stream in
-  let lift_data = Ffmpeg_copy_content.Video.lift_data in
+  let lift_data data = Ffmpeg_copy_content.Video.lift_data data in
   ( idx,
     stream,
     mk_decoder ~stream_idx ~lift_data ~stream_time_base ~put_data:G.put_video
