@@ -36,7 +36,8 @@ type gst = {
 }
 
 (** Generic decoder. *)
-let create_decoder ?(merge_tracks = false) _ ~channels ~mode fname =
+let create_decoder ?(merge_tracks = false) _ ~width ~height ~channels ~mode
+    fname =
   let decode_audio, decode_video =
     match mode with
       | `Both -> (true, true)
@@ -90,8 +91,6 @@ let create_decoder ?(merge_tracks = false) _ ~channels ~mode fname =
     in
     { bin; audio_sink; video_sink }
   in
-  let width = Lazy.force Frame.video_width in
-  let height = Lazy.force Frame.video_height in
   let started = ref false in
   let init ~reset buffer =
     if reset then
@@ -137,7 +136,7 @@ let create_decoder ?(merge_tracks = false) _ ~channels ~mode fname =
       let y_stride = round4 width in
       let uv_stride = round4 (width / 2) in
       let img = Image.YUV420.make_data width height buf y_stride uv_stride in
-      let stream = Video.single img in
+      let stream = Video.Canvas.single_image img in
       let fps = { Decoder.num = Lazy.force Frame.video_rate; den = 1 } in
       buffer.Decoder.put_yuva420p ~fps stream);
     GU.flush ~log gst.bin
@@ -189,19 +188,21 @@ let priority =
     "Priority for the GStreamer decoder" ~d:0
 
 let channels { Frame.audio } =
-  if audio = Frame_content.None.format then 0
-  else Frame_content.Audio.channels_of_format audio
+  if audio = Content.None.format then 0
+  else Content.Audio.channels_of_format audio
 
 let create_file_decoder filename content_type ctype =
   let mode =
     match (content_type.Frame.video, content_type.Frame.audio) with
-      | video, _ when video = Frame_content.None.format -> `Audio
-      | _, audio when audio = Frame_content.None.format -> `Video
+      | video, _ when video = Content.None.format -> `Audio
+      | _, audio when audio = Content.None.format -> `Video
       | _, _ -> `Both
   in
   let channels = channels content_type in
   let decoder, close, bin =
-    create_decoder ~channels ~merge_tracks:true ~mode `File filename
+    let width, height = Content.Video.dimensions_of_format ctype.Frame.video in
+    create_decoder ~width ~height ~channels ~merge_tracks:true ~mode `File
+      filename
   in
   let remaining () =
     let pos =
@@ -265,9 +266,9 @@ let get_type ~channels filename =
     with Gstreamer.Failed -> 0
   in
   let audio =
-    if audio = 0 then Frame_content.None.format
+    if audio = 0 then Content.None.format
     else
-      Frame_content.(
+      Content.(
         Audio.lift_params
           {
             Contents.channel_layout =
@@ -275,10 +276,10 @@ let get_type ~channels filename =
           })
   in
   let video =
-    if video = 0 then Frame_content.None.format
-    else Frame_content.(default_format Video.kind)
+    if video = 0 then Content.None.format
+    else Content.(default_format Video.kind)
   in
-  { Frame.video; audio; midi = Frame_content.None.format }
+  { Frame.video; audio; midi = Content.None.format }
 
 let file_decoder ~metadata:_ ~ctype filename =
   let channels = channels ctype in
