@@ -34,15 +34,16 @@ let load_image fname =
   in
   let width = Avcodec.Video.get_width codec in
   let height = Avcodec.Video.get_height codec in
+  (* Hardcoding this instead of using Ffmpeg_utils.liq_frame_pixel_format () in
+     order to have alpha channel. *)
+  let out_pixel_format = `Yuva420p in
   let scaler =
-    Scaler.create [] width height pixel_format width height
-      (Ffmpeg_utils.liq_frame_pixel_format ())
+    Scaler.create [] width height pixel_format width height out_pixel_format
   in
   match Av.read_input ~video_frame:[stream] container with
     | `Video_frame (_, frame) ->
-        Some
-          (Ffmpeg_utils.unpack_image ~width ~height
-             (Scaler.convert scaler frame))
+        let frame = Scaler.convert scaler frame in
+        Some (Ffmpeg_utils.unpack_image ~width ~height frame)
     | _ -> None
 
 let () =
