@@ -126,9 +126,9 @@ let file_type filename =
         rate channels;
       Some
         {
-          Frame.audio = Frame_content.Audio.format_of_channels channels;
-          video = Frame_content.None.format;
-          midi = Frame_content.None.format;
+          Frame.audio = Content.Audio.format_of_channels channels;
+          video = Content.None.format;
+          midi = Content.None.format;
         })
 
 let file_decoder ~metadata:_ ~ctype filename =
