@@ -92,7 +92,7 @@ let create_decoder ?(merge_tracks = false) source input =
     in
     Ogg_decoder.init ~log:demuxer_log callbacks
   in
-  let video_scale = Video_converter.scaler () ~proportional:true in
+  let video_scale = Video_converter.scaler () in
   let started = ref false in
   let tracks = Ogg_decoder.get_standard_tracks decoder in
   let first_meta = ref true in
@@ -167,7 +167,7 @@ let create_decoder ?(merge_tracks = false) source input =
             den = info.Ogg_decoder.fps_denominator;
           }
         in
-        buffer.Decoder.put_yuva420p ~fps (Video.single rgb)
+        buffer.Decoder.put_yuva420p ~fps (Video.Canvas.single_image rgb)
       in
       let decode_audio, decode_video =
         if decode_audio && decode_video then
@@ -188,8 +188,8 @@ let create_decoder ?(merge_tracks = false) source input =
         Ogg_decoder.decode_video decoder track (video_feed track))
     with
       (* We catch [Ogg_decoder.End_of_stream] only if asked to
-       * to merge logical tracks or with a stream source. 
-       * In this case, we try to reset the decoder to see if 
+       * to merge logical tracks or with a stream source.
+       * In this case, we try to reset the decoder to see if
        * there could be another sequentialized logical stream
        * starting. Actual reset is handled in the
        * decoding function since we need the actual
@@ -237,9 +237,9 @@ let file_type ~ctype:_ filename =
         (Utils.quote_string filename)
         audio video;
       let audio =
-        if audio = 0 then Frame_content.None.format
+        if audio = 0 then Content.None.format
         else
-          Frame_content.(
+          Content.(
             Audio.lift_params
               {
                 Contents.channel_layout =
@@ -247,10 +247,10 @@ let file_type ~ctype:_ filename =
               })
       in
       let video =
-        if video = 0 then Frame_content.None.format
-        else Frame_content.(default_format Video.kind)
+        if video = 0 then Content.None.format
+        else Content.(default_format Video.kind)
       in
-      Some { Frame.audio; video; midi = Frame_content.None.format })
+      Some { Frame.audio; video; midi = Content.None.format })
 
 let mime_types =
   Dtools.Conf.list
