@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -71,10 +71,9 @@ let () =
           Some
             Frame.
               {
-                audio = Frame_content.None.format;
-                video = Frame_content.None.format;
-                midi =
-                  Frame_content.(Midi.lift_params { Contents.channels = 16 });
+                audio = Content.None.format;
+                video = Content.None.format;
+                midi = Content.(Midi.lift_params { Contents.channels = 16 });
               });
       file_decoder =
         Some (fun ~metadata:_ ~ctype:_ filename -> decoder filename);
