@@ -27,7 +27,7 @@ let log = Ffmpeg_utils.log
 type encoder = {
   mk_stream : Frame.t -> unit;
   encode : Frame.t -> int -> int -> unit;
-  was_keyframe : unit -> bool;
+  can_split : unit -> bool;
   codec_attr : unit -> string option;
   bitrate : unit -> int option;
   video_size : unit -> (int * int) option;
@@ -40,7 +40,12 @@ type handler = {
   mutable started : bool;
 }
 
-type stream_data = { idx : Int64.t; mutable last_start : Int64.t }
+type stream_data = {
+  idx : Int64.t;
+  mutable last_start : Int64.t;
+  mutable ready_count : int;
+  mutable ready : bool;
+}
 
 module Stream = Weak.Make (struct
   type t = stream_data
@@ -56,13 +61,29 @@ end)
      v: ----------->vls
    the last start should be vls so we end up with:
      a: ------>     -------->
-     v: ----------->--------> *)
-let mk_stream_store () =
+     v: ----------->-------->
+
+   Also, if one or more stream is waiting for a key frame,
+   we make the whole set of streams wait for the first key
+   frame. This makes sure that we avoid e.g. starting an
+   audio track with no keyframe before the video has started. *)
+let mk_stream_store total_count =
   let store = Stream.create 1 in
-  fun ~last_start idx ->
-    let data = Stream.merge store { idx; last_start } in
+  let add ~last_start ~ready idx =
+    let data =
+      Stream.merge store { idx; ready_count = 0; last_start; ready = false }
+    in
+    if ready then data.ready_count <- data.ready_count + 1;
+    if data.ready_count = total_count then data.ready <- true;
     if data.last_start < last_start then data.last_start <- last_start;
     data
+  in
+  let remove data =
+    let data = Stream.merge store data in
+    if data.ready_count = 1 then Stream.remove store data
+    else data.ready_count <- data.ready_count - 1
+  in
+  (add, remove)
 
 let mk_format ffmpeg =
   match (ffmpeg.Ffmpeg_format.format, ffmpeg.Ffmpeg_format.output) with
@@ -170,12 +191,12 @@ let encoder ~mk_audio ~mk_video ffmpeg meta =
       | Some "mp4" ->
           encode ~encoder frame start len;
           if encoder.video_stream <> None then (
-            let d = Frame_content.sub Frame.(frame.content.video) start len in
-            video_sent := !video_sent || not (Frame_content.is_empty d))
+            let d = Content.sub (Frame.video frame) start len in
+            video_sent := !video_sent || not (Content.is_empty d))
           else video_sent := true;
           if encoder.audio_stream <> None then (
-            let d = Frame_content.sub Frame.(frame.content.audio) start len in
-            audio_sent := !audio_sent || not (Frame_content.is_empty d))
+            let d = Content.sub (Frame.audio frame) start len in
+            audio_sent := !audio_sent || not (Content.is_empty d))
           else audio_sent := true;
           if not (!audio_sent && !video_sent) then raise Encoder.Not_enough_data;
           Av.flush encoder.output;
@@ -197,7 +218,7 @@ let encoder ~mk_audio ~mk_video ffmpeg meta =
     encode ~encoder frame start len;
     let encoded = Strings.Mutable.flush buf in
     match encoder.video_stream with
-      | Some s when s.was_keyframe () -> `Ok (flushed, encoded)
+      | Some s when s.can_split () -> `Ok (flushed, encoded)
       | None -> `Ok (flushed, encoded)
       | _ -> `Nope (Strings.append flushed encoded)
   in
