@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -47,16 +47,11 @@ let encoder wav =
   let need_header = ref wav.header in
   let encode frame start len =
     let b = AFrame.pcm frame in
+    let start = Frame.audio_of_main start in
     let len = Frame.audio_of_main len in
     (* Resample if needed. *)
     let b, start, len =
-      if ratio = 1. then (b, start, len)
-      else (
-        let b =
-          Audio_converter.Samplerate.resample converter ratio
-            (Audio.sub b start len)
-        in
-        (b, 0, Audio.length b))
+      Audio_converter.Samplerate.resample converter ratio b start len
     in
     let s = Bytes.create (sample_size / 8 * len * channels) in
     let of_audio =
@@ -67,7 +62,7 @@ let encoder wav =
         | 8 -> fun buf s off -> Audio.U8.of_audio buf s off
         | _ -> failwith "unsupported sample size"
     in
-    of_audio (Audio.sub b start len) s 0;
+    of_audio b start s 0 len;
     let s = Bytes.unsafe_to_string s in
     if !need_header then (
       need_header := false;
