@@ -22,12 +22,15 @@
 
 type t = {
   (* Samplerate is lazy in order to avoid forcing the evaluation of the
-       samplerate at typing time, see #933. For channels this is pointless since
-       we really need that for typing. *)
+     samplerate at typing time, see #933. For channels this is pointless since
+     we really need that for typing. *)
   samplerate : int Lazy.t;
   channels : int;
+  width : int Lazy.t;
+  height : int Lazy.t;
 }
 
 let to_string w =
-  let samplerate = Lazy.force w.samplerate in
-  Printf.sprintf "%%avi(samplerate=%d,channels=%d)" samplerate w.channels
+  Printf.sprintf "%%avi(samplerate=%d,channels=%d,width=%d,height=%d)"
+    (Lazy.force w.samplerate) w.channels (Lazy.force w.width)
+    (Lazy.force w.height)
