@@ -36,6 +36,7 @@ class virtual base dev mode =
   let samples_per_frame = AFrame.size () in
   let periods = Alsa_settings.periods#get in
   object (self)
+    inherit Source.no_seek
     method virtual log : Log.t
     method virtual audio_channels : int
     val mutable alsa_rate = -1
@@ -161,8 +162,8 @@ class output ~kind ~clock_safe ~start ~infallible ~on_stop ~on_start dev
   object (self)
     inherit
       Output.output
-        ~infallible ~on_stop ~on_start ~content_kind:(Source.Kind.of_kind kind)
-          ~name ~output_kind:"output.alsa" val_source start as super
+        ~infallible ~on_stop ~on_start ~content_kind:(Kind.of_kind kind) ~name
+          ~output_kind:"output.alsa" val_source start as super
 
     inherit base dev [Pcm.Playback]
 
@@ -209,10 +210,11 @@ class output ~kind ~clock_safe ~start ~infallible ~on_stop ~on_start dev
       with e ->
         begin
           match e with
-          | Buffer_xrun ->
-              self#log#severe
-                "Underrun! You may minimize them by increasing the buffer size."
-          | _ -> self#log#severe "Alsa error: %s" (string_of_error e)
+            | Buffer_xrun ->
+                self#log#severe
+                  "Underrun! You may minimize them by increasing the buffer \
+                   size."
+            | _ -> self#log#severe "Alsa error: %s" (string_of_error e)
         end;
         if e = Buffer_xrun || e = Suspended || e = Interrupted then (
           self#log#severe "Trying to recover..";
@@ -228,8 +230,7 @@ class input ~kind ~clock_safe ~start ~on_stop ~on_start ~fallible dev =
 
     inherit
       Start_stop.active_source
-        ~get_clock:Alsa_settings.get_clock
-          ~content_kind:(Source.Kind.of_kind kind)
+        ~get_clock:Alsa_settings.get_clock ~content_kind:(Kind.of_kind kind)
         ~name:(Printf.sprintf "alsa_in(%s)" dev)
         ~clock_safe ~on_start ~on_stop ~fallible ~autostart:start ()
 
@@ -256,10 +257,11 @@ class input ~kind ~clock_safe ~start ~on_stop ~on_start ~fallible dev =
       with e ->
         begin
           match e with
-          | Buffer_xrun ->
-              self#log#severe
-                "Overrun! You may minimize them by increasing the buffer size."
-          | _ -> self#log#severe "Alsa error: %s" (string_of_error e)
+            | Buffer_xrun ->
+                self#log#severe
+                  "Overrun! You may minimize them by increasing the buffer \
+                   size."
+            | _ -> self#log#severe "Alsa error: %s" (string_of_error e)
         end;
         if e = Buffer_xrun || e = Suspended || e = Interrupted then (
           self#log#severe "Trying to recover..";
