@@ -41,7 +41,8 @@ class audio_output ~pass_metadata ~name ~kind source_val =
         ~infallible:false ~on_stop:noop ~on_start:noop ~content_kind:kind ~name
           ~output_kind:"ffmpeg.filter.input" source_val true
 
-    initializer Source.Kind.unify (Lang.to_source source_val)#kind self#kind
+    inherit Source.no_seek
+    initializer Kind.unify (Lang.to_source source_val)#kind self#kind
     val mutable input = fun _ -> ()
     method set_input fn = input <- fn
     val mutable init : Avutil.audio Avutil.frame -> unit = fun _ -> assert false
@@ -53,7 +54,7 @@ class audio_output ~pass_metadata ~name ~kind source_val =
     method send_frame memo =
       let frames =
         Ffmpeg_raw_content.(
-          (Audio.get_data Frame.(memo.content.audio)).AudioSpecs.data)
+          (Audio.get_data (AFrame.content memo)).AudioSpecs.data)
       in
       List.iter
         (fun (pos, { Ffmpeg_raw_content.frame }) ->
@@ -91,7 +92,7 @@ class video_output ~pass_metadata ~kind ~name source_val =
         ~infallible:false ~on_stop:noop ~on_start:noop ~content_kind:kind ~name
           ~output_kind:"ffmpeg.filter.input" source_val true
 
-    initializer Source.Kind.unify (Lang.to_source source_val)#kind self#kind
+    initializer Kind.unify (Lang.to_source source_val)#kind self#kind
     val mutable input : Swscale.Frame.t -> unit = fun _ -> ()
     method set_input fn = input <- fn
     val mutable init : Avutil.video Avutil.frame -> unit = fun _ -> assert false
@@ -103,7 +104,7 @@ class video_output ~pass_metadata ~kind ~name source_val =
     method send_frame memo =
       let frames =
         Ffmpeg_raw_content.(
-          (Video.get_data Frame.(memo.content.video)).VideoSpecs.data)
+          (Video.get_data (VFrame.content memo)).VideoSpecs.data)
       in
       List.iter
         (fun (pos, { Ffmpeg_raw_content.frame }) ->
@@ -181,6 +182,7 @@ class audio_input ~self_sync_type ~self_sync ~is_ready ~pull ~pass_metadata kind
   let stream_idx = Ffmpeg_content_base.new_stream_idx () in
   object (self)
     inherit Source.source kind ~name:"ffmpeg.filter.output"
+    inherit Source.no_seek
 
     inherit
       [[ `Audio ]] input_base
@@ -197,11 +199,11 @@ class audio_input ~self_sync_type ~self_sync ~is_ready ~pull ~pass_metadata kind
           sample_format = Some Avfilter.(sample_format v.context);
         }
       in
-      Frame_content.merge self#ctype.Frame.audio
+      Content.merge self#ctype.Frame.audio
         (Ffmpeg_raw_content.Audio.lift_params output_format);
       output <- Some v
 
-    method stype = Source.Fallible
+    method stype = `Fallible
     method remaining = Generator.remaining generator
 
     method private flush_buffer output =
@@ -261,6 +263,7 @@ class video_input ~self_sync_type ~self_sync ~is_ready ~pull ~pass_metadata ~fps
   let stream_idx = Ffmpeg_content_base.new_stream_idx () in
   object (self)
     inherit Source.source kind ~name:"ffmpeg.filter.output"
+    inherit Source.no_seek
 
     inherit
       [[ `Video ]] input_base
@@ -275,11 +278,11 @@ class video_input ~self_sync_type ~self_sync ~is_ready ~pull ~pass_metadata ~fps
           pixel_aspect = Avfilter.(pixel_aspect v.context);
         }
       in
-      Frame_content.merge self#ctype.Frame.video
+      Content.merge self#ctype.Frame.video
         (Ffmpeg_raw_content.Video.lift_params output_format);
       output <- Some v
 
-    method stype = Source.Fallible
+    method stype = `Fallible
     method remaining = Generator.remaining generator
 
     method private flush_buffer output =
