@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
         ~name ~content_kind:kind ~fallible:true ~clock_safe ~on_start ~on_stop
           ~autostart () as super
 
+    inherit Source.no_seek
     val mutable connect_task = None
     val mutable container = None
     val generator = generator
@@ -85,7 +86,7 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
 
     method source_status = source_status
 
-    method private connect_fn () =
+    method private connect_task () =
       try
         source_status <- `Polling;
         let opts = Hashtbl.copy opts in
@@ -107,6 +108,7 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
           Ffmpeg_decoder.mk_decoder ?audio ?video ~decode_first_metadata:true
             ~target_position:(ref None) input
         in
+        let buffer = Decoder.mk_buffer ~ctype:self#ctype generator in
         Generator.set_rewrite_metadata generator (fun m ->
             Hashtbl.replace m "source_url" url;
             m);
@@ -123,64 +125,63 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
               | Some (`Frame (_, stream, _)) -> Av.get_metadata stream
               | None -> [])
         in
-        container <- Some (input, decoder, get_metadata);
+        container <- Some (input, decoder, buffer, get_metadata);
         source_status <- `Connected url;
         on_connect input;
         -1.
       with e ->
         self#log#info "Connection failed: %s" (Printexc.to_string e);
-        self#disconnect;
+        self#disconnect_fn;
         if debug then raise e;
         poll_delay
 
-    method private connect =
-      self#mutexify
-        (fun () ->
-          if container = None then (
-            match connect_task with
-              | Some t -> Duppy.Async.wake_up t
-              | None ->
-                  let t =
-                    Duppy.Async.add ~priority:Tutils.Blocking Tutils.scheduler
-                      self#connect_fn
-                  in
-                  connect_task <- Some t;
-                  Duppy.Async.wake_up t))
-        ()
+    method private connect_fn =
+      if container = None then (
+        match connect_task with
+          | Some t -> Duppy.Async.wake_up t
+          | None ->
+              let t =
+                Duppy.Async.add ~priority:`Blocking Tutils.scheduler
+                  self#connect_task
+              in
+              connect_task <- Some t;
+              Duppy.Async.wake_up t)
 
-    method private disconnect =
-      self#mutexify
-        (fun () ->
-          match container with
-            | None -> ()
-            | Some (input, _, _) ->
-                Printf.printf "Setting interrupt to true\n%!";
-                Tutils.mutexify interrupt_m (fun () -> interrupt <- true) ();
-                (try Av.close input
-                 with exn ->
-                   let bt = Printexc.get_backtrace () in
-                   Utils.log_exception ~log:self#log ~bt
-                     (Printf.sprintf "Error while disconnecting: %s"
-                        (Printexc.to_string exn)));
-                container <- None;
-                source_status <- `Stopped;
-                Tutils.mutexify interrupt_m (fun () -> interrupt <- false) ();
-                on_disconnect ())
-        ()
+    method private connect = self#mutexify (fun () -> self#connect_fn) ()
+
+    method private disconnect_fn =
+      match container with
+        | None -> ()
+        | Some (input, _, _, _) ->
+            Tutils.mutexify interrupt_m (fun () -> interrupt <- true) ();
+            (try Av.close input
+             with exn ->
+               let bt = Printexc.get_backtrace () in
+               Utils.log_exception ~log:self#log ~bt
+                 (Printf.sprintf "Error while disconnecting: %s"
+                    (Printexc.to_string exn)));
+            container <- None;
+            source_status <- `Stopped;
+            Tutils.mutexify interrupt_m (fun () -> interrupt <- false) ();
+            on_disconnect ()
+
+    method private disconnect = self#mutexify (fun () -> self#disconnect_fn) ()
 
     method private reconnect =
-      self#disconnect;
-      self#connect
+      self#mutexify
+        (fun () ->
+          self#disconnect_fn;
+          self#connect_fn)
+        ()
 
     val mutable last_metadata = []
 
     method private get_frame frame =
       let pos = Frame.position frame in
       try
-        let _, decoder, get_metadata =
+        let _, decoder, buffer, get_metadata =
           self#mutexify (fun () -> Option.get container) ()
         in
-        let buffer = Decoder.mk_buffer ~ctype:self#ctype generator in
         while Generator.length generator < Lazy.force Frame.size do
           decoder buffer
         done;
@@ -367,7 +368,9 @@ let register_input is_http =
               ([], fun_t [(false, "", getter_t string_t)] unit_t),
               "Set the source's url.",
               fun s ->
-                val_fun [("", "", None)] (fun p ->
+                val_fun
+                  [("", "", None)]
+                  (fun p ->
                     s#set_url (to_string_getter (List.assoc "" p));
                     unit) );
             ( "status",
@@ -431,7 +434,7 @@ let register_input is_http =
       in
       let poll_delay = Lang.to_float (List.assoc "poll_delay" p) in
       let url = Lang.to_string_getter (Lang.assoc "" 1 p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       if is_http then (
         let timeout = Lang.to_float (List.assoc "timeout" p) in
         let user_agent = Lang.to_string (List.assoc "user_agent" p) in
