@@ -84,10 +84,10 @@ class virtual ['a, 'b] element_factory ~on_error =
             (fun () ->
               begin
                 match element with
-                | None -> ()
-                | Some el ->
-                    ignore (Element.set_state el.bin Element.State_null);
-                    ignore (Element.get_state el.bin)
+                  | None -> ()
+                  | Some el ->
+                      ignore (Element.set_state el.bin Element.State_null);
+                      ignore (Element.get_state el.bin)
               end;
               let el = self#make_element in
               element <- Some el;
@@ -145,7 +145,7 @@ class virtual ['a, 'b] element_factory ~on_error =
       if delay >= 0. then
         Duppy.Task.add Tutils.scheduler
           {
-            Duppy.Task.priority = Tutils.Non_blocking;
+            Duppy.Task.priority = `Non_blocking;
             events = [`Delay delay];
             handler =
               (fun _ ->
@@ -197,7 +197,7 @@ class output ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop
          GStreamer is waiting for some data before answering that we are
          playing. *)
       (* ignore (Element.get_state el.bin); *)
-      self#register_task ~priority:Tutils.Blocking Tutils.scheduler
+      self#register_task ~priority:`Blocking Tutils.scheduler
 
     method stop =
       self#stop_task;
@@ -274,9 +274,9 @@ class output ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop
             Gstreamer.App_src.push_buffer_bytes ~duration ~presentation_time
               (Option.get el.audio) data 0 (Bytes.length data));
           if has_video then (
-            let buf = VFrame.yuva420p frame in
-            for i = 0 to Video.length buf - 1 do
-              let img = Video.get buf i in
+            let buf = VFrame.data frame in
+            for i = 0 to Video.Canvas.length buf - 1 do
+              let img = Video.Canvas.render buf i in
               let y, u, v = Image.YUV420.data img in
               let buf =
                 Gstreamer.Buffer.of_data_list
@@ -351,7 +351,7 @@ let () =
         fun () -> ignore (Lang.apply f [])
       in
       let source = List.assoc "" p in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new output
          ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop source start
          ("", Some pipeline, None)
@@ -382,7 +382,7 @@ let () =
         fun () -> ignore (Lang.apply f [])
       in
       let source = List.assoc "" p in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new output
          ~kind ~clock_safe ~infallible ~on_error ~on_start ~on_stop source start
          ("", None, Some pipeline)
@@ -433,7 +433,7 @@ let () =
         fun () -> ignore (Lang.apply f [])
       in
       let source = List.assoc "" p in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new output
          ~kind ~clock_safe ~infallible ~on_error ~on_start ~on_stop ~blocking
          source start
@@ -466,19 +466,18 @@ class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
       | None, None ->
           failwith "There should be at least one audio or video pipeline!"
   in
-  let width = Lazy.force Frame.video_width in
-  let height = Lazy.force Frame.video_height in
   let rlog = ref (fun _ -> ()) in
   let gen = Generator.create ~log_overfull ~log:(fun x -> !rlog x) content in
   object (self)
     inherit Source.source ~name:"input.gstreamer.audio_video" kind as super
+    inherit Source.no_seek
     inherit [string sink, Gstreamer.data sink] element_factory ~on_error
     initializer rlog := fun s -> self#log#important "%s" s
     method set_state s = ignore (Element.set_state self#get_element.bin s)
     method pause_cmd = self#set_state Element.State_paused
     method play_cmd = self#set_state Element.State_playing
     method restart_cmd = self#restart
-    method stype = Source.Fallible
+    method stype = `Fallible
     method remaining = -1
 
     (* Source is ready when ready = true and gst has some audio or some video. *)
@@ -505,7 +504,7 @@ class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
     method wake_up activations =
       super#wake_up activations;
       try
-        self#register_task ~priority:Tutils.Blocking Tutils.scheduler;
+        self#register_task ~priority:`Blocking Tutils.scheduler;
         ignore (Element.set_state self#get_element.bin Element.State_playing);
         ignore (Element.get_state self#get_element.bin)
       with exn ->
@@ -587,22 +586,23 @@ class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
         let buf = Audio.create self#audio_channels len in
         Audio.S16LE.to_audio b 0 buf 0 len;
         Generator.put_audio gen
-          (Frame_content.Audio.lift_data buf)
+          (Content.Audio.lift_data buf)
           0 (Frame.main_of_audio len)
       done
 
     method private fill_video video =
+      let width, height = self#video_dimensions in
       while video.pending () > 0 && not self#is_generator_at_max do
         let b = video.pull () in
         let img =
           Image.YUV420.make_data width height b (Image.Data.round 4 width)
             (Image.Data.round 4 (width / 2))
         in
-        let stream = Video.single img in
+        let stream = Video.Canvas.single_image img in
         Generator.put_video gen
-          (Frame_content.Video.lift_data stream)
+          (Content.Video.lift_data stream)
           0
-          (Frame.main_of_video (Video.length stream))
+          (Frame.main_of_video (Video.Canvas.length stream))
       done
 
     method get_frame frame =
@@ -715,7 +715,7 @@ let () =
       let video_pipeline =
         Lang.to_string_getter (List.assoc "video_pipeline" p)
       in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new audio_video_input
         p kind
         (pipeline, Some audio_pipeline, Some video_pipeline))
@@ -735,7 +735,7 @@ let () =
   Lang.add_operator "input.gstreamer.audio" proto ~return_t ~category:`Input
     ~flags:[] ~descr:"Stream audio from a GStreamer pipeline." (fun p ->
       let pipeline = Lang.to_string_getter (List.assoc "pipeline" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new audio_video_input p kind ((fun () -> ""), Some pipeline, None)
         :> Source.source))
 
@@ -754,6 +754,6 @@ let () =
   Lang.add_operator "input.gstreamer.video" proto ~return_t ~category:`Input
     ~flags:[] ~descr:"Stream video from a GStreamer pipeline." (fun p ->
       let pipeline = Lang.to_string_getter (List.assoc "pipeline" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new audio_video_input p kind ((fun () -> ""), None, Some pipeline)
         :> Source.source))
