@@ -89,7 +89,7 @@ let common_options ~mode =
       Some (Lang.int 100),
       Some "Interval used to collect statistics" );
     ( "on_connect",
-      Lang.fun_t [(false, "", Lang.unit_t)] Lang.unit_t,
+      Lang.fun_t [] Lang.unit_t,
       Some (Lang.val_cst_fun [] Lang.unit),
       Some "Function to execute when connected." );
     ( "on_disconnect",
@@ -545,7 +545,7 @@ module Poll = struct
                (Printexc.to_string exn));
           -1.
 
-  let task = Duppy.Async.add ~priority:Tutils.Blocking Tutils.scheduler process
+  let task = Duppy.Async.add ~priority:`Blocking Tutils.scheduler process
 
   let add_socket ~mode socket fn =
     Srt.setsockflag socket Srt.sndsyn false;
@@ -632,7 +632,7 @@ class virtual networking_agent ~on_connect ~on_disconnect ~stats_interval =
        fun () ->
          if stats_interval > 0. then (
            let t =
-             Duppy.Async.add ~priority:Tutils.Non_blocking Tutils.scheduler
+             Duppy.Async.add ~priority:`Non_blocking Tutils.scheduler
                self#collect_stats
            in
            stats_task <- Some t;
@@ -695,7 +695,6 @@ class virtual caller ~payload_size ~messageapi ~hostname ~port
             -1.
           with exn ->
             self#log#important "Connect failed: %s" (Printexc.to_string exn);
-            !on_disconnect ();
             if not task_should_stop then 0. else -1.)
         ()
 
@@ -707,7 +706,7 @@ class virtual caller ~payload_size ~messageapi ~hostname ~port
             | Some t -> Duppy.Async.wake_up t
             | None ->
                 let t =
-                  Duppy.Async.add ~priority:Tutils.Blocking Tutils.scheduler
+                  Duppy.Async.add ~priority:`Blocking Tutils.scheduler
                     self#connect_fn
                 in
                 connect_task <- Some t;
@@ -830,7 +829,7 @@ class virtual input_base ~kind ~max ~log_overfull ~clock_safe ~on_connect
 
     inherit
       Start_stop.active_source
-        ~name:"input.srt" ~content_kind:(Source.Kind.of_kind kind) ~clock_safe
+        ~name:"input.srt" ~content_kind:(Kind.of_kind kind) ~clock_safe
           ~on_start ~on_stop ~autostart ~fallible:true () as super
 
     val mutable decoder_data = None
@@ -1140,9 +1139,9 @@ class output_caller ~kind ~payload_size ~messageapi ~on_start ~on_stop
   object
     inherit
       output_base
-        ~kind:(Source.Kind.of_kind kind) ~payload_size ~messageapi ~on_start
-          ~on_stop ~infallible ~autostart ~stats_interval ~on_connect
-          ~on_disconnect ~encoder_factory source
+        ~kind:(Kind.of_kind kind) ~payload_size ~messageapi ~on_start ~on_stop
+          ~infallible ~autostart ~stats_interval ~on_connect ~on_disconnect
+          ~encoder_factory source
 
     inherit
       caller
@@ -1157,9 +1156,9 @@ class output_listener ~kind ~payload_size ~messageapi ~on_start ~on_stop
   object
     inherit
       output_base
-        ~kind:(Source.Kind.of_kind kind) ~payload_size ~messageapi ~on_start
-          ~on_stop ~infallible ~autostart ~stats_interval ~on_connect
-          ~on_disconnect ~encoder_factory source
+        ~kind:(Kind.of_kind kind) ~payload_size ~messageapi ~on_start ~on_stop
+          ~infallible ~autostart ~stats_interval ~on_connect ~on_disconnect
+          ~encoder_factory source
 
     inherit
       listener
