@@ -192,8 +192,8 @@ class input ~kind ~hostname ~port ~get_stream_decoder ~bufferize ~log_overfull =
         Unix.close socket;
         begin
           match e with
-          | Failure s -> self#log#severe "Feeding stopped: %s." s
-          | e -> self#log#severe "Feeding stopped: %s." (Printexc.to_string e)
+            | Failure s -> self#log#severe "Feeding stopped: %s." s
+            | e -> self#log#severe "Feeding stopped: %s." (Printexc.to_string e)
         end;
         if should_stop () then has_stopped ()
         else self#feed (should_stop, has_stopped)
@@ -238,7 +238,7 @@ let () =
                (fmt, "Cannot get a stream encoder for that format"))
       in
       let source = Lang.assoc "" 2 p in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new output
          ~kind ~on_start ~on_stop ~infallible ~autostart ~hostname ~port
          ~encoder_factory:fmt source
@@ -281,7 +281,7 @@ let () =
                      "Cannot get a stream decoder for this MIME" ))
           | Some decoder_factory -> decoder_factory
       in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new input
          ~kind ~hostname ~port ~bufferize ~log_overfull ~get_stream_decoder
         :> Source.source))
