@@ -29,9 +29,6 @@ type t = Type.t
 
 type scheme = Type.scheme
 
-(** Position in source code. *)
-type pos = Type.pos
-
 (** {2 Values} *)
 
 (** A typed value. *)
@@ -41,7 +38,7 @@ module Ground : sig
 
   type content = Term.Ground.content = {
     descr : t -> string;
-    to_json : compact:bool -> json5:bool -> t -> string;
+    to_json : pos:Pos.t list -> t -> Json.t;
     compare : t -> t -> int;
     typ : Type.ground;
   }
@@ -50,13 +47,11 @@ module Ground : sig
   val to_string : t -> string
 end
 
-type value = Term.Value.t = { pos : pos option; value : in_value }
-
+type value = Value.t = { pos : Pos.Option.t; value : in_value }
 and env = (string * value) list
-
 and lazy_env = (string * value Lazy.t) list
 
-and in_value = Term.Value.in_value =
+and in_value = Value.in_value =
   | Ground of Ground.t
   | Source of Source.source
   | Encoder of Encoder.format
@@ -64,21 +59,14 @@ and in_value = Term.Value.in_value =
   | Tuple of value list
   | Null
   | Meth of string * value * value
-  | Ref of value ref
-  | Fun of (string * string * value option) list * env * lazy_env * Term.t
-  (* A function with given arguments (argument label, argument variable,
-     default value), parameters already passed to the function, closure and
-     value. *)
-  | FFI of (string * string * value option) list * env * (env -> value)
+  | Ref of value Atomic.t
+  | Fun of (string * string * value option) list * lazy_env * Term.t
+  (* A function with given arguments (argument label, argument variable, default
+     value), closure and value. *)
+  | FFI of (string * string * value option) list * (env -> value)
 
 val demeth : value -> value
 
-(** Get a string representation of a value. *)
-val print_value : value -> string
-
-(** Compare values. *)
-val compare_values : value -> value -> int
-
 (** Iter a function over all sources contained in a value. This only applies to
     statically referenced objects, i.e. it does not explore inside reference
     cells. [on_reference] is used when we encounter a reference cell that may
@@ -88,7 +76,7 @@ val iter_sources :
 
 (** {2 Computation} *)
 
-val apply_fun : (value -> env -> value) ref
+val apply_fun : (?pos:Pos.t -> value -> env -> value) ref
 
 (** Multiapply a value to arguments. The argument [t] is the type of the result
    of the application. *)
@@ -114,6 +102,7 @@ val add_builtin :
   descr:string ->
   ?flags:Documentation.flag list ->
   ?meth:(string * Type.scheme * string * value) list ->
+  ?examples:string list ->
   string ->
   proto ->
   t ->
@@ -141,7 +130,7 @@ val internal : Frame.content_kind
 
 (* Audio (PCM format) *)
 val audio_pcm : Frame.content_kind
-val audio_params : Frame_content.Audio.params -> Frame.content_kind
+val audio_params : Content.Audio.params -> Frame.content_kind
 val audio_n : int -> Frame.content_kind
 val audio_mono : Frame.content_kind
 val audio_stereo : Frame.content_kind
@@ -179,9 +168,9 @@ val to_string : value -> string
 val to_string_getter : value -> unit -> string
 val to_float : value -> float
 val to_float_getter : value -> unit -> float
+val to_error : value -> Runtime_error.runtime_error
 val to_source : value -> Source.source
 val to_format : value -> Encoder.format
-val to_request : value -> Request.t
 val to_int : value -> int
 val to_int_getter : value -> unit -> int
 val to_num : value -> [ `Int of int | `Float of float ]
@@ -191,7 +180,7 @@ val to_valued_option : (value -> 'a) -> value -> 'a option
 val to_default_option : default:'a -> (value -> 'a) -> value -> 'a
 val to_product : value -> value * value
 val to_tuple : value -> value list
-val to_ref : value -> value ref
+val to_ref : value -> value Atomic.t
 val to_metadata_list : value -> (string * string) list
 val to_metadata : value -> Frame.metadata
 val to_string_list : value -> string list
@@ -219,7 +208,7 @@ val list_t : t -> t
 val of_list_t : t -> t
 val nullable_t : t -> t
 val ref_t : t -> t
-val request_t : t
+val error_t : t
 val source_t : ?methods:bool -> t -> t
 val of_source_t : t -> t
 val format_t : t -> t
@@ -250,13 +239,13 @@ val float : float -> value
 val string : string -> value
 val list : value list -> value
 val null : value
+val error : Runtime_error.runtime_error -> value
 val source : Source.source -> value
-val request : Request.t -> value
 val product : value -> value -> value
 val tuple : value list -> value
 val meth : value -> (string * value) list -> value
 val record : (string * value) list -> value
-val reference : value ref -> value
+val reference : value Atomic.t -> value
 
 (** Build a function from an OCaml function. Items in the prototype indicate
     the label and optional values. *)
@@ -267,34 +256,23 @@ val val_fun : (string * string * value option) list -> (env -> value) -> value
   * when the constant is ground. *)
 val val_cst_fun : (string * value option) list -> value -> value
 
+(** Extract position from the environment. Used inside function execution. *)
+val pos : env -> Pos.t list
+
 (** Convert a metadata packet to a list associating strings to strings. *)
 val metadata : Frame.metadata -> value
 
 (** Raise an error. *)
-val error :
-  ?bt:Printexc.raw_backtrace -> ?pos:pos list -> ?message:string -> string -> 'a
+val raise_error :
+  ?bt:Printexc.raw_backtrace ->
+  ?message:string ->
+  pos:Pos.List.t ->
+  string ->
+  'a
+
+(** Convert an exception into a runtime error. *)
+val runtime_error_of_exception :
+  bt:Printexc.raw_backtrace -> kind:string -> exn -> Runtime_error.runtime_error
 
 (** Re-raise an error as a runtime error. *)
 val raise_as_runtime : bt:Printexc.raw_backtrace -> kind:string -> exn -> 'a
-
-(* Abstract type and value. *)
-module type Abstract = sig
-  type content
-
-  val t : t
-  val to_value : content -> value
-  val of_value : value -> content
-  val is_value : value -> bool
-end
-
-module type AbstractDef = sig
-  type content
-
-  val name : string
-  val to_json : compact:bool -> json5:bool -> content -> string
-  val descr : content -> string
-  val compare : content -> content -> int
-end
-
-module MkAbstract (Def : AbstractDef) :
-  Abstract with type content := Def.content
