@@ -96,8 +96,8 @@ let var_lit =
     ( var_underscore
     | Star '_', var_char, Star (var_char | decimal_digit | '_' | '\'') )]
 
-let var_ref = [%sedlex.regexp? "ref", Plus (var_char | decimal_digit | '\'')]
 let var = [%sedlex.regexp? var_lit | so | math | other_math]
+let encoder = [%sedlex.regexp? '%', Plus (var_char | decimal_digit | '.')]
 
 let time =
   [%sedlex.regexp?
@@ -183,14 +183,22 @@ let rec token lexbuf =
     | "%argsof" -> ARGS_OF
     | '#', Star (Compl '\n'), eof -> EOF
     | eof -> EOF
-    | "def" -> PP_DEF
-    | "replaces" -> REPLACES
+    | "def", Plus skipped, "rec", Plus skipped -> PP_DEF `Recursive
+    | "def", Plus skipped, "replaces", Plus skipped -> PP_DEF `Replaces
+    | "def" -> PP_DEF `None
     | "try" -> TRY
     | "catch" -> CATCH
     | "do" -> DO
-    | "let" -> LET
+    | "let", Plus skipped, "replaces", Plus skipped -> LET `Replaces
+    | "let", Plus skipped, "eval", Plus skipped -> LET `Eval
+    | "let", Plus skipped, "json.parse", Star skipped, '[' ->
+        LETLBRA `Json_parse
+    | "let", Plus skipped, "json.parse", Plus skipped -> LET `Json_parse
+    | "let", Plus skipped, "json.stringify", Star skipped, '[' ->
+        LETLBRA `Json_stringify
+    | "let", Plus skipped, "json.stringify", Plus skipped -> LET `Json_stringify
+    | "let" -> LET `None
     | "fun" -> FUN
-    | "rec" -> REC
     | '=' -> GETS
     | "end" -> END
     | "begin" -> BEGIN
@@ -199,34 +207,20 @@ let rec token lexbuf =
     | "else" -> ELSE
     | "elsif" -> ELSIF
     | "->" -> YIELDS
-    | "%ogg" -> OGG
-    | "%vorbis" -> VORBIS
-    | "%opus" -> OPUS
-    | "%flac" -> FLAC
-    | "%audio" -> AUDIO
-    | "%audio.raw" -> AUDIO_RAW
-    | "%audio.copy" -> AUDIO_COPY
-    | "%audio.none" -> AUDIO_NONE
-    | "%video" -> VIDEO
-    | "%video.raw" -> VIDEO_RAW
-    | "%video.copy" -> VIDEO_COPY
-    | "%video.none" -> VIDEO_NONE
-    | "%ffmpeg" -> FFMPEG
-    | "%vorbis.cbr" -> VORBIS_CBR
-    | "%vorbis.abr" -> VORBIS_ABR
-    | "%theora" -> THEORA
-    | "%external" -> EXTERNAL
-    | "%gstreamer" -> GSTREAMER
-    | "%speex" -> SPEEX
-    | "%wav" -> WAV
-    | "%avi" -> AVI
-    | "%mp3" -> MP3
-    | "%mp3.cbr" -> MP3
-    | "%mp3.abr" -> MP3_ABR
-    | "%mp3.vbr" -> MP3_VBR
-    | "%mp3.fxp" -> SHINE
-    | "%shine" -> SHINE
-    | "%fdkaac" -> FDKAAC
+    | encoder ->
+        let e = Sedlexing.Utf8.lexeme lexbuf in
+        let e = String.sub e 1 (String.length e - 1) in
+        ENCODER e
+    | '.', Star skipped, var ->
+        let m = Sedlexing.Utf8.lexeme lexbuf in
+        let lexbuf = Sedlexing.Utf8.from_string m in
+        let rec f () =
+          match%sedlex lexbuf with
+            | '.', Star skipped -> f ()
+            | var -> DOTVAR (Sedlexing.Utf8.lexeme lexbuf)
+            | _ -> assert false
+        in
+        f ()
     | '.' -> DOT
     | "..." -> DOTDOTDOT
     | '[' -> LBRA
@@ -236,6 +230,7 @@ let rec token lexbuf =
     | '{' -> LCUR
     | '}' -> RCUR
     | ',' -> COMMA
+    | "::" -> COLONCOLON
     | ':' -> COLON
     | ';' -> SEQ
     | ";;" -> SEQSEQ
@@ -256,7 +251,6 @@ let rec token lexbuf =
     | "mod" -> BIN3 (Sedlexing.Utf8.lexeme lexbuf)
     | "??" -> COALESCE
     | "*" -> TIMES
-    | var_ref -> VAR (Sedlexing.Utf8.lexeme lexbuf)
     | "!" -> GET
     | ":=" -> SET
     | '_' -> UNDERSCORE
@@ -303,11 +297,27 @@ let rec token lexbuf =
         let s = read_string '\'' startp (Buffer.create 17) lexbuf in
         let _, endp = Sedlexing.lexing_positions lexbuf in
         PP_STRING (s, (startp, endp))
-    | _ ->
+    | "r/" ->
+        let startp, _ = Sedlexing.lexing_positions lexbuf in
+        let regexp = read_string '/' startp (Buffer.create 17) lexbuf in
+        let flags = read_regexp_flags [] lexbuf in
+        let _, endp = Sedlexing.lexing_positions lexbuf in
+        PP_REGEXP (regexp, flags, (startp, endp))
+    | any ->
         raise
           (Term.Parse_error
              ( Sedlexing.lexing_positions lexbuf,
                "Parse error: " ^ Sedlexing.Utf8.lexeme lexbuf ))
+    | _ -> failwith "Internal error"
+
+and read_regexp_flags flags lexbuf =
+  match%sedlex lexbuf with
+    | 'g' | 'i' | 's' | 'm' | 'u' ->
+        let matched = Sedlexing.Utf8.lexeme lexbuf in
+        read_regexp_flags (matched.[0] :: flags) lexbuf
+    | _ ->
+        Sedlexing.rollback lexbuf;
+        flags
 
 and read_string c pos buf lexbuf =
   (* See: https://en.wikipedia.org/wiki/Escape_sequences_in_C *)
@@ -341,8 +351,15 @@ and read_string c pos buf lexbuf =
         Buffer.add_char buf matched.[1];
         read_string c pos buf lexbuf
     | '\\', '?' ->
-        Buffer.add_char buf '\x3f';
-        read_string c pos buf lexbuf
+        (* For regexp, we want to make sure \? is kept as-is
+           and does not need any further escaping. *)
+        if c = '/' then (
+          Buffer.add_char buf '\\';
+          Buffer.add_char buf '?';
+          read_string c pos buf lexbuf)
+        else (
+          Buffer.add_char buf '\x3f';
+          read_string c pos buf lexbuf)
     | '\\', 'x', ascii_hex_digit, ascii_hex_digit ->
         let matched = Sedlexing.Utf8.lexeme lexbuf in
         let idx = String.index matched 'x' in
@@ -370,13 +387,18 @@ and read_string c pos buf lexbuf =
        Some more text *)
     | '\\', '\n', Star skipped -> read_string c pos buf lexbuf
     | '\\', any ->
-        Printf.printf "Warning: illegal backslash escape in string.\n";
+        if c <> '/' then (
+          let pos =
+            Pos.to_string (pos, snd (Sedlexing.lexing_positions lexbuf))
+          in
+          Printf.printf
+            "Warning at position %s: illegal backslash escape in string.\n" pos);
         Buffer.add_string buf (Sedlexing.Utf8.lexeme lexbuf);
         read_string c pos buf lexbuf
-    | Plus (Compl ('"' | '\'' | '\\')) ->
+    | Plus (Compl ('"' | '\'' | '\\' | '/')) ->
         Buffer.add_string buf (Sedlexing.Utf8.lexeme lexbuf);
         read_string c pos buf lexbuf
-    | '"' | '\'' ->
+    | '"' | '\'' | '/' ->
         let matched = Sedlexing.Utf8.lexeme lexbuf in
         let c' = matched.[0] in
         if c = c' then Buffer.contents buf
@@ -384,12 +406,18 @@ and read_string c pos buf lexbuf =
           Buffer.add_char buf c';
           read_string c pos buf lexbuf)
     | eof ->
+        let msg =
+          if c = '/' then "Regexp not terminated"
+          else "String is not terminated"
+        in
         raise
-          (Term.Parse_error
-             ( (pos, snd (Sedlexing.lexing_positions lexbuf)),
-               "String is not terminated" ))
+          (Term.Parse_error ((pos, snd (Sedlexing.lexing_positions lexbuf)), msg))
     | _ ->
+        let msg =
+          if c = '/' then "Illegal regexp character: "
+          else "Illegal string character: "
+        in
         raise
           (Term.Parse_error
              ( (pos, snd (Sedlexing.lexing_positions lexbuf)),
-               "Illegal string character: " ^ Sedlexing.Utf8.lexeme lexbuf ))
+               msg ^ Sedlexing.Utf8.lexeme lexbuf ))
