@@ -4,14 +4,6 @@
 type token = 
   | YIELDS
   | WHILE
-  | WAV
-  | VORBIS_CBR
-  | VORBIS_ABR
-  | VORBIS
-  | VIDEO_RAW
-  | VIDEO_NONE
-  | VIDEO_COPY
-  | VIDEO
   | VARLPAR of (string)
   | VARLBRA of (string)
   | VAR of (string)
@@ -22,21 +14,20 @@ type token =
   | TIMES
   | TIME of (int option list)
   | TILD
-  | THEORA
   | THEN
-  | STRING of (string)
-  | SPEEX
-  | SHINE
+  | STRING of (string )
+  | SLASH
   | SET
   | SEQSEQ
   | SEQ
   | RPAR
   | REPLACES
-  | REC
+  | REGEXP of (string * char list )
   | RCUR
   | RBRA
   | QUESTION
-  | PP_STRING of (string * (Lexing.position * Lexing.position))
+  | PP_STRING of (string * Pos.t)
+  | PP_REGEXP of (string * char list * Pos.t)
   | PP_INT_DOT_LCUR of (int)
   | PP_INCLUDE of (string)
   | PP_IFVERSION of ([ `Eq | `Geq | `Leq | `Gt | `Lt] * string)
@@ -48,42 +39,36 @@ type token =
   | PP_ENDIF
   | PP_ELSE
   | PP_DEFINE
-  | PP_DEF
+  | PP_DEF of (Parser_helper.lexer_let_decoration)
   | PP_COMMENT of (string list)
-  | OPUS
   | OPEN
-  | OGG
   | NOT
-  | MP3_VBR
-  | MP3_ABR
-  | MP3
   | MINUS
   | LPAR
-  | LET
+  | LETLBRA of (Parser_helper.lexer_let_decoration)
+  | LET of (Parser_helper.lexer_let_decoration)
   | LCUR
   | LBRA
   | INTERVAL of (int option list * int option list)
   | INT of (int)
   | IF
-  | GSTREAMER
   | GETS
   | GET
   | FUN
   | FOR
   | FLOAT of (float)
-  | FLAC
-  | FFMPEG
-  | FDKAAC
-  | EXTERNAL
   | EOF
   | END
+  | ENCODER of (string)
   | ELSIF
   | ELSE
+  | DOTVAR of (string)
   | DOTDOTDOT
   | DOT
   | DO
-  | DEF of (Doc.item * (string*string) list * (string*string) list)
+  | DEF of ((Doc.item * (string*string) list * (string*string) list)*Parser_helper.let_decoration)
   | COMMA
+  | COLONCOLON
   | COLON
   | COALESCE
   | CATCH
@@ -93,11 +78,6 @@ type token =
   | BIN2 of (string)
   | BIN1 of (string)
   | BEGIN
-  | AVI
-  | AUDIO_RAW
-  | AUDIO_NONE
-  | AUDIO_COPY
-  | AUDIO
   | ARGS_OF
 
 (* This exception is raised by the monolithic API functions. *)
