@@ -20,7 +20,7 @@
 
  *****************************************************************************)
 
-type tokenizer = unit -> Parser.token * (Lexing.position * Lexing.position)
+type tokenizer = unit -> Parser.token * Pos.t
 
 let fst3 (x, _, _) = x
 let snd3 (_, y, _) = y
@@ -31,56 +31,9 @@ let mk_tokenizer ?(fname = "") lexbuf =
   fun () ->
     match Lexer.token lexbuf with
       | Parser.PP_STRING (s, pos) -> (Parser.STRING s, pos)
+      | Parser.PP_REGEXP (r, flags, pos) -> (Parser.REGEXP (r, flags), pos)
       | token -> (token, Sedlexing.lexing_positions lexbuf)
 
-(* TODO: also parse optional arguments? *)
-let get_encoder_format tokenizer =
-  let ogg_audio_item = function
-    | Parser.VORBIS -> Lang_vorbis.make []
-    | Parser.VORBIS_CBR -> Lang_vorbis.make_cbr []
-    | Parser.VORBIS_ABR -> Lang_vorbis.make_abr []
-    | Parser.SPEEX -> Lang_speex.make []
-    | Parser.OPUS -> Lang_opus.make []
-    | Parser.FLAC -> Lang_flac.make_ogg []
-    | _ -> failwith "ogg format expected"
-  in
-  let is_ogg_audio_item token =
-    try
-      let _ = ogg_audio_item token in
-      true
-    with _ -> false
-  in
-  let ogg_video_item = function
-    | Parser.THEORA -> Lang_theora.make []
-    | _ -> failwith "ogg format expected"
-  in
-  let is_ogg_video_item token =
-    try
-      let _ = ogg_video_item token in
-      true
-    with _ -> false
-  in
-  let token = fst (tokenizer ()) in
-  match token with
-    | Parser.MP3 -> Lang_mp3.make_cbr []
-    | Parser.MP3_VBR -> Lang_mp3.make_vbr []
-    | Parser.MP3_ABR -> Lang_mp3.make_vbr []
-    | Parser.SHINE -> Lang_shine.make []
-    | Parser.FDKAAC -> Lang_fdkaac.make []
-    | Parser.FLAC -> Lang_flac.make []
-    | Parser.EXTERNAL -> Lang_external_encoder.make []
-    | Parser.GSTREAMER -> Lang_gstreamer.make []
-    | Parser.WAV -> Lang_wav.make []
-    | ogg when is_ogg_audio_item ogg ->
-        let audio = ogg_audio_item ogg in
-        Encoder.Ogg { Ogg_format.audio = Some audio; video = None }
-    | ogg when is_ogg_video_item ogg ->
-        let video = ogg_video_item ogg in
-        Encoder.Ogg { Ogg_format.audio = None; video = Some video }
-    (* TODO *)
-    (* | Parser.OGG -> Lang_encoders.mk ... [] *)
-    | _ -> failwith "expected an encoding format after %ifencoder"
-
 (* The Lang_lexer is not quite enough for our needs, so we first define
    convenient layers between it and the parser. First a pre-processor which
    evaluates %ifdefs. *)
@@ -121,12 +74,19 @@ let eval_ifdefs tokenizer =
           in
           if test then go_on () else skip ()
       | (Parser.PP_IFENCODER, _ | Parser.PP_IFNENCODER, _) as tok ->
-          let fmt = get_encoder_format tokenizer in
           let has_enc =
             try
+              let fmt =
+                let token = fst (tokenizer ()) in
+                match token with
+                  | Parser.ENCODER e ->
+                      Lang_encoder.make_encoder ~pos:None (Term.make Term.unit)
+                        (e, [])
+                  | _ -> failwith "expected an encoding format after %ifencoder"
+              in
               let (_ : Encoder.factory) = Encoder.get_factory fmt in
               true
-            with Not_found -> false
+            with _ -> false
           in
           let test =
             if fst tok = Parser.PP_IFENCODER then fun x -> x else not
@@ -263,7 +223,7 @@ type doc_type = [ `Full | `Argsof of string list ]
 (* Glue the documenting comments to the corresponding PP_DEF (read pre-DEF) *
    and strip out other comments. *)
 let parse_comments tokenizer =
-  let documented_def (doc, doc_startp) (startp, endp) =
+  let documented_def decoration (doc, doc_startp) (startp, endp) =
     let startp =
       match doc_startp with Some startp -> startp | None -> startp
     in
@@ -412,10 +372,11 @@ let parse_comments tokenizer =
     in
     List.iter
       (function
-        | `Category c -> doc#add_subsection "_category" (Doc.trivial c)
-        | `Flag c -> doc#add_subsection "_flag" (Doc.trivial c))
+        | `Category c ->
+            doc#add_subsection "_category" (Lazy.from_val (Doc.trivial c))
+        | `Flag c -> doc#add_subsection "_flag" (Lazy.from_val (Doc.trivial c)))
       special;
-    (Parser.DEF (doc, params, methods), (startp, endp))
+    (Parser.DEF ((doc, params, methods), decoration), (startp, endp))
   in
   let comment = ref ([], None) in
   let rec token () =
@@ -423,10 +384,13 @@ let parse_comments tokenizer =
       | Parser.PP_COMMENT c, (startp, _) ->
           comment := (c, Some startp);
           token ()
-      | Parser.PP_DEF, pos ->
+      | Parser.PP_DEF decoration, pos ->
+          let decoration =
+            Parser_helper.let_decoration_of_lexer_let_decoration decoration
+          in
           let c = !comment in
           comment := ([], None);
-          documented_def c pos
+          documented_def decoration c pos
       | x ->
           comment := ([], None);
           x
@@ -458,6 +422,23 @@ let int_meth tokenizer =
   in
   token
 
+(* Replace DOTVAR v with DOT, VAR v *)
+let dotvar tokenizer =
+  let state = ref None in
+  let token () =
+    match !state with
+      | Some t ->
+          state := None;
+          t
+      | None -> (
+          match tokenizer () with
+            | Parser.DOTVAR v, pos ->
+                state := Some (Parser.VAR v, pos);
+                (Parser.DOT, pos)
+            | t -> t)
+  in
+  token
+
 (** Change MINUS to UMINUS if the minus is not preceded by a number (or an
    expression which could produce a number). *)
 let uminus tokenizer =
@@ -546,7 +527,8 @@ let expand_define tokenizer =
 let mk_tokenizer ?fname ~pwd lexbuf =
   let tokenizer =
     mk_tokenizer ?fname lexbuf |> includer pwd |> eval_ifdefs |> parse_comments
-    |> expand_string |> int_meth |> uminus |> strip_newlines |> expand_define
+    |> expand_string |> int_meth |> dotvar |> uminus |> strip_newlines
+    |> expand_define
   in
   fun () ->
     let t, (startp, endp) = tokenizer () in
