@@ -22,35 +22,29 @@
 
 (** Terms and values in the Liquidsoap language. *)
 
-(** An error at runtime. *)
-include Runtime_error
-
 (** An internal error. Those should not happen in theory... *)
-exception Internal_error of (pos list * string)
+exception Internal_error of (Pos.t list * string)
 
 (** A parsing error. *)
-exception Parse_error of (pos * string)
+exception Parse_error of (Pos.t * string)
 
 (** Unsupported format *)
-exception Unsupported_format of (pos * Encoder.format)
+exception Unsupported_format of (Pos.t option * string)
 
 let () =
   Printexc.register_printer (function
     | Internal_error (pos, e) ->
         Some
           (Printf.sprintf "Lang_values.Internal_error at %s: %s"
-             (Runtime_error.print_pos_list pos)
-             e)
+             (Pos.List.to_string pos) e)
     | Parse_error (pos, e) ->
         Some
           (Printf.sprintf "Lang_values.Parse_error at %s: %s"
-             (Runtime_error.print_pos pos)
-             e)
-    | Unsupported_format (pos, f) ->
+             (Pos.to_string pos) e)
+    | Unsupported_format (pos, e) ->
         Some
           (Printf.sprintf "Lang_values.Unsupported_format at %s: %s"
-             (Runtime_error.print_pos pos)
-             (Encoder.string_of_format f))
+             (Pos.Option.to_string pos) e)
     | _ -> None)
 
 let conf =
@@ -81,17 +75,17 @@ let profile = ref false
 (* In a sense this could be moved to Type, but I like to keep that part free of
    some specificities of liquidsoap, as much as possible. *)
 
-let ref_t ?pos ?level t =
-  Type.make ?pos ?level
-    (Type.Constr { Type.name = "ref"; params = [(Type.Invariant, t)] })
+let ref_t ?pos t =
+  Type.make ?pos
+    (Type.Constr { Type.constructor = "ref"; params = [(Type.Invariant, t)] })
 
 (** A frame kind type is a purely abstract type representing a
     frame kind. *)
-let frame_kind_t ?pos ?level audio video midi =
-  Type.make ?pos ?level
+let frame_kind_t ?pos audio video midi =
+  Type.make ?pos
     (Type.Constr
        {
-         Type.name = "stream_kind";
+         Type.constructor = "stream_kind";
          Type.params =
            [
              (Type.Covariant, audio);
@@ -100,86 +94,74 @@ let frame_kind_t ?pos ?level audio video midi =
            ];
        })
 
-let kind_t ?pos ?level kind =
-  let evar ?(constraints = []) () =
-    Type.fresh ~constraints
-      ~pos:(match pos with None -> None | Some pos -> pos)
-      ~level:(-1)
-  in
-  let mk_format f = Type.make ?pos ?level (Type.Ground (Type.Format f)) in
+let kind_t ?pos kind =
+  let evar ?(constraints = []) () = Type.var ~constraints ?pos () in
+  let mk_format f = Type.make ?pos (Type.Ground (Type.Format f)) in
   match kind with
     | `Any -> evar ()
     | `Internal -> evar ~constraints:[Type.InternalMedia] ()
     | `Kind k ->
-        Type.make ?pos ?level
+        Type.make ?pos
           (Type.Constr
              {
-               Type.name = Frame_content.string_of_kind k;
+               Type.constructor = Content.string_of_kind k;
                Type.params = [(Type.Covariant, evar ())];
              })
     | `Format f ->
-        let k = Frame_content.kind f in
-        Type.make ?pos ?level
+        let k = Content.kind f in
+        Type.make ?pos
           (Type.Constr
              {
-               Type.name = Frame_content.string_of_kind k;
+               Type.constructor = Content.string_of_kind k;
                Type.params = [(Type.Covariant, mk_format f)];
              })
 
 let of_frame_kind_t t =
-  match (Type.deref t).Type.descr with
+  let t = Type.deref t in
+  match t.Type.descr with
     | Type.Constr
         {
-          Type.name = "stream_kind";
+          Type.constructor = "stream_kind";
           Type.params = [(_, audio); (_, video); (_, midi)];
         } ->
         { Frame.audio; video; midi }
-    | Type.EVar (_, _) ->
+    | Type.Var ({ contents = Type.Free _ } as var) ->
         let audio = kind_t `Any in
         let video = kind_t `Any in
         let midi = kind_t `Any in
-        Type.bind t (frame_kind_t audio video midi);
+        var := Type.Link (Type.Invariant, frame_kind_t audio video midi);
         { Frame.audio; video; midi }
     | _ -> assert false
 
 (** Type of audio formats that can encode frame of a given kind. *)
-let format_t ?pos ?level k =
-  Type.make ?pos ?level
-    (Type.Constr { Type.name = "format"; Type.params = [(Type.Covariant, k)] })
+let format_t ?pos k =
+  Type.make ?pos
+    (Type.Constr
+       { Type.constructor = "format"; Type.params = [(Type.Covariant, k)] })
 
 (** Type of sources carrying frames of a given kind. *)
-let source_t ?pos ?level k =
-  Type.make ?pos ?level
-    (Type.Constr { Type.name = "source"; Type.params = [(Type.Invariant, k)] })
+let source_t ?pos k =
+  Type.make ?pos
+    (Type.Constr
+       { Type.constructor = "source"; Type.params = [(Type.Invariant, k)] })
 
 (* Filled in later to avoid dependency cycles. *)
 let source_methods_t = ref (fun () : Type.t -> assert false)
 
 let of_source_t t =
   match (Type.deref t).Type.descr with
-    | Type.Constr { Type.name = "source"; Type.params = [(_, t)] } -> t
+    | Type.Constr { Type.constructor = "source"; Type.params = [(_, t)] } -> t
     | _ -> assert false
 
-let request_t ?pos ?level () = Type.make ?pos ?level (Type.Ground Type.Request)
-
-let type_of_format ~pos ~level f =
+let type_of_format ~pos f =
   let kind = Encoder.kind_of_format f in
-  let audio = kind_t ~pos ~level kind.Frame.audio in
-  let video = kind_t ~pos ~level kind.Frame.video in
-  let midi = kind_t ~pos ~level kind.Frame.midi in
-  format_t ~pos ~level (frame_kind_t ~pos ~level audio video midi)
+  let audio = kind_t ~pos kind.Frame.audio in
+  let video = kind_t ~pos kind.Frame.video in
+  let midi = kind_t ~pos kind.Frame.midi in
+  format_t ~pos (frame_kind_t ~pos audio video midi)
 
 (** {2 Terms} *)
 
-(** The way we implement this mini-language is not very efficient. It should not
-    matter, since very little computation is done here. It is mostly used for a
-    single run on startup to build the sources, and then sometimes for building
-    transitions. Terms are small, no recursion is possible. In order to report
-    informative errors, including runtime errors (invalid values of a valid type
-    given to a FF) we need to keep a complete AST all the way long.  We actually
-    don't need the types anymore after the static checking, but I don't want to
-    bother with stripping down to another datatype. *)
-
 (** Sets of variables. *)
 module Vars = Set.Make (String)
 
@@ -188,7 +170,7 @@ module Ground = struct
 
   type content = {
     descr : t -> string;
-    to_json : compact:bool -> json5:bool -> t -> string;
+    to_json : pos:Pos.t list -> t -> Json.t;
     compare : t -> t -> int;
     typ : Type.ground;
   }
@@ -207,28 +189,23 @@ module Ground = struct
     with Found c -> c
 
   let to_string (v : t) = (find v).descr v
-  let to_json ~compact ~json5 (v : t) = (find v).to_json ~compact ~json5 v
+  let to_json ~pos (v : t) = (find v).to_json ~pos v
   let to_type (v : t) = (find v).typ
   let compare (v : t) = (find v).compare v
 
-  type t +=
-    | Bool of bool
-    | Int of int
-    | String of string
-    | Float of float
-    | Request of Request.t
+  type t += Bool of bool | Int of int | String of string | Float of float
 
   let () =
     let compare conv v v' = Stdlib.compare (conv v) (conv v') in
     let to_bool = function Bool b -> b | _ -> assert false in
     let to_string b = string_of_bool (to_bool b) in
-    let to_json ~compact:_ ~json5:_ = to_string in
+    let to_json ~pos:_ b = `Bool (to_bool b) in
     register
       (function Bool _ -> true | _ -> false)
       { descr = to_string; to_json; compare = compare to_bool; typ = Type.Bool };
     let to_int = function Int i -> i | _ -> assert false in
     let to_string i = string_of_int (to_int i) in
-    let to_json ~compact:_ ~json5:_ = to_string in
+    let to_json ~pos:_ i = `Int (to_int i) in
     register
       (function Int _ -> true | _ -> false)
       { descr = to_string; to_json; compare = compare to_int; typ = Type.Int };
@@ -236,7 +213,7 @@ module Ground = struct
       | String s -> Utils.quote_string s
       | _ -> assert false
     in
-    let to_json ~compact:_ ~json5:_ = to_string in
+    let to_json ~pos:_ = function String s -> `String s | _ -> assert false in
     register
       (function String _ -> true | _ -> false)
       {
@@ -246,17 +223,7 @@ module Ground = struct
         typ = Type.String;
       };
     let to_float = function Float f -> f | _ -> assert false in
-    let to_json ~compact:_ ~json5 f =
-      let f = to_float f in
-      match classify_float f with
-        | FP_infinite when json5 -> if f < 0. then "-Infinity" else "Infinity"
-        | FP_nan when json5 -> if f < 0. then "-NaN" else "NaN"
-        | FP_infinite | FP_nan -> "null"
-        | _ ->
-            let s = string_of_float f in
-            let s = Printf.sprintf "%s" s in
-            if s.[String.length s - 1] = '.' then Printf.sprintf "%s0" s else s
-    in
+    let to_json ~pos:_ f = `Float (to_float f) in
     register
       (function Float _ -> true | _ -> false)
       {
@@ -264,27 +231,14 @@ module Ground = struct
         to_json;
         compare = compare to_float;
         typ = Type.Float;
-      };
-    let to_request = function Request r -> r | _ -> assert false in
-    let descr r =
-      Printf.sprintf "<request(id=%d)>" (Request.get_id (to_request r))
-    in
-    let to_json ~compact:_ ~json5:_ r = Printf.sprintf "%S" (descr r) in
-    let compare r r' =
-      Stdlib.compare
-        (Request.get_id (to_request r))
-        (Request.get_id (to_request r'))
-    in
-    register
-      (function Request _ -> true | _ -> false)
-      { descr; compare; to_json; typ = Type.Request }
+      }
 end
 
 module type GroundDef = sig
   type content
 
   val descr : content -> string
-  val to_json : compact:bool -> json5:bool -> content -> string
+  val to_json : pos:Pos.t list -> content -> Json.t
   val compare : content -> content -> int
   val typ : Type.ground
 end
@@ -294,7 +248,7 @@ module MkGround (D : GroundDef) = struct
 
   let () =
     let to_ground = function Ground g -> g | _ -> assert false in
-    let to_json ~compact ~json5 v = D.to_json ~compact ~json5 (to_ground v) in
+    let to_json ~pos v = D.to_json ~pos (to_ground v) in
     let compare v v' = D.compare (to_ground v) (to_ground v') in
     let descr v = D.descr (to_ground v) in
     Ground.register
@@ -304,20 +258,28 @@ end
 
 type t = { mutable t : Type.t; term : in_term }
 
+(** Documentation for declarations: general documentation, parameters, methods. *)
+and doc = Doc.item * (string * string) list * (string * string) list
+
 and let_t = {
-  doc : Doc.item * (string * string) list * (string * string) list;
+  doc : doc;
   (* name, arguments, methods *)
   replace : bool;
   (* whether the definition replaces a previously existing one (keeping methods) *)
   pat : pattern;
-  mutable gen : (int * Type.constraints) list;
+  mutable gen : Type.var list;
   def : t;
   body : t;
 }
 
+and encoder_params = (string * [ `Term of t | `Encoder of encoder ]) list
+
+(** A formal encoder. *)
+and encoder = string * encoder_params
+
 and in_term =
   | Ground of Ground.t
-  | Encoder of Encoder.format
+  | Encoder of encoder
   | List of t list
   | Tuple of t list
   | Null
@@ -336,13 +298,16 @@ and in_term =
    * restrict the environment captured when a closure is
    * formed. *)
   | Fun of Vars.t * (string * string * Type.t * t option) list * t
+  (* A recursive function, the first string is the name of the recursive
+     variable. *)
   | RFun of string * Vars.t * (string * string * Type.t * t option) list * t
 
-(* A recursive function, the first string is the name of the recursive
-   variable. *)
 and pattern =
   | PVar of string list  (** a field *)
   | PTuple of pattern list  (** a tuple *)
+  | PList of (pattern list * string option * pattern list)  (** a list *)
+  | PMeth of (pattern option * (string * pattern option) list)
+      (** a value with methods *)
 
 type term = t
 
@@ -350,59 +315,139 @@ let unit = Tuple []
 
 (* Only used for printing very simple functions. *)
 let is_ground x =
-  match x.term with
-    | Ground _ | Encoder _ -> true
-    (* | Ref x -> is_ground x *)
-    | _ -> false
+  match x.term with Ground _ -> true (* | Ref x -> is_ground x *) | _ -> false
 
-(** Print terms, (almost) assuming they are in normal form. *)
-let rec print_term v =
+let rec string_of_pat = function
+  | PVar l -> String.concat "." l
+  | PTuple l -> "(" ^ String.concat ", " (List.map string_of_pat l) ^ ")"
+  | PList (l, spread, l') ->
+      "["
+      ^ String.concat ", "
+          (List.map string_of_pat l
+          @ (match spread with None -> [] | Some v -> ["..." ^ v])
+          @ List.map string_of_pat l')
+      ^ "]"
+  | PMeth (pat, l) ->
+      (match pat with None -> "" | Some pat -> string_of_pat pat ^ ".")
+      ^ "{"
+      ^ String.concat ", "
+          (List.map
+             (fun (lbl, pat) ->
+               match pat with
+                 | None -> lbl
+                 | Some pat -> lbl ^ ": " ^ string_of_pat pat)
+             l)
+      ^ "}"
+
+(** String representation of terms, (almost) assuming they are in normal
+    form. *)
+let rec to_string v =
   match v.term with
     | Ground g -> Ground.to_string g
-    | Encoder e -> Encoder.string_of_format e
-    | List l -> "[" ^ String.concat ", " (List.map print_term l) ^ "]"
-    | Tuple l -> "(" ^ String.concat ", " (List.map print_term l) ^ ")"
+    | Encoder e ->
+        let rec aux (e, p) =
+          let p =
+            p
+            |> List.map (function
+                 | "", `Term v -> to_string v
+                 | l, `Term v -> l ^ "=" ^ to_string v
+                 | _, `Encoder e -> aux e)
+            |> String.concat ", "
+          in
+          "%" ^ e ^ "(" ^ p ^ ")"
+        in
+        aux e
+    | List l -> "[" ^ String.concat ", " (List.map to_string l) ^ "]"
+    | Tuple l -> "(" ^ String.concat ", " (List.map to_string l) ^ ")"
     | Null -> "null"
-    | Cast (e, t) -> "(" ^ print_term e ^ " : " ^ Type.print t ^ ")"
-    | Meth (l, v, e) -> print_term e ^ ".{" ^ l ^ " = " ^ print_term v ^ "}"
-    | Invoke (e, l) -> print_term e ^ "." ^ l
-    | Open (m, e) -> "open " ^ print_term m ^ " " ^ print_term e
-    | Fun (_, [], v) when is_ground v -> "{" ^ print_term v ^ "}"
+    | Cast (e, t) -> "(" ^ to_string e ^ " : " ^ Repr.string_of_type t ^ ")"
+    | Meth (l, v, e) -> to_string e ^ ".{" ^ l ^ " = " ^ to_string v ^ "}"
+    | Invoke (e, l) -> to_string e ^ "." ^ l
+    | Open (m, e) -> "open " ^ to_string m ^ " " ^ to_string e
+    | Fun (_, [], v) when is_ground v -> "{" ^ to_string v ^ "}"
     | Fun _ | RFun _ -> "<fun>"
     | Var s -> s
     | App (hd, tl) ->
         let tl =
           List.map
             (fun (lbl, v) ->
-              (if lbl = "" then "" else lbl ^ " = ") ^ print_term v)
+              (if lbl = "" then "" else lbl ^ " = ") ^ to_string v)
             tl
         in
-        print_term hd ^ "(" ^ String.concat "," tl ^ ")"
-    | Let _ | Seq _ -> assert false
+        to_string hd ^ "(" ^ String.concat "," tl ^ ")"
+    (* | Let _ | Seq _ -> assert false *)
+    | Let l ->
+        Printf.sprintf "let %s = %s in %s" (string_of_pat l.pat)
+          (to_string l.def) (to_string l.body)
+    | Seq (e, e') -> to_string e ^ "; " ^ to_string e'
 
-let rec string_of_pat = function
-  | PVar l -> String.concat "." l
-  | PTuple l -> "(" ^ String.concat ", " (List.map string_of_pat l) ^ ")"
+(** Create a new value. *)
+let make ?pos ?t e =
+  let t = match t with Some t -> t | None -> Type.var ?pos () in
+  if Lazy.force debug then
+    Printf.eprintf "%s (%s): assigned type var %s\n"
+      (Pos.Option.to_string t.Type.pos)
+      (try to_string { t; term = e } with _ -> "<?>")
+      (Repr.string_of_type t);
+  { t; term = e }
 
 let rec free_vars_pat = function
   | PVar [] -> assert false
   | PVar [_] -> Vars.empty
   | PVar (x :: _) -> Vars.singleton x
   | PTuple l -> List.fold_left Vars.union Vars.empty (List.map free_vars_pat l)
+  | PList (l, spread, l') ->
+      List.fold_left Vars.union Vars.empty
+        (List.map free_vars_pat
+           (l @ (match spread with None -> [] | Some v -> [PVar [v]]) @ l'))
+  | PMeth (pat, l) ->
+      List.fold_left Vars.union
+        (match pat with None -> Vars.empty | Some pat -> free_vars_pat pat)
+        (List.map free_vars_pat
+           (List.fold_left
+              (fun cur (lbl, pat) ->
+                [PVar [lbl]]
+                @ (match pat with None -> [] | Some pat -> [pat])
+                @ cur)
+              [] l))
 
 let rec bound_vars_pat = function
   | PVar [] -> assert false
   | PVar [x] -> Vars.singleton x
   | PVar _ -> Vars.empty
   | PTuple l -> List.fold_left Vars.union Vars.empty (List.map bound_vars_pat l)
+  | PList (l, spread, l') ->
+      List.fold_left Vars.union Vars.empty
+        (List.map bound_vars_pat
+           (l @ (match spread with None -> [] | Some v -> [PVar [v]]) @ l'))
+  | PMeth (pat, l) ->
+      List.fold_left Vars.union
+        (match pat with None -> Vars.empty | Some pat -> bound_vars_pat pat)
+        (List.map bound_vars_pat
+           (List.fold_left
+              (fun cur (lbl, pat) ->
+                [PVar [lbl]]
+                @ (match pat with None -> [] | Some pat -> [pat])
+                @ cur)
+              [] l))
 
 let rec free_vars tm =
   match tm.term with
-    | Ground _ | Encoder _ -> Vars.empty
+    | Ground _ -> Vars.empty
     | Var x -> Vars.singleton x
     | Tuple l ->
         List.fold_left (fun v a -> Vars.union v (free_vars a)) Vars.empty l
     | Null -> Vars.empty
+    | Encoder e ->
+        let rec enc (_, p) =
+          List.fold_left
+            (fun v (_, t) ->
+              match t with
+                | `Term t -> Vars.union v (free_vars t)
+                | `Encoder e -> Vars.union v (enc e))
+            Vars.empty p
+        in
+        enc e
     | Cast (e, _) -> free_vars e
     | Seq (a, b) -> Vars.union (free_vars a) (free_vars b)
     | Meth (_, v, e) -> Vars.union (free_vars v) (free_vars e)
@@ -426,21 +471,12 @@ let free_vars ?(bound = []) body =
    ignored). *)
 let can_ignore t =
   match (Type.demeth t).Type.descr with
-    | Type.Tuple [] | Type.EVar _ -> true
-    | _ -> false
-
-(* TODO: what about functions with methods? *)
-let is_fun t =
-  match (Type.deref t).Type.descr with Type.Arrow _ -> true | _ -> false
-
-let is_source t =
-  match (Type.demeth t).Type.descr with
-    | Type.Constr { Type.name = "source"; _ } -> true
+    | Type.Tuple [] | Type.Var _ -> true
     | _ -> false
 
 (** {1 Basic checks and errors} *)
 
-exception Unbound of Type.pos option * string
+exception Unbound of Pos.Option.t * string
 exception Ignored of t
 
 (** [No_label (f,lbl,first,x)] indicates that the parameter [x] could not be
@@ -450,18 +486,22 @@ exception Ignored of t
   * more helpful. *)
 exception No_label of t * string * bool * t
 
+(** Some mandatory arguments with given label and typed were not passed to the
+    function during an application. *)
+exception Missing_arguments of Pos.Option.t * (string * Type.t) list
+
 (** Check that all let-bound variables are used.
   * No check is performed for variable arguments.
   * This cannot be done at parse-time (as for the computatin of the
   * free variables of functions) because we need types, as well as
   * the ability to distinguish toplevel and inner let-in terms. *)
-exception Unused_variable of (string * Type.pos)
+exception Unused_variable of (string * Pos.t)
 
 let check_unused ~throw ~lib tm =
   let rec check ?(toplevel = false) v tm =
     match tm.term with
       | Var s -> Vars.remove s v
-      | Ground _ | Encoder _ -> v
+      | Ground _ -> v
       | Tuple l -> List.fold_left (fun a -> check a) v l
       | Null -> v
       | Cast (e, _) -> check v e
@@ -470,6 +510,14 @@ let check_unused ~throw ~lib tm =
       | Open (a, b) -> check (check v a) b
       | Seq (a, b) -> check ~toplevel (check v a) b
       | List l -> List.fold_left (fun x y -> check x y) v l
+      | Encoder e ->
+          let rec enc v (_, p) =
+            List.fold_left
+              (fun v (_, t) ->
+                match t with `Term t -> check v t | `Encoder e -> enc v e)
+              v p
+          in
+          enc v e
       | App (hd, l) ->
           let v = check v hd in
           List.fold_left (fun v (_, t) -> check v t) v l
@@ -515,7 +563,7 @@ let check_unused ~throw ~lib tm =
                      at toplevel (sort of a lib situation...) *)
                   if
                     s <> "_"
-                    && not (can_ignore def.t || (toplevel && is_fun def.t))
+                    && not (can_ignore def.t || (toplevel && Type.is_fun def.t))
                   then throw (Unused_variable (s, Option.get tm.t.Type.pos)))
               bvpat;
           Vars.union v mask
@@ -523,120 +571,52 @@ let check_unused ~throw ~lib tm =
   (* Unused free variables may remain *)
   ignore (check ~toplevel:true Vars.empty tm)
 
-(** Values are untyped normal forms of terms. *)
-module Value = struct
-  type t = { pos : Type.pos option; value : in_value }
+(* Abstract types. *)
 
-  and env = (string * t) list
+module type Abstract = sig
+  type content
 
-  (* Some values have to be lazy in the environment because of recursive functions. *)
-  and lazy_env = (string * t Lazy.t) list
-
-  and in_value =
-    | Ground of Ground.t
-    | Source of Source.source
-    | Encoder of Encoder.format
-    | List of t list
-    | Tuple of t list
-    | Null
-    (* TODO: It would be better to have a list of methods associated to each
-       value than a constructor here. However, I am keeping as is for now because
-       implementation is safer this way. *)
-    | Meth of string * t * t
-    | Ref of t ref
-    (* The first environment contains the parameters already passed to the
-       function. Next parameters will be inserted between that and the second
-       env which is part of the closure. *)
-    | Fun of (string * string * t option) list * env * lazy_env * term
-    (* For a foreign function only the arguments are visible, the closure
-       doesn't capture anything in the environment. *)
-    | FFI of (string * string * t option) list * env * (env -> t)
-
-  let unit : in_value = Tuple []
-
-  let string_of_float f =
-    let s = string_of_float f in
-    if s.[String.length s - 1] = '.' then s ^ "0" else s
-
-  let rec print_value v =
-    match v.value with
-      | Ground g -> Ground.to_string g
-      | Source _ -> "<source>"
-      | Encoder e -> Encoder.string_of_format e
-      | List l -> "[" ^ String.concat ", " (List.map print_value l) ^ "]"
-      | Ref a -> Printf.sprintf "ref(%s)" (print_value !a)
-      | Tuple l -> "(" ^ String.concat ", " (List.map print_value l) ^ ")"
-      | Null -> "null"
-      | Meth (l, v, e) when Lazy.force debug ->
-          print_value e ^ ".{" ^ l ^ "=" ^ print_value v ^ "}"
-      | Meth _ ->
-          let rec split e =
-            match e.value with
-              | Meth (l, v, e) ->
-                  let m, e = split e in
-                  ((l, v) :: m, e)
-              | _ -> ([], e)
-          in
-          let m, e = split v in
-          let m =
-            List.rev m
-            |> List.map (fun (l, v) -> l ^ " = " ^ print_value v)
-            |> String.concat ", "
-          in
-          let e =
-            match e.value with Tuple [] -> "" | _ -> print_value e ^ "."
-          in
-          e ^ "{" ^ m ^ "}"
-      | Fun ([], _, _, x) when is_ground x -> "{" ^ print_term x ^ "}"
-      | Fun (l, _, _, x) when is_ground x ->
-          let f (label, _, value) =
-            match (label, value) with
-              | "", None -> "_"
-              | "", Some v -> Printf.sprintf "_=%s" (print_value v)
-              | label, Some v -> Printf.sprintf "~%s=%s" label (print_value v)
-              | label, None -> Printf.sprintf "~%s=_" label
-          in
-          let args = List.map f l in
-          Printf.sprintf "fun (%s) -> %s" (String.concat "," args)
-            (print_term x)
-      | Fun _ | FFI _ -> "<fun>"
-
-  (** Find a method in a value. *)
-  let rec invoke x l =
-    match x.value with
-      | Meth (l', y, _) when l' = l -> y
-      | Meth (_, _, x) -> invoke x l
-      | _ -> failwith ("Could not find method " ^ l ^ " of " ^ print_value x)
-
-  (** Perform a sequence of invokes: invokes x [l1;l2;l3;...] is x.l1.l2.l3... *)
-  let rec invokes x = function l :: ll -> invokes (invoke x l) ll | [] -> x
-
-  let split_meths e =
-    let rec aux hide e =
-      match e.value with
-        | Meth (l, v, e) ->
-            if List.mem l hide then aux hide e
-            else (
-              let m, e = aux (l :: hide) e in
-              ((l, v) :: m, e))
-        | _ -> ([], e)
-    in
-    aux [] e
-
-  let rec demeth v = match v.value with Meth (_, _, v) -> demeth v | _ -> v
-
-  let rec remeth t u =
-    match t.value with
-      | Meth (l, v, t) -> { t with value = Meth (l, v, remeth t u) }
-      | _ -> u
-
-  let uniq_meth t =
-    let rec f meths t =
-      match t.value with
-        | Meth (m, _, t') when List.mem m meths ->
-            { t with value = (f meths t').value }
-        | Meth (m, v, t') -> { t with value = Meth (m, v, f (m :: meths) t') }
-        | _ -> t
-    in
-    f [] t
+  val t : Type.t
+  val to_ground : content -> Ground.t
+  val of_ground : Ground.t -> content
+  val is_ground : Ground.t -> bool
+  val to_term : content -> t
+  val of_term : t -> content
+  val is_term : t -> bool
+end
+
+module type AbstractDef = sig
+  type content
+
+  val name : string
+  val to_json : pos:Pos.t list -> content -> Json.t
+  val descr : content -> string
+  val compare : content -> content -> int
+end
+
+module MkAbstract (Def : AbstractDef) = struct
+  type Type.ground += Type
+  type Ground.t += Value of Def.content
+
+  let () =
+    let to_value = function Value v -> v | _ -> assert false in
+    let compare v v' = Def.compare (to_value v) (to_value v') in
+    let descr v = Def.descr (to_value v) in
+    let to_json ~pos v = Def.to_json ~pos (to_value v) in
+    Ground.register
+      (function Value _ -> true | _ -> false)
+      { Ground.descr; to_json; compare; typ = Type };
+    Type.register_ground_printer (function Type -> Some Def.name | _ -> None);
+    Type.register_ground_resolver (fun s ->
+        if s = Def.name then Some Type else None)
+
+  type content = Def.content
+
+  let t = Type.make (Type.Ground Type)
+  let of_ground = function Value c -> c | _ -> assert false
+  let to_ground c = Value c
+  let is_ground = function Value _ -> true | _ -> false
+  let of_term t = match t.term with Ground (Value c) -> c | _ -> assert false
+  let to_term c = { t = Type.make (Type.Ground Type); term = Ground (Value c) }
+  let is_term t = match t.term with Ground (Value _) -> true | _ -> false
 end
