@@ -23,8 +23,11 @@
 open Term
 open Typing
 
+let debug = ref false
+
 (** {1 Type checking / inference} *)
 
+(** Terms for which generalization is safe. *)
 let rec value_restriction t =
   match t.term with
     | Var _ -> true
@@ -35,10 +38,11 @@ let rec value_restriction t =
     | Meth (_, t, u) -> value_restriction t && value_restriction u
     (* | Invoke (t, _) -> value_restriction t *)
     | Ground _ -> true
+    | Let l -> value_restriction l.def && value_restriction l.body
     | _ -> false
 
-(** A simple mechanism for delaying printing toplevel tasks
-  * as late as possible, to avoid seeing too many unknown variables. *)
+(** A simple mechanism for delaying printing toplevel tasks as late as possible,
+    to avoid seeing too many unknown variables. *)
 let add_task, pop_tasks =
   let q = Queue.create () in
   ( (fun f -> Queue.add f q),
@@ -52,7 +56,7 @@ let add_task, pop_tasks =
 (** Generate a type with fresh variables for a pattern. *)
 let rec type_of_pat ~level ~pos = function
   | PVar x ->
-      let a = Type.fresh_evar ~level ~pos in
+      let a = Type.var ~level ?pos () in
       ([(x, a)], a)
   | PTuple l ->
       let env, l =
@@ -63,143 +67,196 @@ let rec type_of_pat ~level ~pos = function
           ([], []) l
       in
       let l = List.rev l in
-      (env, Type.make ~level ~pos (Type.Tuple l))
+      (env, Type.make ?pos (Type.Tuple l))
+  | PList (l, spread, l') ->
+      let fold_env l ty =
+        List.fold_left
+          (fun (env, ty, ety) p ->
+            let env', ty' = type_of_pat ~level ~pos p in
+            let ty = Typing.sup ~pos ty ty' in
+            (env' @ env, ty, ty' :: ety))
+          ([], ty, []) l
+      in
+      let ty = Type.var ~level ?pos () in
+      let env, ty, ety = fold_env l ty in
+      let env', ty, ety' = fold_env l' ty in
+      let spread_env =
+        match spread with
+          | None -> []
+          | Some v ->
+              [([v], Type.make ?pos Type.(List { t = ty; json_repr = `Tuple }))]
+      in
+      List.iter (fun ety -> Typing.(ety <: ty)) (ety @ ety');
+      ( env' @ spread_env @ env,
+        Type.make ?pos Type.(List { t = ty; json_repr = `Tuple }) )
+  | PMeth (pat, l) ->
+      let env, ty =
+        match pat with
+          | None -> ([], Type.make ?pos (Type.Tuple []))
+          | Some pat -> type_of_pat ~level ~pos pat
+      in
+      let env, ty =
+        List.fold_left
+          (fun (env, ty) (lbl, p) ->
+            let env', a =
+              match p with
+                | None -> ([], Type.var ~level ?pos ())
+                | Some pat -> type_of_pat ~level ~pos pat
+            in
+            let ty =
+              Type.make ?pos
+                Type.(
+                  Meth
+                    ( {
+                        meth = lbl;
+                        scheme = ([], a);
+                        doc = "";
+                        json_name = None;
+                      },
+                      ty ))
+            in
+            (env' @ [([lbl], a)] @ env, ty))
+          (env, ty) l
+      in
+      (env, ty)
 
-(* Type-check an expression.
- * [level] should be the sum of the lengths of [env] and [builtins],
- * that is the size of the typing context, that is the number of surrounding
- * abstractions. *)
+(* Type-check an expression. *)
 let rec check ?(print_toplevel = false) ~throw ~level ~(env : Typing.env) e =
-  (* The role of this function is not only to type-check but also to assign
-   * meaningful levels to type variables, and unify the types of
-   * all occurrences of the same variable, since the parser does not do it. *)
-  assert (e.t.Type.level = -1);
-  e.t.Type.level <- level;
-
-  (* The toplevel position of the (un-dereferenced) type
-   * is the actual parsing position of the value.
-   * When we synthesize a type against which the type of the term is unified,
-   * we have to set the position information in order not to loose it. *)
+  let check = check ~throw in
+  if !debug then Printf.printf "\n# %s : ?\n\n%!" (Term.to_string e);
+  let check ?print_toplevel ~level ~env e =
+    check ?print_toplevel ~level ~env e;
+    if !debug then
+      Printf.printf "\n# %s : %s\n\n%!" (Term.to_string e) (Type.to_string e.t)
+  in
+  (* The toplevel position of the (un-dereferenced) type is the actual parsing
+     position of the value. When we synthesize a type against which the type of
+     the term is unified, we have to set the position information in order not
+     to loose it. *)
   let pos = e.t.Type.pos in
-  let mk t = Type.make ~level ~pos t in
+  let mk t = Type.make ?pos t in
   let mkg t = mk (Type.Ground t) in
   let check_fun ~proto ~env e body =
-    let base_check = check ~throw ~level ~env in
-    let proto_t, env, level =
+    let base_check = check ~level ~env in
+    let proto_t, env =
       List.fold_left
-        (fun (p, env, level) -> function
+        (fun (p, env) -> function
           | lbl, var, kind, None ->
-              if Lazy.force debug then
-                Printf.eprintf "Assigning level %d to %s (%s).\n" level var
-                  (Type.print kind);
-              kind.Type.level <- level;
-              ((false, lbl, kind) :: p, (var, ([], kind)) :: env, level + 1)
+              update_level level kind;
+              ((false, lbl, kind) :: p, (var, ([], kind)) :: env)
           | lbl, var, kind, Some v ->
-              if Lazy.force debug then
-                Printf.eprintf "Assigning level %d to %s (%s).\n" level var
-                  (Type.print kind);
-              kind.Type.level <- level;
+              update_level level kind;
               base_check v;
               v.t <: kind;
-              ((true, lbl, kind) :: p, (var, ([], kind)) :: env, level + 1))
-        ([], env, level) proto
+              ((true, lbl, kind) :: p, (var, ([], kind)) :: env))
+        ([], env) proto
     in
     let proto_t = List.rev proto_t in
-    check ~throw ~level ~env body;
+    check ~level ~env body;
     e.t >: mk (Type.Arrow (proto_t, body.t))
   in
   match e.term with
     | Ground g -> e.t >: mkg (Ground.to_type g)
-    | Encoder f -> e.t >: type_of_format ~pos:e.t.Type.pos ~level f
-    | List l ->
-        List.iter (fun x -> check ~throw ~level ~env x) l;
-        let t =
-          List.fold_left
-            (fun t e -> Typing.min_type ~pos:e.t.Type.pos ~level t e.t)
-            (Type.fresh_evar ~level ~pos)
-            l
+    | Encoder f ->
+        (* Ensure that we only use well-formed terms. *)
+        let rec check_enc (_, p) =
+          List.iter
+            (function
+              | _, `Term t -> check ~level ~env t | _, `Encoder e -> check_enc e)
+            p
         in
+        check_enc f;
+        let t =
+          try Lang_encoder.type_of_encoder ~pos:e.t.Type.pos f
+          with Not_found ->
+            let bt = Printexc.get_raw_backtrace () in
+            Printexc.raise_with_backtrace
+              (Unsupported_format (pos, Term.to_string e))
+              bt
+        in
+        e.t >: t
+    | List l ->
+        List.iter (fun x -> check ~level ~env x) l;
+        let t = Type.var ~level ?pos () in
         List.iter (fun e -> e.t <: t) l;
-        e.t >: mk (Type.List t)
+        e.t >: mk Type.(List { t; json_repr = `Tuple })
     | Tuple l ->
-        List.iter (fun a -> check ~throw ~level ~env a) l;
+        List.iter (fun a -> check ~level ~env a) l;
         e.t >: mk (Type.Tuple (List.map (fun a -> a.t) l))
-    | Null -> e.t >: mk (Type.Nullable (Type.fresh_evar ~level ~pos))
+    | Null -> e.t >: mk (Type.Nullable (Type.var ~level ?pos ()))
     | Cast (a, t) ->
-        check ~throw ~level ~env a;
+        check ~level ~env a;
         a.t <: t;
         e.t >: t
     | Meth (l, a, b) ->
-        check ~throw ~level ~env a;
-        check ~throw ~level ~env b;
+        check ~level ~env a;
+        check ~level ~env b;
         e.t
-        >: mk (Type.Meth (l, (Typing.generalizable ~level a.t, a.t), "", b.t))
+        >: mk
+             (Type.Meth
+                ( {
+                    Type.meth = l;
+                    scheme = Typing.generalize ~level a.t;
+                    doc = "";
+                    json_name = None;
+                  },
+                  b.t ))
     | Invoke (a, l) ->
-        check ~throw ~level ~env a;
+        check ~level ~env a;
         let rec aux t =
           match (Type.deref t).Type.descr with
-            | Type.Meth (l', (generalized, b), _, c) ->
+            | Type.(Meth ({ meth = l'; scheme = generalized, b }, c)) ->
                 if l = l' then Typing.instantiate ~level ~generalized b
                 else aux c
             | _ ->
                 (* We did not find the method, the type we will infer is not the
                    most general one (no generalization), but this is safe and
                    enough for records. *)
-                let x = Type.fresh_evar ~level ~pos in
-                let y = Type.fresh_evar ~level ~pos in
-                a.t <: mk (Type.Meth (l, ([], x), "", y));
+                let x = Type.var ~level ?pos () in
+                let y = Type.var ~level ?pos () in
+                a.t
+                <: mk
+                     Type.(
+                       Meth
+                         ( {
+                             meth = l;
+                             scheme = ([], x);
+                             doc = "";
+                             json_name = None;
+                           },
+                           y ));
                 x
         in
         e.t >: aux a.t
     | Open (a, b) ->
-        check ~throw ~level ~env a;
+        check ~level ~env a;
         a.t <: mk Type.unit;
         let rec aux env t =
           match (Type.deref t).Type.descr with
-            | Type.Meth (l, (g, u), _, t) -> aux ((l, (g, u)) :: env) t
+            | Type.(Meth ({ meth = l; scheme = g, u }, t)) ->
+                aux ((l, (g, u)) :: env) t
             | _ -> env
         in
         let env = aux env a.t in
-        check ~throw ~level ~env b;
+        check ~level ~env b;
         e.t >: b.t
     | Seq (a, b) ->
-        check ~throw ~env ~level a;
+        check ~env ~level a;
         if not (can_ignore a.t) then throw (Ignored a);
-        check ~print_toplevel ~throw ~level ~env b;
+        check ~print_toplevel ~level ~env b;
         e.t >: b.t
-    (* Special case for if branchings in order to backtrack in the case where
-       the then branch is nullable but the else branch is not, see #1816. *)
-    | App
-        ( ({ term = Var "if" } as e_if),
-          [("", e_cond); ("then", e_then); ("else", e_else)] ) ->
-        check ~throw ~level ~env e_if;
-        check ~throw ~level ~env e_cond;
-        check ~throw ~level ~env e_then;
-        check ~throw ~level ~env e_else;
-        let a = Type.fresh_evar ~level ~pos in
-        e_cond.t <: Type.make (Type.Ground Type.Bool);
-        e_else.t <: Type.make (Type.Arrow ([], a));
-        if
-          try
-            e_then.t <: Type.make (Type.Arrow ([], a));
-            true
-          with _ -> false
-        then e.t >: a
-        else (
-          e_then.t <: Type.make (Type.Arrow ([], Type.make (Type.Nullable a)));
-          e.t >: Type.make (Type.Nullable a))
     | App (a, l) -> (
-        check ~throw ~level ~env a;
-        List.iter (fun (_, b) -> check ~throw ~env ~level b) l;
+        check ~level ~env a;
+        List.iter (fun (_, b) -> check ~env ~level b) l;
 
-        (* If [a] is known to have a function type, manually dig through
-         * it for better error messages. Otherwise generate its type
-         * and unify -- in that case the optionality can't be guessed
-         * and mandatory is the default. *)
+        (* If [a] is known to have a function type, manually dig through it for
+           better error messages. Otherwise generate its type and unify -- in
+           that case the optionality can't be guessed and mandatory is the
+           default. *)
         match (Type.demeth a.t).Type.descr with
           | Type.Arrow (ap, t) ->
-              (* Find in l the first arg labeled lbl,
-               * return it together with the remaining of the list. *)
+              (* Find in l the first arg labeled lbl, return it together with the
+                 remaining of the list. *)
               let get_arg lbl l =
                 let rec aux acc = function
                   | [] -> None
@@ -210,8 +267,8 @@ let rec check ?(print_toplevel = false) ~throw ~level ~(env : Typing.env) e =
                 aux [] l
               in
               let _, ap =
-                (* Remove the applied parameters,
-                 * check their types on the fly. *)
+                (* Remove the applied parameters, check their types on the
+                   fly. *)
                 List.fold_left
                   (fun (already, ap) (lbl, v) ->
                     match get_arg lbl ap with
@@ -223,12 +280,18 @@ let rec check ?(print_toplevel = false) ~throw ~level ~(env : Typing.env) e =
                           (lbl :: already, ap'))
                   ([], ap) l
               in
-              (* See if any mandatory argument remains, check the return type. *)
-              if List.for_all (fun (o, _, _) -> o) ap then e.t >: t
-              else e.t >: Type.make ~level ~pos:None (Type.Arrow (ap, t))
+              (* See if any mandatory argument is missing. *)
+              let mandatory =
+                List.filter_map
+                  (fun (o, l, t) -> if o then None else Some (l, t))
+                  ap
+              in
+              if mandatory <> [] then
+                raise (Term.Missing_arguments (pos, mandatory));
+              e.t >: t
           | _ ->
               let p = List.map (fun (lbl, b) -> (false, lbl, b.t)) l in
-              a.t <: Type.make ~level ~pos:None (Type.Arrow (p, e.t)))
+              a.t <: Type.make (Type.Arrow (p, e.t)))
     | Fun (_, proto, body) -> check_fun ~proto ~env e body
     | RFun (x, _, proto, body) ->
         let env = (x, ([], e.t)) :: env in
@@ -239,28 +302,16 @@ let rec check ?(print_toplevel = false) ~throw ~level ~(env : Typing.env) e =
           with Not_found -> raise (Unbound (e.t.Type.pos, var))
         in
         e.t >: Typing.instantiate ~level ~generalized orig;
-        if Lazy.force debug then
-          Printf.eprintf "Instantiate %s[%d] : %s becomes %s\n" var
-            (Type.deref e.t).Type.level (Type.print orig) (Type.print e.t)
+        if Lazy.force Term.debug then
+          Printf.eprintf "Instantiate %s : %s becomes %s\n" var
+            (Type.to_string orig) (Type.to_string e.t)
     | Let ({ pat; replace; def; body; _ } as l) ->
-        check ~throw ~level ~env def;
+        check ~level:(level + 1) ~env def;
         let generalized =
-          if value_restriction def then (
-            let f x t =
-              let x' =
-                Type.filter_vars
-                  (function
-                    | { Type.descr = Type.EVar (i, _); level = l; _ } ->
-                        (not (List.mem_assoc i x)) && l >= level
-                    | _ -> assert false)
-                  t
-              in
-              x' @ x
-            in
-            f [] def.t)
-          else []
+          (* Printf.printf "generalize at %d: %B\n\n!" level (value_restriction def); *)
+          if value_restriction def then fst (generalize ~level def.t) else []
         in
-        let penv, pa = type_of_pat ~level ~pos pat in
+        let penv, pa = type_of_pat ~level:(level + 1) ~pos pat in
         def.t <: pa;
         let penv =
           List.map
@@ -272,6 +323,9 @@ let rec check ?(print_toplevel = false) ~throw ~level ~(env : Typing.env) e =
                       if replace then Type.remeth (snd (List.assoc x env)) a
                       else a
                     in
+                    if !debug then
+                      Printf.printf "\nLET %s : %s\n%!" x
+                        (Repr.string_of_scheme (generalized, a));
                     (x, (generalized, a))
                 | l :: ll -> (
                     try
@@ -281,7 +335,7 @@ let rec check ?(print_toplevel = false) ~throw ~level ~(env : Typing.env) e =
                         if replace then Type.remeth (snd (Type.invokes t ll)) a
                         else a
                       in
-                      (l, (g, Type.meths ~pos ~level ll (generalized, a) t))
+                      (l, (g, Type.meths ?pos ll (generalized, a) t))
                     with Not_found -> raise (Unbound (pos, l))))
             penv
         in
@@ -293,9 +347,9 @@ let rec check ?(print_toplevel = false) ~throw ~level ~(env : Typing.env) e =
                 (let name = string_of_pat pat in
                  let l = String.length name and max = 5 in
                  if l >= max then name else name ^ String.make (max - l) ' ')
-                (Type.pp_type_generalized generalized)
+                (fun f t -> Repr.print_scheme f (generalized, t))
                 def.t);
-        check ~print_toplevel ~throw ~level:(level + 1) ~env body;
+        check ~print_toplevel ~level ~env body;
         e.t >: body.t
 
 (* The simple definition for external use. *)
@@ -303,9 +357,10 @@ let check ?(ignored = false) ~throw e =
   let print_toplevel = !Configure.display_types in
   try
     let env = Environment.default_typing_environment () in
-    check ~print_toplevel ~throw ~level:(List.length env) ~env e;
+    check ~print_toplevel ~throw ~level:0 ~env e;
     if print_toplevel && (Type.deref e.t).Type.descr <> Type.unit then
-      add_task (fun () -> Format.printf "@[<2>-     :@ %a@]@." Type.pp_type e.t);
+      add_task (fun () ->
+          Format.printf "@[<2>-     :@ %a@]@." Repr.print_type e.t);
     if ignored && not (can_ignore e.t) then throw (Ignored e);
     pop_tasks ()
   with e ->
