@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,7 +31,10 @@ type env = (string * scheme) list
 val instantiate : level:int -> generalized:var list -> t -> t
 
 (** Find all generalizable variables. *)
-val generalizable : level:int -> t -> var list
+val generalize : level:int -> t -> scheme
+
+(** Lower all type variables to given level. *)
+val update_level : int -> t -> unit
 
 (** Subtyping. *)
 val ( <: ) : t -> t -> unit
@@ -39,5 +42,5 @@ val ( <: ) : t -> t -> unit
 (** Suptyping. *)
 val ( >: ) : t -> t -> unit
 
-(** Minimum of two types. *)
-val min_type : ?pos:pos option -> ?level:int -> t -> t -> t
+(** Supremeum of two types. *)
+val sup : pos:Pos.Option.t -> t -> t -> t
