@@ -20,9 +20,8 @@
 
  *****************************************************************************)
 
-open Term
-open Term.Ground
-open Lang_encoders
+open Value
+open Ground
 
 let make params =
   let defaults =
@@ -31,16 +30,27 @@ let make params =
          number of channels too early, see #933. *)
       Avi_format.channels = 2;
       samplerate = Frame.audio_rate;
+      width = Frame.video_width;
+      height = Frame.video_height;
     }
   in
   let avi =
     List.fold_left
       (fun f -> function
-        | "channels", { term = Ground (Int c); _ } ->
+        | "channels", `Value { value = Ground (Int c); _ } ->
             { f with Avi_format.channels = c }
-        | "samplerate", { term = Ground (Int i); _ } ->
+        | "samplerate", `Value { value = Ground (Int i); _ } ->
             { f with Avi_format.samplerate = Lazy.from_val i }
-        | _, t -> raise (generic_error t))
+        | "width", `Value { value = Ground (Int i); _ } ->
+            { f with Avi_format.width = Lazy.from_val i }
+        | "height", `Value { value = Ground (Int i); _ } ->
+            { f with Avi_format.height = Lazy.from_val i }
+        | t -> Lang_encoder.raise_generic_error t)
       defaults params
   in
   Encoder.AVI avi
+
+let kind_of_encoder p =
+  Encoder.audio_video_kind (Lang_encoder.channels_of_params p)
+
+let () = Lang_encoder.register "avi" kind_of_encoder make
