@@ -20,9 +20,10 @@
 
  *****************************************************************************)
 
-open Term
-open Term.Ground
-open Lang_encoders
+open Value
+open Ground
+
+let kind_of_encoder p = Encoder.audio_kind (Lang_encoder.channels_of_params p)
 
 let make params =
   let defaults =
@@ -31,7 +32,7 @@ let make params =
       (* We use a hardcoded value in order not to force the evaluation of the
            number of channels too early, see #933. *) = 2;
       samplerate = Frame.audio_rate;
-      video = false;
+      video = None;
       header = true;
       restart_on_crash = false;
       restart = External_encoder_format.No_condition;
@@ -41,34 +42,60 @@ let make params =
   let ext =
     List.fold_left
       (fun f -> function
-        | "channels", { term = Ground (Int c); _ } ->
+        | "channels", `Value { value = Ground (Int c); _ } ->
             { f with External_encoder_format.channels = c }
-        | "samplerate", { term = Ground (Int i); _ } ->
+        | "samplerate", `Value { value = Ground (Int i); _ } ->
             { f with External_encoder_format.samplerate = Lazy.from_val i }
-        | "video", { term = Ground (Bool h); _ } ->
-            { f with External_encoder_format.video = h }
-        | "header", { term = Ground (Bool h); _ } ->
+        | "video", `Value { value = Ground (Bool b); _ } ->
+            let w, h =
+              match f.External_encoder_format.video with
+                | None -> (Frame.video_width, Frame.video_height)
+                | Some (w, h) -> (w, h)
+            in
+            {
+              f with
+              External_encoder_format.video = (if b then Some (w, h) else None);
+            }
+        | "width", `Value { value = Ground (Int w); _ } ->
+            let _, h =
+              match f.External_encoder_format.video with
+                | None -> (Frame.video_width, Frame.video_height)
+                | Some (w, h) -> (w, h)
+            in
+            let w = Lazy.from_val w in
+            { f with External_encoder_format.video = Some (w, h) }
+        | "height", `Value { value = Ground (Int h); _ } ->
+            let w, _ =
+              match f.External_encoder_format.video with
+                | None -> (Frame.video_width, Frame.video_height)
+                | Some (w, h) -> (w, h)
+            in
+            let h = Lazy.from_val h in
+            { f with External_encoder_format.video = Some (w, h) }
+        | "header", `Value { value = Ground (Bool h); _ } ->
             { f with External_encoder_format.header = h }
-        | "restart_on_crash", { term = Ground (Bool h); _ } ->
+        | "restart_on_crash", `Value { value = Ground (Bool h); _ } ->
             { f with External_encoder_format.restart_on_crash = h }
-        | "", { term = Var s; _ }
+        | "", `Value { value = Ground (String s); _ }
           when String.lowercase_ascii s = "restart_on_metadata" ->
             {
               f with
               External_encoder_format.restart = External_encoder_format.Metadata;
             }
-        | "restart_after_delay", { term = Ground (Int i); _ } ->
+        | "restart_after_delay", `Value { value = Ground (Int i); _ } ->
             {
               f with
               External_encoder_format.restart = External_encoder_format.Delay i;
             }
-        | "process", { term = Ground (String s); _ } ->
+        | "process", `Value { value = Ground (String s); _ } ->
             { f with External_encoder_format.process = s }
-        | "", { term = Ground (String s); _ } ->
+        | "", `Value { value = Ground (String s); _ } ->
             { f with External_encoder_format.process = s }
-        | _, t -> raise (generic_error t))
+        | t -> Lang_encoder.raise_generic_error t)
       defaults params
   in
   if ext.External_encoder_format.process = "" then
     raise External_encoder_format.No_process;
   Encoder.External ext
+
+let () = Lang_encoder.register "external" kind_of_encoder make
