@@ -20,32 +20,31 @@
 
  *****************************************************************************)
 
-open Term
-open Term.Ground
-open Lang_encoders
+open Value
+open Ground
+
+let kind_of_encoder p = Encoder.audio_kind (Lang_encoder.channels_of_params p)
 
 let allowed_bitrates =
   [
     8; 16; 24; 32; 40; 48; 56; 64; 80; 96; 112; 128; 144; 160; 192; 224; 256; 320;
   ]
 
-let check_samplerate ?t i =
+let check_samplerate ~pos i =
   lazy
     (let i = Lazy.force i in
      let allowed =
        [8000; 11025; 12000; 16000; 22050; 24000; 32000; 44100; 48000]
      in
-     if not (List.mem i allowed) then (
-       match t with
-         | Some t -> raise (Error (t, "invalid samplerate value"))
-         | None -> failwith "invalid samplerate value");
+     if not (List.mem i allowed) then
+       Lang_encoder.raise_error ~pos "invalid samplerate value";
      i)
 
 let mp3_base_defaults () =
   {
     Mp3_format.stereo = true;
     stereo_mode = Mp3_format.Joint_stereo;
-    samplerate = check_samplerate Frame.audio_rate;
+    samplerate = check_samplerate ~pos:None Frame.audio_rate;
     bitrate_control = Mp3_format.CBR 128;
     internal_quality = 2;
     id3v2 = None;
@@ -54,39 +53,45 @@ let mp3_base_defaults () =
   }
 
 let mp3_base f = function
-  | "stereo", { term = Ground (Bool b); _ } -> { f with Mp3_format.stereo = b }
-  | "mono", { term = Ground (Bool b); _ } ->
+  | "stereo", `Value { value = Ground (Bool b); _ } ->
+      { f with Mp3_format.stereo = b }
+  | "mono", `Value { value = Ground (Bool b); _ } ->
       { f with Mp3_format.stereo = not b }
-  | "stereo_mode", ({ term = Ground (String m); _ } as t) ->
+  | "stereo_mode", `Value { value = Ground (String m); pos } ->
       let mode =
         match m with
           | "default" -> Mp3_format.Default
           | "joint_stereo" -> Mp3_format.Joint_stereo
           | "stereo" -> Mp3_format.Stereo
-          | _ -> raise (Error (t, "invalid stereo mode"))
+          | _ -> Lang_encoder.raise_error ~pos "invalid stereo mode"
       in
       { f with Mp3_format.stereo_mode = mode }
-  | "internal_quality", ({ term = Ground (Int q); _ } as t) ->
+  | "internal_quality", `Value { value = Ground (Int q); pos } ->
       if q < 0 || q > 9 then
-        raise (Error (t, "internal quality must be a value between 0 and 9"));
+        Lang_encoder.raise_error ~pos
+          "internal quality must be a value between 0 and 9";
       { f with Mp3_format.internal_quality = q }
-  | "msg_interval", { term = Ground (Float i); _ } ->
+  | "msg_interval", `Value { value = Ground (Float i); _ } ->
       { f with Mp3_format.msg_interval = i }
-  | "msg", { term = Ground (String m); _ } -> { f with Mp3_format.msg = m }
-  | "samplerate", ({ term = Ground (Int i); _ } as t) ->
-      { f with Mp3_format.samplerate = check_samplerate ~t (Lazy.from_val i) }
-  | "id3v2", ({ term = Ground (Bool true); _ } as t) -> (
+  | "msg", `Value { value = Ground (String m); _ } ->
+      { f with Mp3_format.msg = m }
+  | "samplerate", `Value { value = Ground (Int i); pos } ->
+      { f with Mp3_format.samplerate = check_samplerate ~pos (Lazy.from_val i) }
+  | "id3v2", `Value { value = Ground (Bool true); pos } -> (
       match !Mp3_format.id3v2_export with
         | None ->
-            raise (Error (t, "no id3v2 support available for the mp3 encoder"))
+            Lang_encoder.raise_error ~pos
+              "no id3v2 support available for the mp3 encoder"
         | Some g -> { f with Mp3_format.id3v2 = Some g })
-  | "id3v2", { term = Ground (Bool false); _ } ->
+  | "id3v2", `Value { value = Ground (Bool false); _ } ->
       { f with Mp3_format.id3v2 = None }
-  | "", { term = Var s; _ } when String.lowercase_ascii s = "mono" ->
+  | "", `Value { value = Ground (String s); _ }
+    when String.lowercase_ascii s = "mono" ->
       { f with Mp3_format.stereo = false }
-  | "", { term = Var s; _ } when String.lowercase_ascii s = "stereo" ->
+  | "", `Value { value = Ground (String s); _ }
+    when String.lowercase_ascii s = "stereo" ->
       { f with Mp3_format.stereo = true }
-  | _, t -> raise (generic_error t)
+  | t -> Lang_encoder.raise_generic_error t
 
 let make_cbr params =
   let defaults =
@@ -104,9 +109,9 @@ let make_cbr params =
   let mp3 =
     List.fold_left
       (fun f -> function
-        | "bitrate", ({ term = Ground (Int i); _ } as t) ->
+        | "bitrate", `Value { value = Ground (Int i); pos } ->
             if not (List.mem i allowed_bitrates) then
-              raise (Error (t, "invalid bitrate value"));
+              Lang_encoder.raise_error ~pos "invalid bitrate value";
             set_bitrate f i
         | x -> mp3_base f x)
       defaults params
@@ -196,29 +201,30 @@ let make_abr_vbr ~default params =
     in
     List.fold_left
       (fun f -> function
-        | "quality", ({ term = Ground (Int q); _ } as t) when is_vbr f ->
+        | "quality", `Value { value = Ground (Int q); pos } when is_vbr f ->
             if q < 0 || q > 9 then
-              raise (Error (t, "quality should be in [0..9]"));
+              Lang_encoder.raise_error ~pos "quality should be in [0..9]";
             set_quality f (Some q)
-        | "hard_min", { term = Ground (Bool b); _ } -> set_hard_min f (Some b)
-        | "bitrate", ({ term = Ground (Int i); _ } as t) ->
+        | "hard_min", `Value { value = Ground (Bool b); _ } ->
+            set_hard_min f (Some b)
+        | "bitrate", `Value { value = Ground (Int i); pos } ->
             if not (List.mem i allowed_bitrates) then
-              raise (Error (t, "invalid bitrate value"));
+              Lang_encoder.raise_error ~pos "invalid bitrate value";
             set_mean_bitrate f (Some i)
-        | "min_bitrate", ({ term = Ground (Int i); _ } as t) ->
+        | "min_bitrate", `Value { value = Ground (Int i); pos } ->
             if not (List.mem i allowed_bitrates) then
-              raise (Error (t, "invalid bitrate value"));
+              Lang_encoder.raise_error ~pos "invalid bitrate value";
             set_min_bitrate f (Some i)
-        | "max_bitrate", ({ term = Ground (Int i); _ } as t) ->
+        | "max_bitrate", `Value { value = Ground (Int i); pos } ->
             if not (List.mem i allowed_bitrates) then
-              raise (Error (t, "invalid bitrate value"));
+              Lang_encoder.raise_error ~pos "invalid bitrate value";
             set_max_bitrate f (Some i)
         | x -> mp3_base f x)
       default params
   in
   Encoder.MP3 mp3
 
-let make_abr =
+let make_abr p =
   make_abr_vbr
     ~default:
       {
@@ -233,8 +239,9 @@ let make_abr =
               hard_min = Some false;
             };
       }
+    p
 
-let make_vbr =
+let make_vbr p =
   make_abr_vbr
     ~default:
       {
@@ -249,3 +256,10 @@ let make_vbr =
               hard_min = None;
             };
       }
+    p
+
+let () =
+  Lang_encoder.register "mp3" kind_of_encoder make_cbr;
+  Lang_encoder.register "mp3.cbr" kind_of_encoder make_cbr;
+  Lang_encoder.register "mp3.abr" kind_of_encoder make_abr;
+  Lang_encoder.register "mp3.vbr" kind_of_encoder make_vbr
