@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,9 +20,11 @@
 
  *****************************************************************************)
 
-open Term
-open Term.Ground
-open Lang_encoders
+open Value
+open Ground
+
+let kind_of_encoder p =
+  Encoder.audio_video_kind (Lang_encoder.channels_of_params p)
 
 let make params =
   let defaults =
@@ -46,39 +48,39 @@ let make params =
   in
   List.fold_left
     (fun f -> function
-      | "quality", ({ term = Ground (Int i); _ } as t) ->
+      | "quality", `Value { value = Ground (Int i); pos } ->
           (* According to the doc, this should be a value between
            * 0 and 63. *)
           if i < 0 || i > 63 then
-            raise (Error (t, "Theora quality should be in 0..63"));
+            Lang_encoder.raise_error ~pos "Theora quality should be in 0..63";
           { f with Theora_format.bitrate_control = Theora_format.Quality i }
-      | "bitrate", { term = Ground (Int i); _ } ->
+      | "bitrate", `Value { value = Ground (Int i); _ } ->
           { f with Theora_format.bitrate_control = Theora_format.Bitrate i }
-      | "width", ({ term = Ground (Int i); _ } as t) ->
+      | "width", `Value { value = Ground (Int i); pos } ->
           (* According to the doc: must be a multiple of 16, and less than 1048576. *)
           if i mod 16 <> 0 || i >= 1048576 then
-            raise
-              (Error
-                 (t, "invalid frame width value (should be a multiple of 16)"));
+            Lang_encoder.raise_error ~pos
+              "invalid frame width value (should be a multiple of 16)";
           { f with Theora_format.width = lazy i; picture_width = lazy i }
-      | "height", ({ term = Ground (Int i); _ } as t) ->
+      | "height", `Value { value = Ground (Int i); pos } ->
           (* According to the doc: must be a multiple of 16, and less than 1048576. *)
           if i mod 16 <> 0 || i >= 1048576 then
-            raise
-              (Error
-                 (t, "invalid frame height value (should be a multiple of 16)"));
+            Lang_encoder.raise_error ~pos
+              "invalid frame height value (should be a multiple of 16)";
           { f with Theora_format.height = lazy i; picture_height = lazy i }
-      | "picture_width", ({ term = Ground (Int i); _ } as t) ->
+      | "picture_width", `Value { value = Ground (Int i); pos } ->
           (* According to the doc: must not be larger than width. *)
           if i > Lazy.force f.Theora_format.width then
-            raise (Error (t, "picture width must not be larger than width"));
+            Lang_encoder.raise_error ~pos
+              "picture width must not be larger than width";
           { f with Theora_format.picture_width = lazy i }
-      | "picture_height", ({ term = Ground (Int i); _ } as t) ->
+      | "picture_height", `Value { value = Ground (Int i); pos } ->
           (* According to the doc: must not be larger than height. *)
           if i > Lazy.force f.Theora_format.height then
-            raise (Error (t, "picture height must not be larger than height"));
+            Lang_encoder.raise_error ~pos
+              "picture height must not be larger than height";
           { f with Theora_format.picture_height = lazy i }
-      | "picture_x", ({ term = Ground (Int i); _ } as t) ->
+      | "picture_x", `Value { value = Ground (Int i); pos } ->
           (* According to the doc: must be no larger than width-picture_width
            * or 255, whichever is smaller. *)
           if
@@ -88,13 +90,11 @@ let make params =
                 - Lazy.force f.Theora_format.picture_width)
                 255
           then
-            raise
-              (Error
-                 ( t,
-                   "picture x must not be larger than width - picture width or \
-                    255, whichever is smaller" ));
+            Lang_encoder.raise_error ~pos
+              "picture x must not be larger than width - picture width or 255, \
+               whichever is smaller";
           { f with Theora_format.picture_x = i }
-      | "picture_y", ({ term = Ground (Int i); _ } as t) ->
+      | "picture_y", `Value { value = Ground (Int i); pos } ->
           (* According to the doc: must be no larger than width-picture_width
            * and frame_height-pic_height-pic_y must be no larger than 255. *)
           if
@@ -102,29 +102,31 @@ let make params =
             > Lazy.force f.Theora_format.height
               - Lazy.force f.Theora_format.picture_height
           then
-            raise
-              (Error
-                 (t, "picture y must not be larger than height - picture height"));
+            Lang_encoder.raise_error ~pos
+              "picture y must not be larger than height - picture height";
           if Lazy.force f.Theora_format.picture_height - i > 255 then
-            raise
-              (Error
-                 (t, "picture height - picture y must not be larger than 255"));
+            Lang_encoder.raise_error ~pos
+              "picture height - picture y must not be larger than 255";
           { f with Theora_format.picture_y = i }
-      | "aspect_numerator", { term = Ground (Int i); _ } ->
+      | "aspect_numerator", `Value { value = Ground (Int i); _ } ->
           { f with Theora_format.aspect_numerator = i }
-      | "aspect_denominator", { term = Ground (Int i); _ } ->
+      | "aspect_denominator", `Value { value = Ground (Int i); _ } ->
           { f with Theora_format.aspect_denominator = i }
-      | "keyframe_frequency", { term = Ground (Int i); _ } ->
+      | "keyframe_frequency", `Value { value = Ground (Int i); _ } ->
           { f with Theora_format.keyframe_frequency = i }
-      | "vp3_compatible", { term = Ground (Bool i); _ } ->
+      | "vp3_compatible", `Value { value = Ground (Bool i); _ } ->
           { f with Theora_format.vp3_compatible = Some i }
-      | "soft_target", { term = Ground (Bool i); _ } ->
+      | "soft_target", `Value { value = Ground (Bool i); _ } ->
           { f with Theora_format.soft_target = i }
-      | "buffer_delay", { term = Ground (Int i); _ } ->
+      | "buffer_delay", `Value { value = Ground (Int i); _ } ->
           { f with Theora_format.buffer_delay = Some i }
-      | "speed", { term = Ground (Int i); _ } ->
+      | "speed", `Value { value = Ground (Int i); _ } ->
           { f with Theora_format.speed = Some i }
-      | "bytes_per_page", { term = Ground (Int i); _ } ->
+      | "bytes_per_page", `Value { value = Ground (Int i); _ } ->
           { f with Theora_format.fill = Some i }
-      | _, t -> raise (generic_error t))
+      | t -> Lang_encoder.raise_generic_error t)
     defaults params
+
+let () =
+  let make p = Encoder.Ogg { Ogg_format.audio = None; video = Some (make p) } in
+  Lang_encoder.register "theora" kind_of_encoder make
