@@ -20,9 +20,10 @@
 
  *****************************************************************************)
 
-open Term
-open Term.Ground
-open Lang_encoders
+open Value
+open Ground
+
+let kind_of_encoder p = Encoder.audio_kind (Lang_encoder.channels_of_params p)
 
 let make params =
   let defaults =
@@ -39,27 +40,31 @@ let make params =
   let wav =
     List.fold_left
       (fun f -> function
-        | "stereo", { term = Ground (Bool b); _ } ->
+        | "stereo", `Value { value = Ground (Bool b); _ } ->
             { f with Wav_format.channels = (if b then 2 else 1) }
-        | "mono", { term = Ground (Bool b); _ } ->
+        | "mono", `Value { value = Ground (Bool b); _ } ->
             { f with Wav_format.channels = (if b then 1 else 2) }
-        | "", { term = Var s; _ } when String.lowercase_ascii s = "stereo" ->
+        | "", `Value { value = Ground (String s); _ }
+          when String.lowercase_ascii s = "stereo" ->
             { f with Wav_format.channels = 2 }
-        | "", { term = Var s; _ } when String.lowercase_ascii s = "mono" ->
+        | "", `Value { value = Ground (String s); _ }
+          when String.lowercase_ascii s = "mono" ->
             { f with Wav_format.channels = 1 }
-        | "channels", { term = Ground (Int c); _ } ->
+        | "channels", `Value { value = Ground (Int c); _ } ->
             { f with Wav_format.channels = c }
-        | "duration", { term = Ground (Float d); _ } ->
+        | "duration", `Value { value = Ground (Float d); _ } ->
             { f with Wav_format.duration = Some d }
-        | "samplerate", { term = Ground (Int i); _ } ->
+        | "samplerate", `Value { value = Ground (Int i); _ } ->
             { f with Wav_format.samplerate = Lazy.from_val i }
-        | "samplesize", ({ term = Ground (Int i); _ } as t) ->
+        | "samplesize", `Value { value = Ground (Int i); pos } ->
             if i <> 8 && i <> 16 && i <> 24 && i <> 32 then
-              raise (Error (t, "invalid sample size"));
+              Lang_encoder.raise_error ~pos "invalid sample size";
             { f with Wav_format.samplesize = i }
-        | "header", { term = Ground (Bool b); _ } ->
+        | "header", `Value { value = Ground (Bool b); _ } ->
             { f with Wav_format.header = b }
-        | _, t -> raise (generic_error t))
+        | t -> Lang_encoder.raise_generic_error t)
       defaults params
   in
   Encoder.WAV wav
+
+let () = Lang_encoder.register "wav" kind_of_encoder make
