@@ -33,6 +33,7 @@ class accelerate ~kind ~ratio ~randomize source_val =
 
     method self_sync = source#self_sync
     method stype = source#stype
+    method seek = source#seek
 
     method remaining =
       let rem = source#remaining in
@@ -123,5 +124,5 @@ let () =
       let src = f "" in
       let ratio = Lang.to_float_getter (f "ratio") in
       let randomize = Lang.to_float_getter (f "randomize") in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new accelerate ~kind ~ratio ~randomize src)
