@@ -44,9 +44,9 @@ class add ~kind ~renorm ~power (sources : ((unit -> float) * source) list)
     val sources = List.rev sources
 
     method stype =
-      if List.exists (fun (_, s) -> s#stype = Infallible) sources then
-        Infallible
-      else Fallible
+      if List.exists (fun (_, s) -> s#stype = `Infallible) sources then
+        `Infallible
+      else `Fallible
 
     method self_sync =
       ( Lazy.force self_sync_type,
@@ -124,7 +124,7 @@ class add ~kind ~renorm ~power (sources : ((unit -> float) * source) list)
                 Audio.amplify c (AFrame.pcm buffer)
                   (Frame.audio_of_main offset)
                   (Frame.audio_of_main (already - offset))
-              with Frame_content.Invalid -> ());
+              with Content.Invalid -> ());
             if rank > 0 then (
               (* The region grows, make sure it is clean before adding.
                * TODO the same should be done for video. *)
@@ -137,24 +137,28 @@ class add ~kind ~renorm ~power (sources : ((unit -> float) * source) list)
                  (* Add to the main buffer. *)
                  Audio.add (AFrame.pcm buf) offset (AFrame.pcm tmp) offset
                    (already - offset)
-               with Frame_content.Invalid -> ());
+               with Content.Invalid -> ());
 
               try
-                let vbuf = VFrame.yuva420p buf in
-                let vtmp = VFrame.yuva420p tmp in
+                let vbuf = VFrame.data buf in
+                let vtmp = VFrame.data tmp in
                 let ( ! ) = Frame.video_of_main in
                 for i = !offset to !already - 1 do
-                  video_loop rank (Video.get vbuf i) (Video.get vtmp i)
+                  let img =
+                    video_loop rank (Video.Canvas.get vtmp i)
+                      (Video.Canvas.get vbuf i)
+                  in
+                  Video.Canvas.set vbuf i img
                 done
-              with Frame_content.Invalid -> ())
+              with Content.Invalid -> ())
             else (
               try
-                let vbuf = VFrame.yuva420p buf in
+                let vbuf = VFrame.data buf in
                 let ( ! ) = Frame.video_of_main in
                 for i = !offset to !already - 1 do
-                  video_init (Video.get vbuf i)
+                  video_init (Video.Canvas.get vbuf i)
                 done
-              with Frame_content.Invalid -> ());
+              with Content.Invalid -> ());
             (rank + 1, max end_offset already))
           (0, offset) sources
       in
@@ -210,12 +214,12 @@ let () =
           (Error.Invalid_value
              ( List.assoc "weights" p,
                "there should be as many weights as sources" ));
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new add
          ~kind ~renorm ~power
          (List.map2 (fun w s -> (w, s)) weights sources)
          (fun _ -> ())
-         (fun _ buf tmp -> Video.Image.add tmp buf)
+         (fun _ tmp buf -> Video.Canvas.Image.add tmp buf)
         :> Source.source))
 
 let tile_pos n =
@@ -277,7 +281,9 @@ let () =
         let x, y, w, h = tp.(n) in
         let x, y, w, h =
           if proportional then (
-            let sw, sh = (Video.Image.width buf, Video.Image.height buf) in
+            let sw, sh =
+              (Video.Canvas.Image.width buf, Video.Canvas.Image.height buf)
+            in
             if w * sh < sw * h then (
               let h' = sh * w / sw in
               (x, y + ((h - h') / 2), w, h'))
@@ -286,9 +292,13 @@ let () =
               (x + ((w - w') / 2), y, w', h)))
           else (x, y, w, h)
         in
+        let tmp = Video.Canvas.Image.render tmp in
         let tmp' = Video.Image.create w h in
         scale tmp tmp';
-        Video.Image.add tmp' ~x ~y buf
+        let tmp' =
+          Video.Canvas.Image.make ~x ~y ~width:(-1) ~height:(-1) tmp'
+        in
+        Video.Canvas.Image.add tmp' buf
       in
       let video_init buf = video_loop 0 buf buf in
       if List.length weights <> List.length sources then
@@ -296,7 +306,7 @@ let () =
           (Error.Invalid_value
              ( List.assoc "weights" p,
                "there should be as many weights as sources" ));
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new add
          ~kind ~renorm ~power
          (List.map2 (fun w s -> (w, s)) weights sources)
