@@ -25,7 +25,7 @@ open Source
 class available ~kind ~track_sensitive ~override p (source : source) =
   object
     inherit operator ~name:"source.available" kind [source]
-    method stype = Fallible
+    method stype = `Fallible
     method remaining = source#remaining
     method abort_track = source#abort_track
     method seek = source#seek
@@ -77,5 +77,5 @@ let () =
       let override = List.assoc "override" p |> Lang.to_bool in
       let s = Lang.assoc "" 1 p |> Lang.to_source in
       let pred = Lang.assoc "" 2 p |> Lang.to_bool_getter in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new available ~kind ~track_sensitive ~override pred s)
