@@ -54,5 +54,5 @@ let () =
     (fun p ->
       let f v = List.assoc v p in
       let src = Lang.to_source (f "") in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new clip ~kind src)
