@@ -59,5 +59,5 @@ let () =
     (fun p ->
       let f v = List.assoc v p in
       let mu, src = (Lang.to_float (f "mu"), Lang.to_source (f "")) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new compand ~kind src mu)
