@@ -40,8 +40,8 @@ class cross ~kind val_source ~duration_getter ~override_duration ~rms_width
     inherit
       Child_support.base ~check_self_sync:true [val_source] as child_support
 
-    initializer Source.Kind.unify s#kind kind
-    method stype = Source.Fallible
+    initializer Kind.unify s#kind kind
+    method stype = `Fallible
 
     (* This is complicated. crossfade should never be used with [self_sync]
      * sources but we do not have a static way of knowing it at the moment.
@@ -107,9 +107,6 @@ class cross ~kind val_source ~duration_getter ~override_duration ~rms_width
 
     (* Give a default value for the transition source. *)
     val mutable transition_source = None
-
-    (* This is used to track the state of the transition source and switch back
-       to the current source w/o introducing artificial track marks. *)
     val mutable pending_after = Generator.create ()
 
     method private prepare_transition_source s =
@@ -278,7 +275,7 @@ class cross ~kind val_source ~duration_getter ~override_duration ~rms_width
       self#update_cross_length buf_frame start;
       Generator.feed gen_before
         ~metadata:(Frame.get_all_metadata buf_frame)
-        buf_frame.Frame.content
+        (Frame.content buf_frame)
         (Frame.main_of_audio start)
         (Frame.main_of_audio (stop - start));
 
@@ -308,7 +305,7 @@ class cross ~kind val_source ~duration_getter ~override_duration ~rms_width
         in
         Generator.feed gen_after
           ~metadata:(Frame.get_all_metadata buf_frame)
-          buf_frame.Frame.content
+          (Frame.content buf_frame)
           (Frame.main_of_audio start)
           (Frame.main_of_audio (stop - start));
         let after_len = Generator.length gen_after in
@@ -404,6 +401,12 @@ class cross ~kind val_source ~duration_getter ~override_duration ~rms_width
         | `Limit -> -1
         | `After -> (Option.get transition_source)#remaining
 
+    method seek len =
+      match status with
+        | `Before | `Idle -> source#seek len
+        | `Limit -> -1
+        | `After -> (Option.get transition_source)#seek len
+
     method is_ready =
       match status with
         | `Idle | `Before -> source#is_ready
@@ -501,7 +504,7 @@ let () =
       let conservative = Lang.to_bool (List.assoc "conservative" p) in
       let active = Lang.to_bool (List.assoc "active" p) in
       let source = Lang.assoc "" 2 p in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       let c =
         new cross
           ~kind source transition ~conservative ~active ~duration_getter
