@@ -49,6 +49,7 @@ class cue_cut ~kind ~m_cue_in ~m_cue_out ~on_cue_in ~on_cue_out source_val =
     method is_ready = source#is_ready
     method abort_track = source#abort_track
     method self_sync = source#self_sync
+    method seek = source#seek
 
     method remaining =
       let source_remaining = source#remaining in
@@ -226,5 +227,5 @@ let () =
       let on_cue_out = Lang.assoc "on_cue_out" 1 p in
       let on_cue_out () = ignore (Lang.apply on_cue_out []) in
       let s = Lang.assoc "" 1 p in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new cue_cut ~kind ~m_cue_in ~m_cue_out ~on_cue_in ~on_cue_out s)
