@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ class echo ~kind (source : source) delay feedback ping_pong =
       let effect = Option.get effect in
       effect#set_delay (delay ());
       effect#set_feedback (feedback ());
-      effect#process (Audio.sub b offset (position - offset))
+      effect#process b offset (position - offset)
   end
 
 let () =
@@ -90,5 +90,5 @@ let () =
             (Error.Invalid_value (f "feedback", "feedback should be negative"));
         fun () -> Audio.lin_of_dB (feedback ())
       in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new echo ~kind src duration feedback pp)
