@@ -175,5 +175,5 @@ let () =
           Lang.to_int (f "coeffs"),
           Lang.to_source (f "") )
       in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new fir ~kind src freq beta num :> Source.source))
