@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -59,9 +59,9 @@ class flanger ~kind (source : source) delay freq feedback phase =
                 (delay *. (1. -. cos (omega +. (float c *. phase ()))) /. 2.))
             mod past_len
           in
-          past.(c).{past_pos} <- b.(c).{i};
-          b.(c).{i} <-
-            (b.(c).{i} +. (past.(c).{delay} *. feedback)) /. (1. +. feedback)
+          past.(c).(past_pos) <- b.(c).(i);
+          b.(c).(i) <-
+            (b.(c).(i) +. (past.(c).(delay) *. feedback)) /. (1. +. feedback)
         done;
         omega <- omega +. d_omega;
         while omega > 2. *. pi do
@@ -102,5 +102,5 @@ let () =
           Lang.to_source (f "") )
       in
       let feedback () = Audio.lin_of_dB (feedback ()) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new flanger ~kind src duration freq feedback phase :> Source.source))
