@@ -60,10 +60,10 @@ class frei0r_filter ~kind ~name bgra instance params (source : source) =
         | None -> ()
         | Some (rgb, offset, length) ->
             params ();
-            let rgb = Frame_content.Video.get_data rgb in
+            let rgb = Content.Video.get_data rgb in
             for i = offset to offset + length - 1 do
               (* TODO: we could try to be more efficient than converting to/from RGBA32 and swap colors... *)
-              let img = Video.get rgb i in
+              let img = Video.Canvas.render rgb i in
               let img = Image.YUV420.to_RGBA32 img in
               if bgra then Image.RGBA32.swap_rb img;
               let src = Image.RGBA32.data (Image.RGBA32.copy img) in
@@ -71,7 +71,7 @@ class frei0r_filter ~kind ~name bgra instance params (source : source) =
               Frei0r.update1 instance t src dst;
               if bgra then Image.RGBA32.swap_rb img;
               let img = Image.YUV420.of_RGBA32 img in
-              Video.set rgb i img;
+              Video.Canvas.put rgb i img;
               t <- t +. dt
             done
   end
@@ -81,11 +81,12 @@ class frei0r_mixer ~kind ~name bgra instance params (source : source) source2 =
   let dt = 1. /. float fps in
   object (self)
     inherit operator ~name:("frei0r." ^ name) kind [source; source2] as super
+    inherit Source.no_seek
 
     method stype =
       match (source#stype, source2#stype) with
-        | Infallible, Infallible -> Infallible
-        | _ -> Fallible
+        | `Infallible, `Infallible -> `Infallible
+        | _ -> `Fallible
 
     method remaining =
       match (source#remaining, source2#remaining) with
@@ -130,13 +131,14 @@ class frei0r_mixer ~kind ~name bgra instance params (source : source) source2 =
              * because there's usually only one image per video frame. *)
             assert (offset = offset');
             let length = min length length' in
-            let rgb = Frame_content.Video.get_data rgb in
-            let rgb' = Frame_content.Video.get_data rgb' in
+            let rgb = Content.Video.get_data rgb in
+            let rgb' = Content.Video.get_data rgb' in
             for i = offset to offset + length - 1 do
               (* TODO: we could try to be more efficient than converting to/from RGBA32 and swap colors... *)
-              let img = Video.get rgb i in
+              let img = Video.Canvas.render rgb i in
               let img = Image.YUV420.to_RGBA32 img in
-              let img' = Video.get rgb' i in
+              let img' = Video.Canvas.get rgb' i in
+              let img' = Video.Canvas.Image.render img' in
               let img' = Image.YUV420.to_RGBA32 img' in
               if bgra then Image.RGBA32.swap_rb img;
               if bgra then Image.RGBA32.swap_rb img';
@@ -146,7 +148,7 @@ class frei0r_mixer ~kind ~name bgra instance params (source : source) source2 =
               Frei0r.update2 instance t src src' dst;
               if bgra then Image.RGBA32.swap_rb img;
               let img = Image.YUV420.of_RGBA32 img in
-              Video.set rgb i img;
+              Video.Canvas.put rgb i img;
               t <- t +. dt
             done
         | _ -> ()
@@ -157,7 +159,8 @@ class frei0r_source ~kind ~name bgra instance params =
   let dt = 1. /. float fps in
   object
     inherit source ~name:("frei0r." ^ name) kind
-    method stype = Infallible
+    inherit Source.no_seek
+    method stype = `Infallible
     method is_ready = true
     method self_sync = (`Static, false)
     val mutable must_fail = false
@@ -173,15 +176,15 @@ class frei0r_source ~kind ~name bgra instance params =
         params ();
         let start = VFrame.position frame in
         let stop = VFrame.size () in
-        let rgb = VFrame.yuva420p frame in
+        let rgb = VFrame.data frame in
         for i = start to stop - 1 do
-          let img = Video.get rgb i in
+          let img = Video.Canvas.render rgb i in
           let img = Image.YUV420.to_RGBA32 img in
           let dst = Image.RGBA32.data img in
           Frei0r.update0 instance t dst;
           if bgra then Image.RGBA32.swap_rb img;
           let img = Image.YUV420.of_RGBA32 img in
-          Video.set rgb i img;
+          Video.Canvas.put rgb i img;
           t <- t +. dt
         done;
         VFrame.add_break frame stop)
@@ -318,7 +321,7 @@ let register_plugin fname =
       in
       let f v = List.assoc v p in
       let params = params instance p in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       if inputs = 1 then (
         let source = Lang.to_source (f "") in
         new frei0r_filter ~kind ~name bgra instance params source)
