@@ -112,7 +112,7 @@ let () =
     [("", Lang.source_t return_t, None, None)]
     (fun p ->
       let s = Lang.to_source (List.assoc "" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       let s = new insert_metadata ~kind s in
       s)
 
@@ -127,6 +127,7 @@ class replay ~kind meta src =
     method abort_track = src#abort_track
     method remaining = src#remaining
     method self_sync = src#self_sync
+    method seek = src#seek
 
     method private get_frame ab =
       let start = Frame.position ab in
