@@ -35,7 +35,8 @@ let lilv_enabled =
 
 class virtual base ~kind source =
   object
-    inherit operator ~name:"lilv" (Source.Kind.of_kind kind) [source]
+    inherit operator ~name:"lilv" (Kind.of_kind kind) [source]
+    inherit Source.no_seek
     method stype = source#stype
     method remaining = source#remaining
     method seek = source#seek
@@ -47,7 +48,8 @@ class virtual base ~kind source =
 class virtual base_nosource ~kind =
   object
     inherit source ~name:"lilv" kind
-    method stype = Infallible
+    inherit Source.no_seek
+    method stype = `Infallible
     method is_ready = true
     val mutable must_fail = false
     method abort_track = must_fail <- true
@@ -69,8 +71,7 @@ class lilv_mono ~kind (source : source) plugin input output params =
     method wake_up a =
       super#wake_up a;
       let i =
-        Array.init
-          (Frame_content.Audio.channels_of_format self#ctype.Frame.audio)
+        Array.init (Content.Audio.channels_of_format self#ctype.Frame.audio)
           (fun _ ->
             Plugin.instantiate plugin
               (float_of_int (Lazy.force Frame.audio_rate)))
@@ -246,20 +247,20 @@ let params_of_plugin plugin =
               let bounds = ref " (" in
               begin
                 match min with
-                | Some f -> (
-                    match t with
-                      | `Float ->
-                          bounds := Printf.sprintf "%s%.6g <= " !bounds f)
-                | None -> ()
+                  | Some f -> (
+                      match t with
+                        | `Float ->
+                            bounds := Printf.sprintf "%s%.6g <= " !bounds f)
+                  | None -> ()
               end;
               bounds := !bounds ^ "`" ^ Port.symbol p ^ "`";
               begin
                 match max with
-                | Some f -> (
-                    match t with
-                      | `Float ->
-                          bounds := Printf.sprintf "%s <= %.6g" !bounds f)
-                | None -> ()
+                  | Some f -> (
+                      match t with
+                        | `Float ->
+                            bounds := Printf.sprintf "%s <= %.6g" !bounds f)
+                  | None -> ()
               end;
               !bounds ^ ")")
           in
@@ -293,6 +294,9 @@ let register_plugin plugin =
   let inputs, outputs = get_audio_ports plugin in
   let ni = Array.length inputs in
   let no = Array.length outputs in
+  (* Ensure that we support the number of channels. *)
+  ignore (Audio_converter.Channel_layout.layout_of_channels ni);
+  ignore (Audio_converter.Channel_layout.layout_of_channels no);
   let liq_params, params = params_of_plugin plugin in
   let mono = ni = 1 && no = 1 in
   let input_kind = Lang.audio_n ni in
@@ -328,7 +332,7 @@ let register_plugin plugin =
       let params = params p in
       if ni = 0 then
         new lilv_nosource
-          ~kind:(Source.Kind.of_kind (Lang.audio_n no))
+          ~kind:(Kind.of_kind (Lang.audio_n no))
           plugin outputs params
       else if no = 0 then
         (* TODO: can we really use such a type? *)
