@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ open Source
 
 class map_metadata ~kind source rewrite_f insert_missing update strip =
   object (self)
-    inherit operator ~name:"map_metadata" kind [source]
+    inherit operator ~name:"metadata.map" kind [source]
     method stype = source#stype
     method is_ready = source#is_ready
     method remaining = source#remaining
@@ -69,7 +69,7 @@ class map_metadata ~kind source rewrite_f insert_missing update strip =
 let register =
   let kind = Lang.any in
   let return_t = Lang.kind_type_of_kind_format kind in
-  Lang.add_operator "map_metadata"
+  Lang.add_operator "metadata.map"
     [
       ( "",
         Lang.fun_t
@@ -108,5 +108,5 @@ let register =
       let update = Lang.to_bool (List.assoc "update" p) in
       let strip = Lang.to_bool (List.assoc "strip" p) in
       let missing = Lang.to_bool (List.assoc "insert_missing" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new map_metadata ~kind source f missing update strip)
