@@ -59,5 +59,5 @@ let () =
     (fun p ->
       let f = to_fun_float (Lang.assoc "" 1 p) in
       let src = Lang.to_source (Lang.assoc "" 2 p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new map ~kind src f)
