@@ -23,17 +23,12 @@
 class on_track ~kind f s =
   object
     inherit Source.operator ~name:"on_track" kind [s]
-
     method stype = s#stype
-
     method is_ready = s#is_ready
-
     method abort_track = s#abort_track
-
     method remaining = s#remaining
-
     method seek n = s#seek n
-
+    method self_sync = s#self_sync
     val mutable called = false
 
     method private get_frame ab =
@@ -42,20 +37,21 @@ class on_track ~kind f s =
       if not called then begin
         let m =
           match Frame.get_metadata ab p with
-            | None ->
-                Lang.list ~t:(Lang.product_t Lang.string_t Lang.string_t) []
+            | None -> Lang.list []
             | Some m -> Lang.metadata m
         in
-        ignore (Lang.apply ~t:Lang.unit_t f [("", m)]);
+        ignore (Lang.apply f [("", m)]);
         called <- true
       end;
       if Frame.is_partial ab then called <- false
   end
 
 let () =
-  let kind = Lang.univ_t 1 in
-  Lang.add_operator "on_track"
+  let kind = Lang.any in
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "source.on_track"
     [
+      ("", Lang.source_t return_t, None, None);
       ( "",
         Lang.fun_t
           [
@@ -68,11 +64,10 @@ let () =
            corresponding metadata as argument. If there is no metadata at the \
            beginning of track, the empty list is passed. That function should \
            be fast because it is executed in the main streaming thread." );
-      ("", Lang.source_t kind, None, None);
     ]
-    ~category:Lang.TrackProcessing ~descr:"Call a given handler on new tracks."
-    ~kind:(Lang.Unconstrained kind)
-    (fun p kind ->
-      let f = Lang.assoc "" 1 p in
-      let s = Lang.to_source (Lang.assoc "" 2 p) in
+    ~category:`Track ~descr:"Call a given handler on new tracks." ~return_t
+    (fun p ->
+      let s = Lang.assoc "" 1 p |> Lang.to_source in
+      let f = Lang.assoc "" 2 p in
+      let kind = Kind.of_kind kind in
       new on_track ~kind f s)
