@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -68,7 +68,8 @@ class pipe ~kind ~replay_delay ~data_len ~process ~bufferize ~log_overfull ~max
     val mutable samplerate = Frame.audio_of_seconds 1.
 
     (* Filled in by wake_up. *)
-    val mutable converter = fun _ -> assert false
+    val mutable converter = fun _ _ _ -> assert false
+    method self_sync = source#self_sync
 
     method private header =
       Bytes.unsafe_of_string
@@ -93,16 +94,15 @@ class pipe ~kind ~replay_delay ~data_len ~process ~bufferize ~log_overfull ~max
               Decoder_utils.from_iff ~format:`Wav ~channels:self#audio_channels
                 ~samplesize)
           ();
-        `Reschedule Tutils.Non_blocking)
+        `Reschedule `Non_blocking)
       else (
         let len = pull bytes 0 Utils.pagesize in
-        let data = converter (Bytes.unsafe_to_string (Bytes.sub bytes 0 len)) in
-        let data = resampler ~samplerate data in
-        let len = Audio.length data in
+        let data = converter bytes 0 len in
+        let data, ofs, len = resampler ~samplerate data 0 (Audio.length data) in
         let buffered = Generator.length abg in
         Generator.put_audio abg
-          (Frame_content.Audio.lift_data data)
-          0 (Frame.main_of_audio len);
+          (Content.Audio.lift_data data)
+          (Frame.main_of_audio ofs) (Frame.main_of_audio len);
         let to_replay =
           Tutils.mutexify mutex
             (fun () ->
@@ -127,13 +127,13 @@ class pipe ~kind ~replay_delay ~data_len ~process ~bufferize ~log_overfull ~max
         in
         begin
           match to_replay with
-          | -1, _ -> ()
-          | _, `Break_and_metadata m ->
-              Generator.add_metadata abg m;
-              Generator.add_break abg
-          | _, `Metadata m -> Generator.add_metadata abg m
-          | _, `Break -> Generator.add_break abg
-          | _ -> ()
+            | -1, _ -> ()
+            | _, `Break_and_metadata m ->
+                Generator.add_metadata abg m;
+                Generator.add_break abg
+            | _, `Metadata m -> Generator.add_metadata abg m
+            | _, `Break -> Generator.add_break abg
+            | _ -> ()
         end;
         if abg_max_len < buffered + len then
           `Delay (Frame.seconds_of_audio (buffered + len - abg_max_len))
@@ -141,7 +141,7 @@ class pipe ~kind ~replay_delay ~data_len ~process ~bufferize ~log_overfull ~max
 
     val mutable handler = None
     val to_write = Queue.create ()
-    method stype = Source.Fallible
+    method stype = `Fallible
 
     method private get_handler =
       match handler with Some h -> h | None -> raise Process_handler.Finished
@@ -167,7 +167,7 @@ class pipe ~kind ~replay_delay ~data_len ~process ~bufferize ~log_overfull ~max
         let slen_of_len len = 2 * len * Array.length buf in
         let slen = slen_of_len blen in
         let sbuf = Bytes.create slen in
-        Audio.S16LE.of_audio buf sbuf 0;
+        Audio.S16LE.of_audio buf 0 sbuf 0 blen;
         let metadata =
           List.sort
             (fun (pos, _) (pos', _) -> compare pos pos')
@@ -263,7 +263,7 @@ class pipe ~kind ~replay_delay ~data_len ~process ~bufferize ~log_overfull ~max
         Some
           (Process_handler.run ~on_stop:self#on_stop ~on_start:self#on_start
              ~on_stdout:self#on_stdout ~on_stdin:self#on_stdin
-             ~priority:Tutils.Blocking ~on_stderr:self#on_stderr ~log process)
+             ~priority:`Blocking ~on_stderr:self#on_stderr ~log process)
 
     method abort_track = source#abort_track
 
@@ -362,7 +362,7 @@ let () =
       let data_len =
         match data_len with None -> -1 | Some v -> Lang.to_int v
       in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new pipe
          ~kind ~replay_delay ~data_len ~bufferize ~max ~log_overfull ~restart
          ~restart_on_error ~process src
