@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ class sequence ~kind ?(merge = false) sources =
         match sources with hd :: _ -> snd hd#self_sync | [] -> false )
 
     method stype =
-      match List.rev sources with hd :: _ -> hd#stype | [] -> Source.Fallible
+      match List.rev sources with hd :: _ -> hd#stype | [] -> `Fallible
 
     method private wake_up activation =
       List.iter
@@ -68,6 +68,8 @@ class sequence ~kind ?(merge = false) sources =
         List.fold_left ( + ) 0 (List.map (fun s -> s#remaining) seq_sources))
       else (List.hd seq_sources)#remaining
 
+    method seek len = match seq_sources with s :: _ -> s#seek len | [] -> 0
+
     method abort_track =
       if merge then (
         match List.rev seq_sources with
@@ -110,6 +112,7 @@ class merge_tracks ~kind source =
     method abort_track = source#abort_track
     method remaining = -1
     method self_sync = source#self_sync
+    method seek = source#seek
 
     method private get_frame buf =
       source#get buf;
@@ -142,7 +145,7 @@ let () =
        which is played as much as available."
     ~return_t:k
     (fun p ->
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new sequence
         ~kind
         ~merge:(Lang.to_bool (List.assoc "merge" p))
@@ -159,5 +162,5 @@ let () =
        as one big track, so `on_track()` will not trigger for example."
     ~return_t:k
     (fun p ->
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new merge_tracks ~kind (Lang.to_source (List.assoc "" p)))
