@@ -78,5 +78,5 @@ let () =
       let random = AFrame.duration () *. random in
       let freeze = Lang.to_float (List.assoc "freeze" p) in
       let src = Lang.to_source (List.assoc "" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new map ~kind src delay random freeze)
