@@ -44,6 +44,7 @@ class soundtouch ~kind source_val rate tempo pitch =
     method stype = source#stype
     method self_sync = source#self_sync
     method is_ready = source#is_ready
+    method seek = source#seek
     method remaining = -1
 
     method abort_track =
@@ -65,7 +66,7 @@ class soundtouch ~kind source_val rate tempo pitch =
         let buf = Audio.create self#audio_channels available in
         ignore (Soundtouch.get_samples_ni st buf 0 available);
         Generator.put_audio abg
-          (Frame_content.Audio.lift_data buf)
+          (Content.Audio.lift_data buf)
           0
           (Frame.main_of_audio available));
       if AFrame.is_partial databuf then Generator.add_break abg;
@@ -121,5 +122,5 @@ let () =
       let tempo = Lang.to_float_getter (f "tempo") in
       let pitch = Lang.to_float_getter (f "pitch") in
       let s = f "" in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new soundtouch ~kind s rate tempo pitch :> Source.source))
