@@ -38,7 +38,7 @@ class bpm ~kind (source : source) =
       bpm <-
         Some
           (Soundtouch.BPM.make
-             (Frame_content.Audio.channels_of_format self#ctype.Frame.audio)
+             (Content.Audio.channels_of_format self#ctype.Frame.audio)
              (Lazy.force Frame.audio_rate))
 
     method private get_frame buf =
@@ -71,5 +71,5 @@ let () =
       ]
     (fun p ->
       let s = Lang.to_source (List.assoc "" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new bpm ~kind s)
