@@ -51,9 +51,12 @@ class still_frame ~name ~kind (source : source) =
             let v = VFrame.get_content buf source in
             match v with
               | Some (v, off, _) ->
-                  let v = Frame_content.Video.get_data v in
-                  let i = Video.get v off in
-                  let i = i |> Image.YUV420.to_RGBA32 |> Image.RGBA32.to_BMP in
+                  let v = Content.Video.get_data v in
+                  let i = Video.Canvas.get v off in
+                  let i =
+                    i |> Video.Canvas.Image.render |> Image.YUV420.to_RGBA32
+                    |> Image.RGBA32.to_BMP
+                  in
                   let oc = open_out f in
                   output_string oc i;
                   close_out oc;
@@ -64,7 +67,7 @@ class still_frame ~name ~kind (source : source) =
 let () =
   let k = Lang.video_yuva420p in
   let return_t = Lang.kind_type_of_kind_format k in
-  let kind = Source.Kind.of_kind k in
+  let kind = Kind.of_kind k in
   let name = "video.still_frame" in
   Lang.add_operator name
     [("", Lang.source_t return_t, None, None)]
@@ -78,7 +81,9 @@ let () =
           ([], Lang.fun_t [(false, "", Lang.string_t)] Lang.unit_t),
           "Save current image, argument is the file name to save to.",
           fun s ->
-            Lang.val_fun [("", "", None)] (fun p ->
+            Lang.val_fun
+              [("", "", None)]
+              (fun p ->
                 s#save (List.assoc "" p |> Lang.to_string);
                 Lang.unit) );
       ]
