@@ -36,6 +36,7 @@ class fade_in ~kind ?(meta = "liq_video_fade_in") duration fader fadefun source
     method abort_track = source#abort_track
     method remaining = source#remaining
     method self_sync = source#self_sync
+    method seek = source#seek
     val mutable state = `Idle
 
     method private get_frame ab =
@@ -69,11 +70,11 @@ class fade_in ~kind ?(meta = "liq_video_fade_in") duration fader fadefun source
       if Frame.is_partial ab then state <- `Idle;
       match video_content with
         | Some (rgb, off, len) ->
-            let rgb = Frame_content.Video.get_data rgb in
+            let rgb = Content.Video.get_data rgb in
             if count < length then
               for i = 0 to min (len - 1) (length - count - 1) do
                 let m = fade (count + i) in
-                fadefun (Video.get rgb (off + i)) m
+                fadefun (Video.Canvas.get rgb (off + i)) m
               done;
             if state <> `Idle then
               state <- `Play (fade, fadefun, length, count + len)
@@ -88,6 +89,7 @@ class fade_out ~kind ?(meta = "liq_video_fade_out") duration fader fadefun
     method stype = source#stype
     method abort_track = source#abort_track
     method self_sync = source#self_sync
+    method seek = source#seek
 
     (* Fade-out length (in video frames) for the current track.
      * The value is set at the beginning of every track, depending on metadata. *)
@@ -129,10 +131,10 @@ class fade_out ~kind ?(meta = "liq_video_fade_out") duration fader fadefun
             (* Process the buffer *)
             match if n >= 0 && n < length then Some n else None with
               | Some n ->
-                  let rgb = Frame_content.Video.get_data rgb in
+                  let rgb = Content.Video.get_data rgb in
                   for i = 0 to len - 1 do
                     let m = fade (n - i) in
-                    fadefun (Video.get rgb (off + i)) m
+                    fadefun (Video.Canvas.get rgb (off + i)) m
                   done
               | None -> ())
         | _ -> ()
@@ -168,46 +170,49 @@ let proto =
   ]
 
 let rec transition_of_string p transition =
-  let translate img x y =
-    let tmp = Video.Image.copy img in
-    Image.YUV420.fill_alpha img 0;
-    Video.Image.add tmp ~x ~y img
-  in
+  let translate img dx dy = Video.Canvas.Image.translate dx dy img in
   let ifm n a = int_of_float (float_of_int n *. a) in
   match transition with
-    | "fade" -> fun () img t -> Video.Image.fill_alpha img (ifm 256 t)
+    | "fade" ->
+        fun () img t ->
+          Video.Canvas.Image.iter
+            (fun img -> Image.YUV420.fill_alpha img (ifm 256 t))
+            img
     | "slide_left" ->
         fun () buf t ->
-          translate buf (ifm (Lazy.force Frame.video_width) (t -. 1.)) 0
+          translate buf (ifm (Video.Canvas.Image.width buf) (t -. 1.)) 0
     | "slide_right" ->
         fun () buf t ->
-          translate buf (ifm (Lazy.force Frame.video_width) (1. -. t)) 0
+          translate buf (ifm (Video.Canvas.Image.width buf) (1. -. t)) 0
     | "slide_up" ->
         fun () buf t ->
-          translate buf 0 (ifm (Lazy.force Frame.video_height) (1. -. t))
+          translate buf 0 (ifm (Video.Canvas.Image.height buf) (1. -. t))
     | "slide_down" ->
         fun () buf t ->
-          translate buf 0 (ifm (Lazy.force Frame.video_height) (t -. 1.))
+          translate buf 0 (ifm (Video.Canvas.Image.height buf) (t -. 1.))
     | "grow" ->
         fun () img t ->
-          let w = Video.Image.width img in
-          let h = Video.Image.height img in
+          let w = Video.Canvas.Image.width img in
+          let h = Video.Canvas.Image.height img in
           let w' = ifm w t in
           let h' = ifm h t in
-          let tmp = Video.Image.create w' h' in
-          Video.Image.scale img tmp;
-          Video.Image.fill_alpha img 0;
+          let img = Video.Canvas.Image.render img in
+          let out = Video.Image.create w' h' in
+          Image.YUV420.scale img out;
           let x = (w - w') / 2 in
           let y = (h - h') / 2 in
-          Video.Image.add ~x ~y tmp img
+          Video.Canvas.Image.create w h |> Video.Canvas.Image.translate x y
     | "disc" ->
-        let w = Lazy.force Frame.video_width in
-        let h = Lazy.force Frame.video_height in
-        let r_max =
-          int_of_float (sqrt (float_of_int ((w * w) + (h * h)))) / 2
-        in
         fun () buf t ->
-          Image.YUV420.disk_alpha buf (w / 2) (h / 2) (ifm r_max t)
+          let w = Video.Canvas.Image.width buf in
+          let h = Video.Canvas.Image.height buf in
+          let r_max =
+            int_of_float (sqrt (float_of_int ((w * w) + (h * h)))) / 2
+          in
+          Video.Canvas.Image.iter
+            (fun buf ->
+              Image.YUV420.disk_alpha buf (w / 2) (h / 2) (ifm r_max t))
+            buf
     | "random" ->
         let trans =
           [|
@@ -284,7 +289,7 @@ let () =
     (fun p ->
       let d, f, t, s = extract p in
       let meta = Lang.to_string (List.assoc "override" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new fade_in ~kind ~meta d f t s);
   Lang.add_operator "video.fade.out"
     (( "override",
@@ -299,5 +304,5 @@ let () =
     (fun p ->
       let d, f, t, s = extract p in
       let meta = Lang.to_string (List.assoc "override" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new fade_out ~kind ~meta d f t s)
