@@ -1,5 +1,5 @@
 (*****************************************************************************
-  
+
   Liquidsoap, a programmable audio stream generator.
   Copyright 2003-2022 Savonet team
 
@@ -7,17 +7,17 @@
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
-  
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details, fully stated in the COPYING
   file at the root of the liquidsoap distribution.
-  
+
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
-  
+
  *****************************************************************************)
 
 (** Output to an harbor server. *)
@@ -33,7 +33,7 @@ module Make (T : T) = struct
     type priority = Tutils.priority
 
     let scheduler = Tutils.scheduler
-    let priority = Tutils.Non_blocking
+    let priority = `Non_blocking
   end
 
   module Duppy_m = Duppy.Monad.Mutex.Factory (Mutex_control)
@@ -58,11 +58,11 @@ module Make (T : T) = struct
   module M = Icecast_utils.Icecast_v (Icecast)
   open M
 
-  (* Max total length for ICY metadata is 255*16 
-   * Format is: "StreamTitle='%s';StreamUrl='%s'" 
+  (* Max total length for ICY metadata is 255*16
+   * Format is: "StreamTitle='%s';StreamUrl='%s'"
    * "StreamTitle='';"; is 15 chars long, "StreamUrl='';"
-   * is 13 chars long, leaving 4052 chars remaining. 
-   * Splitting those in: 
+   * is 13 chars long, leaving 4052 chars remaining.
+   * Splitting those in:
    * * max title length = 3852
    * * max url length = 200 *)
   let max_title = 3852
@@ -255,7 +255,7 @@ module Make (T : T) = struct
 
   let rec client_task c =
     let __pa_duppy_0 =
-      Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking c.handler
+      Duppy.Monad.Io.exec ~priority:`Maybe_blocking c.handler
         (Tutils.mutexify c.mutex
            (fun () ->
              let buflen = Strings.Mutable.length c.buffer in
@@ -275,10 +275,10 @@ module Make (T : T) = struct
                  (fun () -> Duppy_m.unlock c.condition_m))
           else
             Duppy.Monad.Io.write ?timeout:(Some c.timeout)
-              ~priority:Tutils.Non_blocking c.handler (Strings.to_bytes data))
+              ~priority:`Non_blocking c.handler (Strings.to_bytes data))
           (fun () ->
             let __pa_duppy_0 =
-              Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking c.handler
+              Duppy.Monad.Io.exec ~priority:`Maybe_blocking c.handler
                 (let ret = Tutils.mutexify c.mutex (fun () -> c.state) () in
                  Duppy.Monad.return ret)
             in
@@ -412,7 +412,7 @@ module Make (T : T) = struct
       (** File descriptor where to dump. *)
       inherit
         Output.encoded
-          ~content_kind:(Source.Kind.of_kind kind) ~output_kind:T.source_name
+          ~content_kind:(Kind.of_kind kind) ~output_kind:T.source_name
             ~infallible ~autostart ~on_start ~on_stop ~name:mount source
 
       val mutable dump = None
@@ -518,7 +518,7 @@ module Make (T : T) = struct
               || auth_function <> None
              then (
               let default_user = Option.value default_user ~default:"" in
-              Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking handler
+              Duppy.Monad.Io.exec ~priority:`Maybe_blocking handler
                 (Harbor.http_auth_check ~args ~login:(default_user, login) s
                    headers))
              else Duppy.Monad.return ())
@@ -529,7 +529,7 @@ module Make (T : T) = struct
                    client.state <- Done;
                    Harbor.reply s))
           (fun () ->
-            Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking handler
+            Duppy.Monad.Io.exec ~priority:`Maybe_blocking handler
               (Harbor.relayed reply (fun () ->
                    self#log#info "Client %s connected" ip;
                    Tutils.mutexify clients_m
