@@ -42,16 +42,24 @@ let hls_proto kind =
         ("position", "position", None);
         ("extname", "extname", None);
         ("", "", None);
-      ] (fun p ->
+      ]
+      (fun p ->
         let position = Lang.to_int (List.assoc "position" p) in
         let extname = Lang.to_string (List.assoc "extname" p) in
         let sname = Lang.to_string (List.assoc "" p) in
         Lang.string (Printf.sprintf "%s_%d.%s" sname position extname))
   in
   let stream_info_t =
-    let video_size_t = Lang.nullable_t (Lang.product_t Lang.int_t Lang.int_t) in
-    Lang.product_t Lang.string_t
-      (Lang.tuple_t [Lang.int_t; Lang.string_t; Lang.string_t; video_size_t])
+    let info_t =
+      Lang.record_t
+        [
+          ("bandwidth", Lang.int_t);
+          ("codecs", Lang.string_t);
+          ("extname", Lang.string_t);
+          ("video_size", Lang.nullable_t (Lang.product_t Lang.int_t Lang.int_t));
+        ]
+    in
+    Lang.product_t Lang.string_t info_t
   in
   Output.proto
   @ [
@@ -115,6 +123,10 @@ let hls_proto kind =
           "Location of the configuration file used to restart the output. \
            Relative paths are assumed to be with regard to the directory for \
            generated file." );
+      ( "perms",
+        Lang.int_t,
+        Some (Lang.int 0o755),
+        Some "Default directory rights if created. Default: `0o755`" );
       ( "strict_persist",
         Lang.bool_t,
         Some (Lang.bool false),
@@ -242,12 +254,14 @@ class hls_output p =
   let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
   let prefix = Lang.to_string (List.assoc "prefix" p) in
   let directory = Lang.to_string (Lang.assoc "" 1 p) in
+  let perm = Lang.to_int (List.assoc "perm" p) in
   let () =
     if (not (Sys.file_exists directory)) || not (Sys.is_directory directory)
-    then
-      raise
-        (Error.Invalid_value
-           (Lang.assoc "" 1 p, "The target directory does not exist"))
+    then (
+      try Utils.mkdir ~perm directory
+      with exn ->
+        let bt = Printexc.get_raw_backtrace () in
+        Lang.raise_as_runtime ~bt ~kind:"file" exn)
   in
   let persist_at =
     Option.map
@@ -295,12 +309,10 @@ class hls_output p =
     List.map
       (fun el ->
         let name, specs = Lang.to_product el in
-        let bandwidth, codecs, extname, video_size =
-          match Lang.to_tuple specs with
-            | [bandwidth; codecs; extname; video_size] ->
-                (bandwidth, codecs, extname, video_size)
-            | _ -> assert false
-        in
+        let bandwidth = Value.invoke specs "bandwidth" in
+        let codecs = Value.invoke specs "codecs" in
+        let extname = Value.invoke specs "extname" in
+        let video_size = Value.invoke specs "video_size" in
         ( Lang.to_string name,
           ( lazy (Lang.to_int bandwidth),
             lazy (Lang.to_string codecs),
@@ -421,9 +433,7 @@ class hls_output p =
     segments_per_playlist + Lang.to_int (List.assoc "segments_overhead" p)
   in
   let file_perm = Lang.to_int (List.assoc "perm" p) in
-  let kind =
-    Source.Kind.of_kind (Encoder.kind_of_format (List.hd streams).format)
-  in
+  let kind = Kind.of_kind (Encoder.kind_of_format (List.hd streams).format) in
   object (self)
     inherit
       Output.encoded
@@ -497,6 +507,15 @@ class hls_output p =
       let filename =
         segment_name ~position:s.position ~extname:s.extname s.name
       in
+      let directory = Filename.dirname filename in
+      let () =
+        if (not (Sys.file_exists directory)) || not (Sys.is_directory directory)
+        then (
+          try Utils.mkdir ~perm directory
+          with exn ->
+            let bt = Printexc.get_raw_backtrace () in
+            Lang.raise_as_runtime ~bt ~kind:"file" exn)
+      in
       let out_channel = self#open_out filename in
       Strings.iter (output_substring out_channel) s.encoder.Encoder.header;
       let discontinuous = state = `Restarted in
@@ -549,7 +568,7 @@ class hls_output p =
       in
       let discontinuity_sequence, media_sequence =
         match segments with
-          | { current_discontinuity; id } :: _ -> (current_discontinuity, id)
+          | { current_discontinuity; id } :: _ -> (current_discontinuity, id - 1)
           | [] -> (0, 0)
       in
       let filename = self#playlist_name s in
@@ -659,7 +678,7 @@ class hls_output p =
       self#log#info "Reading state file at %s.." (Utils.quote_string persist_at);
       let fd = open_out_bin persist_at in
       let streams =
-        `List
+        `Tuple
           (List.map
              (fun { name; position; discontinuity_count } ->
                `Assoc
@@ -673,16 +692,17 @@ class hls_output p =
       let segments =
         `Assoc
           (List.map
-             (fun (s, l) -> (s, `List (List.map json_of_segment !l)))
+             (fun (s, l) -> (s, `Tuple (List.map json_of_segment !l)))
              segments)
       in
       output_string fd
-        (JSON.to_string (`Assoc [("streams", streams); ("segments", segments)]));
+        (Json.to_string ~compact:false ~json5:false
+           (`Assoc [("streams", streams); ("segments", segments)]));
       close_out fd
 
     method private read_state persist_at =
       let saved_streams, saved_segments =
-        match JSON.from_string (Utils.read_all persist_at) with
+        match Json.from_string (Utils.read_all persist_at) with
           | `Assoc [("streams", streams); ("segments", segments)] ->
               (streams, segments)
           | _ -> raise Invalid_state
@@ -698,14 +718,14 @@ class hls_output p =
                 ] ->
                 (name, position, discontinuity_count)
             | _ -> raise Invalid_state)
-          (match saved_streams with `List l -> l | _ -> raise Invalid_state)
+          (match saved_streams with `Tuple l -> l | _ -> raise Invalid_state)
       in
       let saved_segments =
         match saved_segments with
           | `Assoc l ->
               List.map
                 (function
-                  | s, `List segments ->
+                  | s, `Tuple segments ->
                       (s, ref (List.map segment_of_json segments))
                   | _ -> raise Invalid_state)
                 l
@@ -724,7 +744,7 @@ class hls_output p =
         ({ extname; name; init_position } as s) =
       match init with
         | None -> s.init_state <- `No_init
-        | Some data ->
+        | Some data when not (Strings.is_empty data) ->
             let init_filename =
               segment_name ~position:init_position ~extname name
             in
@@ -733,6 +753,7 @@ class hls_output p =
             self#close_out ~filename:init_filename oc;
             segment.init_filename <- Some init_filename;
             s.init_state <- `Has_init init_filename
+        | Some _ -> raise Encoder.Not_enough_data
 
     method encode frame ofs len =
       List.map
