@@ -403,7 +403,7 @@ class output ~kind p =
   object (self)
     inherit
       Output.encoded
-        ~content_kind:(Source.Kind.of_kind kind) ~output_kind:"output.icecast"
+        ~content_kind:(Kind.of_kind kind) ~output_kind:"output.icecast"
           ~infallible ~autostart ~on_start ~on_stop ~name source
 
     (** In this operator, we don't exactly follow the start/stop
@@ -507,7 +507,7 @@ class output ~kind p =
         if delay >= 0. then (
           (* Ask for a restart after [restart_time]. *)
           self#icecast_stop;
-          restart_time <- Unix.time () +. delay;
+          restart_time <- Unix.gettimeofday () +. delay;
           self#log#important "Will try to reconnect in %.02f seconds." delay)
         else Printexc.raise_with_backtrace e bt
 
@@ -530,8 +530,8 @@ class output ~kind p =
       assert (Cry.get_status connection = Cry.Disconnected);
       begin
         match dumpfile with
-        | Some f -> dump <- Some (open_out_bin f)
-        | None -> ()
+          | Some f -> dump <- Some (open_out_bin f)
+          | None -> ()
       end;
       let display_mount =
         match mount with
@@ -590,11 +590,11 @@ class output ~kind p =
       encoder <- None;
       begin
         match Cry.get_status connection with
-        | Cry.Disconnected -> ()
-        | Cry.Connected _ ->
-            self#log#important "Closing connection...";
-            Cry.close connection;
-            on_disconnect ()
+          | Cry.Disconnected -> ()
+          | Cry.Connected _ ->
+              self#log#important "Closing connection...";
+              Cry.close connection;
+              on_disconnect ()
       end;
       match dump with Some f -> close_out f | None -> ()
   end
