@@ -26,7 +26,7 @@
 val proto : (string * Lang.t * Lang.value option * string option) list
 
 class virtual output :
-  content_kind:Source.Kind.t
+  content_kind:Kind.t
   -> output_kind:string
   -> ?name:string
   -> infallible:bool
@@ -36,7 +36,7 @@ class virtual output :
   -> bool
   -> object
        inherit Source.active_source
-       method stype : Source.source_t
+       method stype : [ `Fallible | `Infallible ]
        method self_sync : Source.self_sync
        method remaining : int
        method output : unit
@@ -45,6 +45,8 @@ class virtual output :
        method is_ready : bool
        method state : Start_stop.state
        method transition_to : Start_stop.state -> unit
+       method seek : int -> int
+       method private video_dimensions : int * int
        method private add_metadata : Request.metadata -> unit
        method private metadata_queue : Request.metadata Queue.t
        method private reset : unit
@@ -58,7 +60,7 @@ class virtual output :
 val meth : (string * Lang.scheme * string * (output -> Lang.value)) list
 
 class virtual encoded :
-  content_kind:Source.Kind.t
+  content_kind:Kind.t
   -> output_kind:string
   -> name:string
   -> infallible:bool
@@ -85,7 +87,7 @@ class dummy :
   -> on_start:(unit -> unit)
   -> on_stop:(unit -> unit)
   -> autostart:bool
-  -> kind:Source.Kind.t
+  -> kind:Kind.t
   -> Lang.value
   -> object
        inherit output
