@@ -79,10 +79,14 @@ let url_proto kind =
   Output.proto
   @ [
       ("url", Lang.string_t, None, Some "Url to output to.");
-      ( "restart_on_error",
-        Lang.bool_t,
-        Some (Lang.bool true),
-        Some "Restart output on errors" );
+      ( "restart_delay",
+        Lang.nullable_t Lang.float_t,
+        Some (Lang.float 2.),
+        Some "If not `null`, restart output on errors after the given delay." );
+      ( "on_error",
+        Lang.fun_t [(false, "", Lang.error_t)] Lang.unit_t,
+        Some (Lang.val_fun [("", "", None)] (fun _ -> Lang.unit)),
+        Some "Callback executed when an error occurs." );
       ( "self_sync",
         Lang.bool_t,
         Some (Lang.bool false),
@@ -104,25 +108,49 @@ class url_output p =
            (format_val, "Encoding format does not support output url!"))
   in
   let format = Encoder.with_url_output format url in
-  let kind = Source.Kind.of_kind (Encoder.kind_of_format format) in
+  let kind = Kind.of_kind (Encoder.kind_of_format format) in
   let source = Lang.assoc "" 2 p in
-  let restart_on_error = Lang.to_bool (List.assoc "restart_on_error" p) in
+  let restart_delay =
+    Lang.to_valued_option Lang.to_float (List.assoc "restart_delay" p)
+  in
+  let on_error = List.assoc "on_error" p in
+  let on_error ~bt exn =
+    let error = Lang.runtime_error_of_exception ~bt ~kind:"output" exn in
+    ignore (Lang.apply on_error [("", Lang.error error)])
+  in
   let self_sync = Lang.to_bool (List.assoc "self_sync" p) in
   let name = "output.url" in
   object (self)
     inherit base p ~kind ~source ~name as super
     method private encoder_factory = encoder_factory ~format format_val
+    val mutable restart_time = 0.
 
-    method encode frame ofs len =
-      try super#encode frame ofs len
-      with e when restart_on_error ->
-        let bt = Printexc.get_backtrace () in
-        Utils.log_exception ~log:self#log ~bt
-          (Printf.sprintf "Error when encoding data: %s" (Printexc.to_string e));
-        self#stop;
-        self#start;
-        self#encode frame ofs len
+    method private restartify : 'a. default:'a -> (unit -> 'a) -> 'a =
+      fun ~default fn ->
+        try
+          match encoder with
+            | None when restart_time <= Unix.gettimeofday () ->
+                super#start;
+                fn ()
+            | None -> default
+            | Some _ -> fn ()
+        with exn -> (
+          let bt = Printexc.get_raw_backtrace () in
+          Utils.log_exception ~log:self#log
+            ~bt:(Printexc.raw_backtrace_to_string bt)
+            (Printf.sprintf "Error while streaming: %s" (Printexc.to_string exn));
+          on_error ~bt exn;
+          match restart_delay with
+            | None -> Printexc.raise_with_backtrace exn bt
+            | Some delay ->
+                super#stop;
+                restart_time <- Unix.gettimeofday () +. delay;
+                self#log#important "Will try to reconnect in %.02f seconds."
+                  delay;
+                default)
 
+    method start = self#restartify ~default:() (fun () -> super#start)
+    method output = self#restartify ~default:() (fun () -> super#output)
     method write_pipe _ _ _ = ()
     method self_sync = (`Static, self_sync)
   end
@@ -244,7 +272,7 @@ class virtual piped_output ~kind p =
       else super#insert_metadata m
   end
 
-(** Out channel virtual class: takes care 
+(** Out channel virtual class: takes care
   * of current out channel and writing to
   * it. *)
 let chan_proto kind arg_doc =
@@ -273,7 +301,7 @@ class virtual ['a] chan_output p =
         if flush then self#flush chan
       with Sys_error e ->
         let bt = Printexc.get_raw_backtrace () in
-        Runtime_error.error ~bt ~message:e "system"
+        Runtime_error.raise ~pos:(Lang.pos p) ~bt ~message:e "system"
 
     method close_pipe =
       self#close_chan (Option.get chan);
@@ -318,7 +346,7 @@ class virtual ['a] file_output_base p =
         fd
       with Sys_error e ->
         let bt = Printexc.get_raw_backtrace () in
-        Runtime_error.error ~bt ~message:e "system"
+        Runtime_error.raise ~pos:(Lang.pos p) ~bt ~message:e "system"
 
     method virtual close_out : 'a -> unit
 
@@ -329,7 +357,7 @@ class virtual ['a] file_output_base p =
         current_filename <- None
       with Sys_error e ->
         let bt = Printexc.get_raw_backtrace () in
-        Runtime_error.error ~bt ~message:e "system"
+        Runtime_error.raise ~pos:(Lang.pos p) ~bt ~message:e "system"
 
     method private on_close = on_close
   end
@@ -410,7 +438,7 @@ let file_proto kind =
 let new_file_output p =
   let format_val = Lang.assoc "" 1 p in
   let format = Lang.to_format format_val in
-  let kind = Source.Kind.of_kind (Encoder.kind_of_format format) in
+  let kind = Kind.of_kind (Encoder.kind_of_format format) in
   if Encoder.file_output format then
     (new file_output_using_encoder ~format_val ~kind p :> piped_output)
   else (new file_output ~format_val ~kind p :> piped_output)
@@ -427,7 +455,7 @@ let () =
 class external_output p =
   let format_val = Lang.assoc "" 1 p in
   let format = Lang.to_format format_val in
-  let kind = Source.Kind.of_kind (Encoder.kind_of_format format) in
+  let kind = Kind.of_kind (Encoder.kind_of_format format) in
   let process = Lang.to_string_getter (Lang.assoc "" 2 p) in
   let self_sync = Lang.to_bool (List.assoc "self_sync" p) in
   object (self)
