@@ -58,10 +58,10 @@ let string_of_metadata metadata =
     (fun k v ->
       if !first then (
         first := false;
-        try Format.fprintf f "%s=\"%s\"" k (Utils.escape_utf8_string v)
+        try Format.fprintf f "%s=%s" k (Utils.quote_utf8_string v)
         with _ -> ())
       else (
-        try Format.fprintf f "\n%s=\"%s\"" k (Utils.escape_utf8_string v)
+        try Format.fprintf f "\n%s=%s" k (Utils.quote_utf8_string v)
         with _ -> ()))
     metadata;
   Format.pp_print_flush f ();
@@ -382,7 +382,7 @@ let push_indicators t l =
      * to be able to disable this check in some cases, like playlist.safe. *)
     local_check t)
 
-let is_ready t =
+let resolved t =
   t.indicators <> []
   && Sys.file_exists (peek_indicator t).string
   && (t.decoder <> None || t.ctype = None)
@@ -391,7 +391,7 @@ let is_ready t =
   * [Some f] if the request successfully lead to a local file [f],
   * [None] otherwise. *)
 let get_filename t =
-  if is_ready t then Some (List.hd (List.hd t.indicators)).string else None
+  if resolved t then Some (List.hd (List.hd t.indicators)).string else None
 
 let update_metadata t =
   let replace = Hashtbl.replace t.root_metadata in
@@ -404,28 +404,26 @@ let update_metadata t =
       | (h :: _) :: _ -> if h.temporary then "true" else "false"
       | _ -> "false");
   begin
-    match get_filename t with
-    | Some f -> replace "filename" f
-    | None -> ()
+    match get_filename t with Some f -> replace "filename" f | None -> ()
   end;
 
   (* STATUS *)
   begin
     match t.resolving with
-    | Some d -> replace "resolving" (pretty_date (Unix.localtime d))
-    | None -> ()
+      | Some d -> replace "resolving" (pretty_date (Unix.localtime d))
+      | None -> ()
   end;
   begin
     match t.on_air with
-    | Some d ->
-        replace "on_air" (pretty_date (Unix.localtime d));
-        replace "on_air_timestamp" (Printf.sprintf "%.02f" d)
-    | None -> ()
+      | Some d ->
+          replace "on_air" (pretty_date (Unix.localtime d));
+          replace "on_air_timestamp" (Printf.sprintf "%.02f" d)
+      | None -> ()
   end;
   begin
     match t.ctype with
-    | None -> ()
-    | Some ct -> replace "kind" (Frame.string_of_content_type ct)
+      | None -> ()
+      | Some ct -> replace "kind" (Frame.string_of_content_type ct)
   end;
   replace "status"
     (match t.status with
@@ -640,7 +638,7 @@ let resolve ~ctype t timeout =
   in
   let result =
     try
-      while not (is_ready t) do
+      while not (resolved t) do
         let timeleft = maxtime -. Unix.time () in
         if timeleft > 0. then resolve_step ()
         else (
