@@ -44,9 +44,9 @@
   * Don't mess with this, type errors will give you a hard time!
   * (By the way, there's no problem of scope escaping for class type
   *  since those are structural, not nominal like a variant type.) *)
-type source_t = Fallible | Infallible
+type source_t = [ `Fallible | `Infallible ]
 
-(** {1 Proto clocks} 
+(** {1 Proto clocks}
   *
   * Roughly describe what a clock is, and build a notion of clock variable
   * on top of that. More concrete clock stuff is done the [Clock] module.
@@ -141,7 +141,7 @@ and 'a link_t =
       (** the clock variable is unknown but depends on other variables *)
   | Same_as of 'a var  (** the clock variable is substituted by another *)
 
-let debug = Utils.getenv_opt "LIQUIDSOAP_DEBUG" <> None
+let debug = Sys.getenv_opt "LIQUIDSOAP_DEBUG" <> None
 let create_known c = Known c
 
 let create_unknown ~sources ~sub_clocks =
@@ -223,86 +223,6 @@ let rec forget var subclock =
     | Link ({ contents = Unknown (sources, clocks) } as r) ->
         r := Unknown (sources, List.filter (( <> ) subclock) clocks)
 
-(** Kind with variables in order to unify kinds. *)
-module Kind = struct
-  open Frame
-
-  exception Conflict of string * string
-
-  type kind = Frame.kind Unifier.t
-  type t = kind fields
-
-  let deref = Unifier.deref
-
-  let of_kind { audio; video; midi } =
-    let f = Unifier.make in
-    { audio = f audio; video = f video; midi = f midi }
-
-  let set_audio { video; midi } audio =
-    { Frame.audio = Unifier.make audio; video; midi }
-
-  let set_video { audio; midi } video =
-    { Frame.audio; video = Unifier.make video; midi }
-
-  let set_midi { audio; video } midi =
-    { Frame.audio; video; midi = Unifier.make midi }
-
-  let content_type { audio; video; midi } =
-    let get t v =
-      match deref v with
-        | `Internal | `Any -> (
-            match t with
-              | `Audio -> Frame_content.default_audio ()
-              | `Video -> Frame_content.default_video ()
-              | `Midi -> Frame_content.default_midi ())
-        | `Format f -> f
-        | `Kind k -> Frame_content.default_format k
-    in
-    {
-      Frame.audio = get `Audio audio;
-      video = get `Video video;
-      midi = get `Midi midi;
-    }
-
-  let to_string { audio; video; midi } =
-    Frame.string_of_content_kind
-      { Frame.audio = deref audio; video = deref video; midi = deref midi }
-
-  let unify_kind k k' =
-    let to_s k = Frame.string_of_kind (deref k) in
-    try
-      match (deref k, deref k') with
-        (* Formats *)
-        | `Kind ki, `Kind ki' when ki = ki' -> Unifier.(k <-- k')
-        (* Params *)
-        | `Format f, `Format f' -> Frame_content.merge f f'
-        (* Format/params *)
-        | `Kind ki, `Format f when Frame_content.kind f = ki ->
-            Unifier.(k <-- k')
-        | `Format f, `Kind ki when Frame_content.kind f = ki ->
-            Unifier.(k' <-- k)
-        (* `Internal/'a *)
-        | `Internal, `Internal -> Unifier.(k <-- k')
-        | `Internal, `Kind ki when Frame_content.is_internal ki ->
-            Unifier.(k <-- k')
-        | `Internal, `Format f when Frame_content.(is_internal (kind f)) ->
-            Unifier.(k <-- k')
-        | `Kind ki, `Internal when Frame_content.is_internal ki ->
-            Unifier.(k' <-- k)
-        | `Format f, `Internal when Frame_content.(is_internal (kind f)) ->
-            Unifier.(k' <-- k)
-        (* Any/'a *)
-        | `Any, _ -> Unifier.(k <-- k')
-        | _, `Any -> Unifier.(k' <-- k)
-        | _ -> assert false
-    with _ -> raise (Conflict (to_s k, to_s k'))
-
-  let unify k k' =
-    unify_kind k.audio k'.audio;
-    unify_kind k.video k'.video;
-    unify_kind k.midi k'.midi
-end
-
 (** {1 Sources} *)
 
 (** Instrumentation. *)
@@ -452,7 +372,10 @@ class virtual operator ?(name = "src") ?audio_in ?video_in ?midi_in out_kind
             ct
 
     method private audio_channels =
-      Frame_content.Audio.channels_of_format self#ctype.Frame.audio
+      Content.Audio.channels_of_format self#ctype.Frame.audio
+
+    method private video_dimensions =
+      Content.Video.dimensions_of_format self#ctype.Frame.video
 
     (** Startup/shutdown.
     *
@@ -621,9 +544,7 @@ class virtual operator ?(name = "src") ?audio_in ?video_in ?midi_in out_kind
     (* [self#seek x] skips [x] main ticks.
      * returns the number of ticks actually skipped.
      * By default it always returns 0, refusing to seek at all. *)
-    method seek (_ : int) =
-      self#log#important "Seeking not supported by this source!";
-      0
+    method virtual seek : int -> int
 
     (* Is there some data available for the next [get]?
      * Must always be true while playing a track, i.e. all tracks
@@ -823,6 +744,15 @@ class virtual active_source ?name ?audio_in ?video_in ?midi_in content_kind =
     inherit active_operator ?name ?audio_in ?video_in ?midi_in content_kind []
   end
 
+class virtual no_seek =
+  object (self)
+    method virtual log : Log.t
+
+    method seek (_ : int) =
+      self#log#important "Seeking not supported by this source!";
+      0
+  end
+
 (** Specialized shortcuts *)
 
 type clock_variable = active_source var
