@@ -34,14 +34,16 @@ type clock_variable
   * protocol such as [input.srt]. *)
 type sync = [ `Auto | `CPU | `None ]
 
-(* Type for source's self_sync. A [`Static] self_sync should never change over
-   the source's lifetime. *)
+(** Type for source's self_sync. The boolean indicates whether the operator
+    takes care of synchronization by itself or not. The first component indicates
+    whether this value can change during the source's lifetime (it cannot if this
+    is [`Static]. *)
 type self_sync = [ `Static | `Dynamic ] * bool
 
 (** The liveness type of a source indicates whether or not it can
   * fail to broadcast.
-  * A Infallible source never fails; it is always ready. *)
-type source_t = Fallible | Infallible
+  * A `Infallible source never fails; it is always ready. *)
+type source_t = [ `Fallible | `Infallible ]
 
 (** Instrumentation. *)
 
@@ -70,19 +72,6 @@ type watcher = {
   after_output : unit -> unit;
 }
 
-module Kind : sig
-  type kind
-  type t = kind Frame.fields
-
-  val of_kind : Frame.content_kind -> t
-  val to_string : t -> string
-
-  exception Conflict of string * string
-
-  val unify_kind : kind -> kind -> unit
-  val unify : t -> t -> unit
-end
-
 (** Generate an identifier from the name of the source. *)
 val generate_id : string -> string
 
@@ -167,6 +156,8 @@ class virtual source :
        (** This method fails when content is not PCM. *)
        method private audio_channels : int
 
+       method private video_dimensions : int * int
+
        (** Retrieve the frame currently being filled. *)
        method memo : Frame.t
 
@@ -181,7 +172,7 @@ class virtual source :
            By default it always returns 0, refusing to seek at all.
            That method may be called from any thread, concurrently
            with [#get], so they should not interfer. *)
-       method seek : int -> int
+       method virtual seek : int -> int
 
        (** [is_ready] tells you if [get] can be called. *)
        method virtual is_ready : bool
@@ -272,6 +263,12 @@ class virtual active_operator :
        inherit active_source
      end
 
+class virtual no_seek :
+  object
+    method virtual log : Log.t
+    method seek : int -> int
+  end
+
 val has_outputs : unit -> bool
 val iterate_new_outputs : (active_source -> unit) -> unit
 
