@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -39,9 +39,10 @@ class mic ~kind ~clock_safe ~fallible ~on_start ~on_stop ~start device =
       Start_stop.active_source
         ~name:"input.alsa" ~fallible ~on_start ~on_stop ~autostart:start
           ~clock_safe ~get_clock:Alsa_settings.get_clock
-          ~content_kind:(Source.Kind.of_kind kind) () as active_source
+          ~content_kind:(Kind.of_kind kind) () as active_source
 
-    inherit [Frame_content.Audio.data] IoRing.input ~nb_blocks as ioring
+    inherit Source.no_seek
+    inherit [Content.Audio.data] IoRing.input ~nb_blocks as ioring
     val mutable initialized = false
     method self_sync = (`Static, true)
 
@@ -61,8 +62,8 @@ class mic ~kind ~clock_safe ~fallible ~on_start ~on_stop ~start device =
     val mutable sample_freq = Lazy.force Frame.audio_rate
 
     val mutable read_fun =
-      fun pcm (buf : Frame_content.Audio.data) ofs len ->
-        Pcm.readn_float_ba pcm (Audio.sub buf ofs len)
+      fun pcm (buf : Content.Audio.data) ofs len ->
+        Pcm.readn_float pcm buf ofs len
 
     val mutable device = None
 
@@ -95,7 +96,7 @@ class mic ~kind ~clock_safe ~fallible ~on_start ~on_stop ~start device =
                     let r = Pcm.readi pcm sbuf 0 len in
                     Audio.S16LE.to_audio
                       (Bytes.unsafe_to_string sbuf)
-                      0 (Audio.sub buf ofs r);
+                      0 buf ofs r;
                     r)
             end;
             sample_freq <- Pcm.set_rate_near dev params sample_freq Dir_eq;
@@ -126,8 +127,8 @@ class mic ~kind ~clock_safe ~fallible ~on_start ~on_stop ~start device =
       with e ->
         begin
           match e with
-          | Buffer_xrun -> self#log#important "Overrun!"
-          | _ -> self#log#severe "Alsa error: %s" (string_of_error e)
+            | Buffer_xrun -> self#log#important "Overrun!"
+            | _ -> self#log#severe "Alsa error: %s" (string_of_error e)
         end;
         if e = Buffer_xrun || e = Suspended || e = Interrupted then (
           self#log#severe "Trying to recover..";
@@ -138,11 +139,7 @@ class mic ~kind ~clock_safe ~fallible ~on_start ~on_stop ~start device =
       assert (0 = AFrame.position buf);
       let buffer = ioring#get_block in
       let fbuf = AFrame.pcm buf in
-      for c = 0 to Array.length fbuf - 1 do
-        Audio.Mono.blit
-          (Audio.Mono.sub buffer.(c) 0 buffer_length)
-          (Audio.Mono.sub fbuf.(c) 0 buffer_length)
-      done;
+      Audio.blit buffer 0 fbuf 0 buffer_length;
       AFrame.add_break buf buffer_length
 
     method reset = ()
