@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ class jack_in ~kind ~clock_safe ~on_start ~on_stop ~fallible ~autostart
         ~name:"input.jack" ~content_kind:kind ~clock_safe ~on_start ~on_stop
           ~fallible ~autostart () as active_source
 
+    inherit Source.no_seek
     inherit [Bytes.t] IoRing.input ~nb_blocks as ioring
 
     method private wake_up l =
@@ -107,8 +108,7 @@ class jack_in ~kind ~clock_safe ~on_start ~on_stop ~fallible ~autostart
       let fbuf = AFrame.pcm buf in
       Audio.S16LE.to_audio
         (Bytes.unsafe_to_string buffer)
-        0
-        (Audio.sub fbuf 0 samples_per_frame);
+        0 fbuf 0 samples_per_frame;
       AFrame.add_break buf samples_per_frame
 
     method reset = ()
@@ -145,7 +145,7 @@ let () =
       in
       let nb_blocks = Lang.to_int (List.assoc "buffer_size" p) in
       let server = Lang.to_string (List.assoc "server" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new jack_in
          ~kind ~clock_safe ~nb_blocks ~server ~fallible ~on_start ~on_stop
          ~autostart
