@@ -32,7 +32,7 @@ class blank ~kind duration =
     val mutable remaining = ticks
 
     method remaining = remaining
-    method stype = Infallible
+    method stype = `Infallible
     method is_ready = true
     method self_sync = (`Static, false)
     method seek x = x
@@ -50,12 +50,13 @@ class blank ~kind duration =
          Audio.clear (AFrame.pcm ab)
            (Frame.audio_of_main position)
            (Frame.audio_of_main length)
-       with Frame_content.Invalid -> ());
+       with Content.Invalid -> ());
 
       (* Video *)
       (try
-         Video.blank (VFrame.yuva420p ab) video_pos (Frame.video_of_main length)
-       with Frame_content.Invalid -> ());
+         Video.Canvas.blank (VFrame.data ab) video_pos
+           (Frame.video_of_main length)
+       with Content.Invalid -> ());
 
       Frame.add_break ab (position + length);
       if Frame.is_partial ab then remaining <- ticks
@@ -77,5 +78,5 @@ let () =
     ]
     (fun p ->
       let d = Lang.to_float (List.assoc "duration" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new blank ~kind d :> source))
