@@ -77,7 +77,7 @@ class video ~name ~kind ~restart ~bufferize ~log_overfull ~restart_on_error ~max
       (* Now we can create the log function *)
       log_ref := self#log#info "%s";
       log_error := self#log#severe "%s";
-      if self#ctype.Frame.audio = Frame_content.None.format then
+      if self#ctype.Frame.audio = Content.None.format then
         Generator.set_mode abg `Video;
       Generator.set_content_type abg self#ctype;
       self#log#debug "Generator mode: %s."
@@ -183,11 +183,11 @@ let () =
                 if
                   out_width = in_width && out_height = in_height
                   && video_format = `I420
-                then src
+                then Video.Canvas.Image.make src
                 else (
                   let dst = Video.Image.create out_width out_height in
                   video_scaler src dst;
-                  dst)
+                  Video.Canvas.Image.make dst)
               in
               video_converter := Some converter
           | `Audio (channels, samplerate) ->
@@ -225,7 +225,7 @@ let () =
                      (width * height * 3));
               let data = (Option.get !video_converter) data in
               Generator.put_video abg
-                (Frame_content.Video.lift_data (Video.single data))
+                (Content.Video.lift_data (Video.Canvas.single data))
                 0 (Frame.main_of_video 1)
           | `Frame (`Audio, _, data) ->
               let converter = Option.get !audio_converter in
@@ -233,7 +233,7 @@ let () =
                 converter (Bytes.unsafe_of_string data) 0 (String.length data)
               in
               Generator.put_audio abg
-                (Frame_content.Audio.lift_data data)
+                (Content.Audio.lift_data data)
                 (Frame.main_of_audio ofs) (Frame.main_of_audio len)
           | _ -> failwith "Invalid chunk."
       in
@@ -242,7 +242,7 @@ let () =
       let restart = Lang.to_bool (List.assoc "restart" p) in
       let restart_on_error = Lang.to_bool (List.assoc "restart_on_error" p) in
       let max = Lang.to_float (List.assoc "max" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new video
         ~name:"input.external.avi" ~kind ~restart ~bufferize ~log_overfull
         ~restart_on_error ~max ~read_header ~on_data command)
@@ -307,7 +307,8 @@ let () =
         (* Img.swap_rb data; *)
         (* Img.Effect.flip data; *)
         Generator.put_video abg
-          (Frame_content.Video.lift_data (Video.single data))
+          (Content.Video.lift_data
+             (Video.Canvas.single (Video.Canvas.Image.make data)))
           0 (Frame.main_of_video 1)
       in
       let bufferize = Lang.to_float (List.assoc "buffer" p) in
@@ -315,7 +316,7 @@ let () =
       let restart = Lang.to_bool (List.assoc "restart" p) in
       let restart_on_error = Lang.to_bool (List.assoc "restart_on_error" p) in
       let max = Lang.to_float (List.assoc "max" p) in
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new video
         ~name:"input.external.rawvideo" ~kind ~restart ~bufferize ~log_overfull
         ~restart_on_error ~max ~on_data command)
