@@ -84,7 +84,7 @@ module Make (Harbor : T) = struct
           =
         login
 
-      method stype = Source.Fallible
+      method stype = `Fallible
       method icy_charset = icy_charset
       method meta_charset = meta_charset
 
@@ -135,17 +135,17 @@ module Make (Harbor : T) = struct
           in
           begin
             match dump with
-            | Some b ->
-                output_string b (Bytes.sub_string buf 0 input);
-                flush b
-            | None -> ()
+              | Some b ->
+                  output_string b (Bytes.sub_string buf 0 input);
+                  flush b
+              | None -> ()
           end;
           begin
             match logf with
-            | Some b ->
-                let time = (Unix.gettimeofday () -. t0) /. 60. in
-                Printf.fprintf b "%f %d\n%!" time self#length
-            | None -> ()
+              | Some b ->
+                  let time = (Unix.gettimeofday () -. t0) /. 60. in
+                  Printf.fprintf b "%f %d\n%!" time self#length
+              | None -> ()
           end;
           input
         in
@@ -220,21 +220,21 @@ module Make (Harbor : T) = struct
         on_connect headers;
         begin
           match dumpfile with
-          | Some f -> (
-              try dump <- Some (open_out_bin (Utils.home_unrelate f))
-              with e ->
-                self#log#severe "Could not open dump file: %s"
-                  (Printexc.to_string e))
-          | None -> ()
+            | Some f -> (
+                try dump <- Some (open_out_bin (Utils.home_unrelate f))
+                with e ->
+                  self#log#severe "Could not open dump file: %s"
+                    (Printexc.to_string e))
+            | None -> ()
         end;
         begin
           match logfile with
-          | Some f -> (
-              try logf <- Some (open_out_bin (Utils.home_unrelate f))
-              with e ->
-                self#log#severe "Could not open log file: %s"
-                  (Printexc.to_string e))
-          | None -> ()
+            | Some f -> (
+                try logf <- Some (open_out_bin (Utils.home_unrelate f))
+                with e ->
+                  self#log#severe "Could not open log file: %s"
+                    (Printexc.to_string e))
+            | None -> ()
         end;
         ignore (Tutils.create (fun () -> self#feed) () "harbor source feeding")
 
@@ -248,17 +248,17 @@ module Make (Harbor : T) = struct
       method private after_disconnect =
         begin
           match dump with
-          | Some f ->
-              close_out f;
-              dump <- None
-          | None -> ()
+            | Some f ->
+                close_out f;
+                dump <- None
+            | None -> ()
         end;
         begin
           match logf with
-          | Some f ->
-              close_out f;
-              logf <- None
-          | None -> ()
+            | Some f ->
+                close_out f;
+                logf <- None
+            | None -> ()
         end;
         on_disconnect ()
 
@@ -287,6 +287,14 @@ module Make (Harbor : T) = struct
                   match s#connected_client with
                     | Some c -> Lang.string c
                     | None -> Lang.null) );
+          ( "disconnect",
+            ([], Lang.fun_t [] Lang.unit_t),
+            "Disconnect the client currently connected to the harbor. Does \
+             nothing if no client is connected.",
+            fun s ->
+              Lang.val_fun [] (fun _ ->
+                  s#disconnect ~lock:true;
+                  Lang.unit) );
           ( "status",
             ([], Lang.fun_t [] Lang.string_t),
             "Current status of the input.",
@@ -486,7 +494,7 @@ module Make (Harbor : T) = struct
         let on_disconnect () =
           ignore (Lang.apply (List.assoc "on_disconnect" p) [])
         in
-        let kind = Source.Kind.of_kind Lang.any in
+        let kind = Kind.of_kind Lang.any in
         new http_input_server
           ~kind ~timeout ~bufferize ~max ~login ~mountpoint ~dumpfile ~logfile
           ~icy ~port ~icy_charset ~meta_charset ~replay_meta ~on_connect
