@@ -34,13 +34,15 @@ class noise ~kind duration =
          let len = Frame.audio_of_main len in
          let b = AFrame.pcm frame in
          Audio.Generator.white_noise b off len
-       with Frame_content.Invalid -> ());
+       with Content.Invalid -> ());
       try
         let off = Frame.video_of_main off in
         let len = Frame.video_of_main len in
-        let b = VFrame.yuva420p frame in
-        Video.iter Video.Image.randomize b off len
-      with Frame_content.Invalid -> ()
+        let b = VFrame.data frame in
+        Video.Canvas.map
+          (Video.Canvas.Image.iter Video.Image.randomize)
+          b off len
+      with Content.Invalid -> ()
   end
 
 let () =
@@ -56,7 +58,7 @@ let () =
     ]
     ~return_t
     (fun p ->
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       new noise
         ~kind
         (Lang.to_valued_option Lang.to_float (List.assoc "duration" p)))
