@@ -42,7 +42,7 @@ class once ~kind ~name ~timeout request =
   object (self)
     inherit source kind ~name as super
     method self_sync = (`Static, false)
-    method stype = Fallible
+    method stype = `Fallible
 
     (* True means that the request has already been played or could not be
        resolved. *)
@@ -69,7 +69,7 @@ class once ~kind ~name ~timeout request =
         (match Request.resolve ~ctype:(Some self#ctype) request timeout with
           | Request.Resolved -> ()
           | ans -> log_failed_request self#log request ans);
-        if not (Request.is_ready request) then (
+        if not (Request.resolved request) then (
           over <- true;
           self#log#critical "Failed to prepare track: request not ready.";
           Request.destroy request)
@@ -153,20 +153,20 @@ class virtual unqueued ~kind ~name =
       Tutils.mutexify plock (fun forced ->
           begin
             match current with
-            | None -> ()
-            | Some cur ->
-                begin
-                  match Request.get_filename cur.req with
-                  | None ->
-                      self#log#severe
-                        "Finished with a non-existent file?! Something may \
-                         have been moved or destroyed during decoding. It is \
-                         VERY dangerous, avoid it!"
-                  | Some f -> self#log#info "Finished with %S." f
-                end;
-                cur.close ();
-                Request.destroy cur.req;
-                must_fail <- forced
+              | None -> ()
+              | Some cur ->
+                  begin
+                    match Request.get_filename cur.req with
+                      | None ->
+                          self#log#severe
+                            "Finished with a non-existent file?! Something may \
+                             have been moved or destroyed during decoding. It \
+                             is VERY dangerous, avoid it!"
+                      | Some f -> self#log#info "Finished with %S." f
+                  end;
+                  cur.close ();
+                  Request.destroy cur.req;
+                  must_fail <- forced
           end;
           current <- None;
           remaining <- 0)
@@ -180,10 +180,10 @@ class virtual unqueued ~kind ~name =
         | `Retry _ | `Empty ->
             self#log#debug "Failed to prepare track: no file.";
             false
-        | `Request req when Request.is_ready req && Request.ctype req <> None ->
+        | `Request req when Request.resolved req && Request.ctype req <> None ->
             assert (Frame.compatible (Option.get (Request.ctype req)) self#ctype);
 
-            (* [Request.is_ready] ensures that we can get a filename from the request,
+            (* [Request.resolved] ensures that we can get a filename from the request,
                and it can be decoded. *)
             let file = Option.get (Request.get_filename req) in
             let decoder = Option.get (Request.get_decoder req) in
@@ -259,7 +259,7 @@ type queue_item = {
 }
 
 (* Scheduler priority for request resolutions. *)
-let priority = Tutils.Maybe_blocking
+let priority = `Maybe_blocking
 
 (** Same thing, with a queue in which we prefetch files, which requests are
     given by [get_next_request]. Heuristical settings determining how the source
@@ -270,7 +270,7 @@ let priority = Tutils.Maybe_blocking
 class virtual queued ~kind ~name ?(prefetch = 1) ?(timeout = 20.) () =
   object (self)
     inherit unqueued ~kind ~name as super
-    method stype = Fallible
+    method stype = `Fallible
 
     method virtual get_next_request
         : [ `Empty | `Request of Request.t | `Retry of unit -> float ]
