@@ -30,13 +30,13 @@ type handler = {
 }
 
 class once :
-  kind:Source.Kind.t
+  kind:Kind.t
   -> name:string
   -> timeout:float
   -> Request.t
   -> object
        inherit Source.source
-       method stype : Source.source_t
+       method stype : [ `Fallible | `Infallible ]
        method self_sync : Source.self_sync
        method is_ready : bool
        method request : Request.t
@@ -44,10 +44,11 @@ class once :
        method private get_frame : Frame.t -> unit
        method resolve : bool
        method abort_track : unit
+       method seek : int -> int
      end
 
 class virtual unqueued :
-  kind:Source.Kind.t
+  kind:Kind.t
   -> name:string
   -> object
        (** [get_next_file] is the only thing you've got to define,
@@ -62,6 +63,7 @@ class virtual unqueued :
        method remaining : int
        method self_sync : Source.self_sync
        method current : handler option
+       method seek : int -> int
      end
 
 type queue_item = {
@@ -71,13 +73,13 @@ type queue_item = {
 }
 
 class virtual queued :
-  kind:Source.Kind.t
+  kind:Kind.t
   -> name:string
   -> ?prefetch:int
   -> ?timeout:float
   -> unit
   -> object
-       method stype : Source.source_t
+       method stype : [ `Fallible | `Infallible ]
 
        (** You should only define this. *)
        method virtual get_next_request :
