@@ -28,8 +28,8 @@ type t = Frame.t
 (* Samples of ticks, and vice versa. *)
 let sot = audio_of_main
 let tos = main_of_audio
-let content b = b.content.audio
-let pcm b = Frame_content.Audio.get_data (content b)
+let content = Frame.audio
+let pcm b = Content.Audio.get_data (content b)
 
 let to_s16le b =
   let fpcm = pcm b in
