@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *****************************************************************************)
 
 include Frame_settings
-open Frame_content
+open Content
 
 (** Data types *)
 
@@ -29,10 +29,7 @@ type 'a fields = { audio : 'a; video : 'a; midi : 'a }
 
 (** High-level description of the content. *)
 type kind =
-  [ `Any
-  | `Internal
-  | `Kind of Frame_content.kind
-  | `Format of Frame_content.format ]
+  [ `Any | `Internal | `Kind of Content.kind | `Format of Content.format ]
 
 let none = `Format None.format
 let audio_pcm = `Kind Audio.kind
@@ -88,9 +85,9 @@ let string_of_content_kind = string_of_fields string_of_kind
 let string_of_content_type = string_of_fields string_of_format
 
 let compatible c c' =
-  Frame_content.compatible c.audio c'.audio
-  && Frame_content.compatible c.video c'.video
-  && Frame_content.compatible c.midi c'.midi
+  Content.compatible c.audio c'.audio
+  && Content.compatible c.video c'.video
+  && Content.compatible c.midi c'.midi
 
 (* Frames *)
 
@@ -105,7 +102,7 @@ let metadata_of_list l =
 
 type t = {
   (* Presentation time, in multiple of frame size. *)
-  mutable pts : int64;
+  mutable pts : int64 option;
   (* End of track markers.
    * A break at the end of the buffer is not an end of track.
    * So maybe we should rather call that an end-of-fill marker,
@@ -123,18 +120,20 @@ type t = {
 let create_content ctype = map_fields (make ~size:!!size) ctype
 
 let create ctype =
-  { pts = 0L; breaks = []; metadata = []; content = create_content ctype }
+  { pts = None; breaks = []; metadata = []; content = create_content ctype }
 
 let dummy =
-  let data = Frame_content.None.data in
+  let data = Content.None.data in
   {
-    pts = 0L;
+    pts = None;
     breaks = [];
     metadata = [];
     content = { audio = data; video = data; midi = data };
   }
 
 let content_type { content } = map_fields format content
+let content { content } = content
+let set_content frame content = frame.content <- content
 let audio { content; _ } = content.audio
 let set_audio frame audio = frame.content <- { frame.content with audio }
 let video { content; _ } = content.video
@@ -151,9 +150,9 @@ let set_breaks b breaks = b.breaks <- breaks
 let add_break b br = b.breaks <- br :: b.breaks
 
 let clear (b : t) =
-  Frame_content.clear b.content.audio;
-  Frame_content.clear b.content.video;
-  Frame_content.clear b.content.midi;
+  Content.clear b.content.audio;
+  Content.clear b.content.video;
+  Content.clear b.content.midi;
   b.breaks <- [];
   b.metadata <- []
 
@@ -161,15 +160,6 @@ let clear_from (b : t) pos =
   b.breaks <- List.filter (fun p -> p <= pos) b.breaks;
   b.metadata <- List.filter (fun (p, _) -> p <= pos) b.metadata
 
-(* Same as clear but leaves the last metadata at position -1. *)
-let advance b =
-  Frame_content.clear b.content.audio;
-  Frame_content.clear b.content.video;
-  Frame_content.clear b.content.midi;
-  b.pts <- Int64.succ b.pts;
-  b.breaks <- [];
-  b.metadata <- []
-
 (** Presentation time stuff. *)
 
 let pts { pts } = pts
@@ -230,7 +220,7 @@ let get_chunk ab from =
      * copy the one from [from] to [p] in [ab].
      * NOTE (toots): This mechanism is super weird. See last test
        in src/test/frame_test.ml. I suspect that it is here b/c we
-       have gotten into the habit of caching the last metadata at 
+       have gotten into the habit of caching the last metadata at
        position -1 and that this is meant to surface it at position 0
        of the next chunk. I sure hope that we can revisit this mechanism
        at some point in the future.. *)
