@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2022 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,9 +26,7 @@ module type S = sig
   type t
 
   val length : t -> int (* ticks *)
-
   val remaining : t -> int (* ticks *)
-
   val clear : t -> unit
   val fill : t -> Frame.t -> unit
   val remove : t -> int -> unit
@@ -39,17 +37,15 @@ module type S_Asio = sig
   type t
 
   val length : t -> int (* ticks *)
-
   val audio_length : t -> int
   val video_length : t -> int
   val remaining : t -> int (* ticks *)
-
   val clear : t -> unit
   val fill : t -> Frame.t -> unit
   val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
   val add_break : ?sync:bool -> ?pos:int -> t -> unit
-  val put_audio : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
-  val put_video : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val put_audio : ?pts:int64 -> t -> Content.data -> int -> int -> unit
+  val put_video : ?pts:int64 -> t -> Content.data -> int -> int -> unit
   val set_mode : t -> [ `Audio | `Video | `Both | `Undefined ] -> unit
 end
 
@@ -284,7 +280,7 @@ module From_frames = struct
           (List.filter (fun x -> x < position) (Frame.breaks frame));
 
     (* Feed all content layers into the generator. *)
-    let content = frame.Frame.content in
+    let content = Frame.content frame in
     let content =
       match copy with
         | `None -> content
@@ -306,7 +302,7 @@ module From_frames = struct
     let blocks = Generator.get fg.generator needed in
     List.iter
       (fun (block, o, o', size) ->
-        let dst = frame.Frame.content in
+        let dst = Frame.content frame in
         Frame.fill_content block o dst (offset + o') size)
       blocks;
     List.iter
@@ -357,11 +353,11 @@ module From_audio_video = struct
     mutable mode : mode;
     mutable current_pts : int64 option;
     mutable current_audio_pts : int64 option;
-    current_audio : Frame_content.data content Generator.t;
-    audio : Frame_content.data content Generator.t;
+    current_audio : Content.data content Generator.t;
+    audio : Content.data content Generator.t;
     mutable current_video_pts : int64 option;
-    current_video : Frame_content.data content Generator.t;
-    video : Frame_content.data content Generator.t;
+    current_video : Content.data content Generator.t;
+    video : Content.data content Generator.t;
     mutable metadata : (int * Frame.metadata) list;
     mutable breaks : int list;
   }
@@ -577,15 +573,15 @@ module From_audio_video = struct
       put_frames ~pts ~current_pts:t.current_audio_pts t.current_audio data o l;
     begin
       match t.mode with
-      (* The buffer's logic is all synchronous so we keep
-         constant empty content for the other type when using
-         the buffer with one single type. *)
-      | `Audio ->
-          t.current_video_pts <-
-            put_frames ~pts ~current_pts:t.current_video_pts t.current_video
-              Frame_content.None.data 0 l
-      | `Both -> ()
-      | _ -> assert false
+        (* The buffer's logic is all synchronous so we keep
+           constant empty content for the other type when using
+           the buffer with one single type. *)
+        | `Audio ->
+            t.current_video_pts <-
+              put_frames ~pts ~current_pts:t.current_video_pts t.current_video
+                Content.None.data 0 l
+        | `Both -> ()
+        | _ -> assert false
     end;
     sync_content t
 
@@ -598,14 +594,14 @@ module From_audio_video = struct
       put_frames ~pts ~current_pts:t.current_video_pts t.current_video data o l;
     begin
       match t.mode with
-      (* The buffer's logic is all synchronous so we keep
-         constant empty content for the other type when using
-         the buffer with one single type. *)
-      | `Video ->
-          t.current_audio_pts <-
-            put_frames ~pts ~current_pts:t.current_audio_pts t.current_audio
-              Frame_content.None.data 0 l
-      | _ -> ()
+        (* The buffer's logic is all synchronous so we keep
+           constant empty content for the other type when using
+           the buffer with one single type. *)
+        | `Video ->
+            t.current_audio_pts <-
+              put_frames ~pts ~current_pts:t.current_audio_pts t.current_audio
+                Content.None.data 0 l
+        | _ -> ()
     end;
     sync_content t
 
@@ -630,7 +626,7 @@ module From_audio_video = struct
     let pts = Frame.pts frame in
     let mode = match mode with Some mode -> mode | None -> t.mode in
     let pos = Frame.position frame in
-    let content = frame.Frame.content in
+    let content = Frame.content frame in
 
     match mode with
       | `Audio ->
@@ -724,13 +720,13 @@ module From_audio_video = struct
     if mode = `Both || mode = `Audio then
       List.iter
         (fun ({ data }, apos, apos', al) ->
-          Frame_content.blit data apos (AFrame.content frame) (fpos + apos') al)
+          Content.blit data apos (AFrame.content frame) (fpos + apos') al)
         audio;
 
     if mode = `Both || mode = `Video then
       List.iter
         (fun ({ data }, vpos, vpos', vl) ->
-          Frame_content.blit data vpos (VFrame.content frame) (fpos + vpos') vl)
+          Content.blit data vpos (VFrame.content frame) (fpos + vpos') vl)
         video;
 
     Frame.set_pts frame t.current_pts;
@@ -825,7 +821,7 @@ module From_audio_video_plus = struct
         let p = Frame.position frame in
         let breaks = Frame.breaks frame in
         Super.fill t.gen frame;
-        let c = frame.Frame.content in
+        let c = Frame.content frame in
         match t.ctype with
           | None -> t.ctype <- Some (Frame.type_of_content c)
           | Some ctype ->
