@@ -172,17 +172,15 @@ module type S_Asio = sig
   type t
 
   val length : t -> int (* ticks *)
-
   val audio_length : t -> int
   val video_length : t -> int
   val remaining : t -> int (* ticks *)
-
   val clear : t -> unit
   val fill : t -> Frame.t -> unit
   val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
   val add_break : ?sync:bool -> ?pos:int -> t -> unit
-  val put_audio : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
-  val put_video : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val put_audio : ?pts:int64 -> t -> Content.data -> int -> int -> unit
+  val put_video : ?pts:int64 -> t -> Content.data -> int -> int -> unit
   val set_mode : t -> [ `Audio | `Video | `Both | `Undefined ] -> unit
 end
 
@@ -244,11 +242,11 @@ module From_audio_video : sig
 
   (* [put_audio ?pts buffer data offset length]: offset and length
    * are in main ticks ! *)
-  val put_audio : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val put_audio : ?pts:int64 -> t -> Content.data -> int -> int -> unit
 
   (* [put_video ?pts buffer data offset length]: offset and length
    * are in main ticks ! *)
-  val put_video : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val put_video : ?pts:int64 -> t -> Content.data -> int -> int -> unit
 
   (** Feed from a frame, only copying data according to the mode.
       Defaults to the generator's mode. *)
@@ -304,11 +302,11 @@ module From_audio_video_plus : sig
 
   (* [put_audio ?pts buffer data offset length]:
    * offset and length are in main ticks! *)
-  val put_audio : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val put_audio : ?pts:int64 -> t -> Content.data -> int -> int -> unit
 
   (* [put_video buffer data offset length]:
    * offset and length are in main ticks! *)
-  val put_video : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val put_video : ?pts:int64 -> t -> Content.data -> int -> int -> unit
   val feed_from_frame : ?mode:mode -> t -> Frame.t -> unit
   val fill : t -> Frame.t -> unit
   val remove : t -> int -> unit
