@@ -50,7 +50,8 @@ let all_chans = 16
 (* chan = None means synth all channels *)
 class dssi ~kind ?chan plugin descr outputs params source =
   object
-    inherit operator ~name:"dssi" (Source.Kind.of_kind kind) [source]
+    inherit operator ~name:"dssi" (Kind.of_kind kind) [source]
+    inherit Source.no_seek
     method stype = source#stype
     method remaining = source#remaining
     method is_ready = source#is_ready
@@ -231,8 +232,7 @@ let () =
   List.iter (register_plugin ~log_errors:true) dssi_load
 
 let () =
-  Lang.add_builtin "dssi.register"
-    ~category:(`Source `Synthesis)
+  Lang.add_builtin "dssi.register" ~category:(`Source `Synthesis)
     ~descr:"Register a DSSI plugin."
     [("", Lang.string_t, None, Some "Path of the DSSI plugin file.")]
     Lang.unit_t
