@@ -55,7 +55,8 @@ let note_of_char c = array_index c + 72
 class keyboard ~kind =
   object (self)
     inherit Source.active_source ~name:"input.keyboard" kind
-    method stype = Source.Infallible
+    inherit Source.no_seek
+    method stype = `Infallible
     method is_ready = true
     method remaining = -1
     method abort_track = ()
@@ -103,7 +104,7 @@ class keyboard ~kind =
           [
             {
               Duppy.Task.handler = task;
-              priority = Tutils.Non_blocking;
+              priority = `Non_blocking;
               events = [`Read Unix.stdin];
             };
           ])
@@ -111,7 +112,7 @@ class keyboard ~kind =
       Duppy.Task.add Tutils.scheduler
         {
           Duppy.Task.handler = task;
-          priority = Tutils.Non_blocking;
+          priority = `Non_blocking;
           events = [`Read Unix.stdin];
         }
 
@@ -136,5 +137,5 @@ let () =
   Lang.add_operator "input.keyboard" [] ~return_t ~category:`Input
     ~flags:[`Hidden; `Experimental] ~descr:"Play notes from the keyboard."
     (fun _ ->
-      let kind = Source.Kind.of_kind kind in
+      let kind = Kind.of_kind kind in
       (new keyboard ~kind :> Source.source))
