@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,9 +28,12 @@ class item ?(sort = true) (doc : string) =
   object
     val doc = doc
     method get_doc = doc
-    val mutable subsections : (string * item) list = []
-    method get_subsections = sort subsections
-    method get_subsection name = List.assoc name subsections
+    val mutable subsections : (string * item Lazy.t) list = []
+
+    method get_subsections =
+      sort subsections |> List.map (fun (k, v) -> (k, Lazy.force v))
+
+    method get_subsection name = Lazy.force (List.assoc name subsections)
     method has_subsection name = List.mem_assoc name subsections
 
     method add_subsection label item =
@@ -59,15 +62,15 @@ let xml_escape s =
 let print_xml (doc : item) print_string =
   let rec print_xml indent doc =
     let prefix = Bytes.unsafe_to_string (Bytes.make indent ' ') in
-    Printf.kprintf print_string "%s<info>%s</info>\n" prefix
+    Printf.ksprintf print_string "%s<info>%s</info>\n" prefix
       (xml_escape doc#get_doc);
     List.iter
       (fun (k, v) ->
-        Printf.kprintf print_string "%s<section>\n" prefix;
-        Printf.kprintf print_string " %s<label>%s</label>\n" prefix
+        Printf.ksprintf print_string "%s<section>\n" prefix;
+        Printf.ksprintf print_string " %s<label>%s</label>\n" prefix
           (xml_escape k);
         print_xml (indent + 1) v;
-        Printf.kprintf print_string "%s</section>\n" prefix)
+        Printf.ksprintf print_string "%s</section>\n" prefix)
       doc#get_subsections
   in
   print_string "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
@@ -88,7 +91,7 @@ let rec to_json (doc : item) =
     `Assoc ss)
 
 let print_json (doc : item) print_string =
-  print_string (JSON.to_string (to_json doc));
+  print_string (Json.to_string ~compact:false ~json5:false (to_json doc));
   print_string "\n"
 
 let print_functions (doc : item) print_string =
@@ -145,8 +148,18 @@ let print_functions_md ~extra (doc : item) print_string =
             Printf.ksprintf print_string "### `%s`\n\n" f;
             Printf.ksprintf print_string "%s\n\n"
               (to_string (List.assoc "_info" desc));
-            Printf.ksprintf print_string "Type:\n```\n%s\n```\n\n"
+            Printf.ksprintf print_string "Type:\n\n```\n%s\n```\n\n"
               (to_string (List.assoc "_type" desc));
+            let examples =
+              List.filter_map
+                (function "_example", `String e -> Some e | _ -> None)
+                desc
+            in
+            List.iter
+              (fun e ->
+                print_string "Example:\n\n";
+                Printf.ksprintf print_string "```liquidsoap\n%s\n```\n\n" e)
+              examples;
             let methods =
               let methods =
                 try List.assoc "_methods" desc |> to_assoc
@@ -164,8 +177,16 @@ let print_functions_md ~extra (doc : item) print_string =
             let args =
               List.filter
                 (fun (n, _) ->
-                  n <> "_info" && n <> "_category" && n <> "_type"
-                  && n <> "_flag" && n <> "_methods")
+                  not
+                    (List.mem n
+                       [
+                         "_info";
+                         "_category";
+                         "_type";
+                         "_flag";
+                         "_methods";
+                         "_example";
+                       ]))
                 desc
             in
             let args =
@@ -266,6 +287,17 @@ let print_lang (i : item) =
       List.remove_assoc "_category" sub
     with Not_found -> sub
   in
+  let sub =
+    let examples, sub = List.partition (fun (l, _) -> l = "_example") sub in
+    let examples = List.map snd examples in
+    let examples = List.map (fun e -> e#get_doc) examples in
+    List.iter
+      (fun e ->
+        Format.fprintf ff "@.Example:@.@.";
+        Format.fprintf ff "@[<2>%s@]@." e)
+      examples;
+    sub
+  in
   let meths, sub =
     try
       ( (List.assoc "_methods" sub)#get_subsections,
