@@ -38,7 +38,7 @@ class virtual base ~name ~kind ~restart ~restart_on_error ~on_data ?read_header
     (** The header was already read. *)
     val mutable header_read = false
 
-    method stype = Source.Fallible
+    method stype = `Fallible
 
     method wake_up _ =
       let on_stdout reader =
@@ -65,7 +65,7 @@ class virtual base ~name ~kind ~restart ~restart_on_error ~on_data ?read_header
       let log s = self#log#important "%s" s in
       process <-
         Some
-          (Process_handler.run ~priority:Tutils.Blocking ~on_stop ~on_stdout
+          (Process_handler.run ~priority:`Blocking ~on_stop ~on_stdout
              ~on_stderr ~log command)
 
     method sleep =
