@@ -52,12 +52,6 @@ module type Lame_t = sig
   val encode_buffer_float :
     encoder -> float array -> float array -> int -> string
 
-  val encode_buffer_float_ba :
-    encoder ->
-    (float, Bigarray.float32_elt, Bigarray.c_layout) Bigarray.Array1.t ->
-    (float, Bigarray.float32_elt, Bigarray.c_layout) Bigarray.Array1.t ->
-    string
-
   val encode_flush : encoder -> string
 
   module Id3tag : sig
