@@ -115,15 +115,22 @@ let () =
              code msg)
     | _ -> None)
 
-let fail msg =
-  raise
-    Runtime_error.(Runtime_error { kind = "http"; msg = Some msg; pos = [] })
+let fail ~pos message = Runtime_error.raise ~pos ~message "http"
+let interrupt = Atomic.make false
+let () = Lifecycle.on_core_shutdown (fun () -> Atomic.set interrupt true)
 
-let parse_http_answer s =
+let mk_read =
+  let interrupt = Atomic.make false in
+  Lifecycle.on_core_shutdown (fun () -> Atomic.set interrupt true);
+  fun fn len ->
+    if Atomic.get interrupt then Curl.Abort
+    else (try Curl.Proceed (fn len) with _ -> Curl.Abort)
+
+let parse_http_answer ~pos s =
   let f v c s = (v, c, s) in
   try Scanf.sscanf s "HTTP/%s %i %[^\r^\n]" f with
-    | Scanf.Scan_failure s -> fail s
-    | _ -> fail "Unknown error"
+    | Scanf.Scan_failure s -> fail ~pos s
+    | _ -> fail ~pos "Unknown error"
 
 let should_stop =
   let should_stop = ref false in
@@ -136,58 +143,81 @@ let should_stop =
   fun _ _ _ _ -> should_stop ()
 
 let rec http_request ?headers ?http_version ~follow_redirect ~timeout ~url
-    ~request ~on_body_data () =
+    ~request ~on_body_data ~pos () =
   let connection = new Curl.handle in
   try
+    (* Check url correctness, fix fixable mistakes.
+     * See: https://github.com/savonet/liquidsoap/issues/2551 *)
+    let url = Uri.(to_string (of_string url)) in
     connection#set_url url;
-    connection#set_writefunction (fun s ->
-        on_body_data s;
-        String.length s);
-    ignore
-      (Option.map
-         (fun headers ->
-           connection#set_httpheader
-             (List.map (fun (k, v) -> Printf.sprintf "%s: %s" k v) headers))
-         headers);
+    connection#set_useragent Http.user_agent;
     connection#set_httpversion
       (match http_version with
         | None -> Curl.HTTP_VERSION_NONE
         | Some "1.0" -> Curl.HTTP_VERSION_1_0
         | Some "1.1" -> Curl.HTTP_VERSION_1_1
         | Some "2.0" -> Curl.HTTP_VERSION_2
-        | Some v -> fail (Printf.sprintf "Unsupported http version %s" v));
-    connection#set_timeoutms timeout;
+        | Some v -> fail ~pos (Printf.sprintf "Unsupported http version %s" v));
+    ignore (Option.map connection#set_timeoutms timeout);
     (match request with
       | `Get -> connection#set_httpget true
-      | `Post data ->
+      | `Post (len, get_data) ->
           connection#set_post true;
-          connection#set_postfieldsize (String.length data);
-          connection#set_postfields data
-      | `Put data ->
+          (match len with
+            | Some len -> connection#set_postfieldsizelarge len
+            | None -> connection#set_httpheader ["Transfer-Encoding: chunked"]);
+          connection#set_readfunction2 (mk_read get_data)
+      | `Put (len, get_data) ->
           connection#set_put true;
-          connection#set_postfieldsize (String.length data);
-          connection#set_postfields data
+          (match len with
+            | Some len -> connection#set_postfieldsizelarge len
+            | None -> connection#set_httpheader ["Transfer-Encoding: chunked"]);
+          connection#set_readfunction2 (mk_read get_data)
       | `Head -> connection#set_nobody true
       | `Delete -> connection#set_customrequest "DELETE");
+    ignore
+      (Option.map
+         (fun headers ->
+           connection#set_httpheader
+             (List.map (fun (k, v) -> Printf.sprintf "%s: %s" k v) headers))
+         headers);
     let response_headers = Buffer.create 1024 in
     connection#set_headerfunction (fun s ->
         Buffer.add_string response_headers s;
         String.length s);
     connection#set_xferinfofunction should_stop;
+    let accepted = ref false in
+    let accepted_m = Mutex.create () in
+    let pending = Buffer.create 1024 in
+    connection#set_writefunction (fun s ->
+        Tutils.mutexify accepted_m
+          (fun () ->
+            if !accepted then on_body_data s else Buffer.add_string pending s)
+          ();
+        String.length s);
     connection#set_noprogress false;
     connection#perform;
     match connection#get_redirecturl with
       | url when url <> "" && follow_redirect ->
           connection#cleanup;
+          Tutils.mutexify accepted_m (fun () -> Buffer.clear pending) ();
           http_request ?headers ?http_version ~follow_redirect ~timeout ~url
-            ~request ~on_body_data ()
+            ~request ~on_body_data ~pos ()
       | _ ->
+          Tutils.mutexify accepted_m
+            (fun () ->
+              begin
+                on_body_data (Buffer.contents pending);
+                Buffer.clear pending;
+                accepted := true
+              end)
+            ();
           let response_headers =
             Pcre.split ~rex:(Pcre.regexp "[\r]?\n")
               (Buffer.contents response_headers)
           in
-          let protocol_version, status_code, status_message =
-            parse_http_answer (List.hd response_headers)
+          let http_version, status_code, status_message =
+            parse_http_answer ~pos (List.hd response_headers)
           in
           let response_headers =
             List.fold_left
@@ -203,7 +233,8 @@ let rec http_request ?headers ?http_version ~follow_redirect ~timeout ~url
               [] (List.tl response_headers)
           in
           connection#cleanup;
-          (protocol_version, status_code, status_message, response_headers)
+          (http_version, status_code, status_message, response_headers)
   with exn ->
+    let bt = Printexc.get_raw_backtrace () in
     connection#cleanup;
-    raise exn
+    Printexc.raise_with_backtrace exn bt
