@@ -20,7 +20,7 @@
 
  *****************************************************************************)
 
-(* A custom implementation of HTTP 
+(* A custom implementation of HTTP
  * requests. *)
 module Liq_http = struct
   type request = Get | Post of string
@@ -39,13 +39,26 @@ module Liq_http = struct
       let timeout =
         match timeout with None -> !default_timeout | Some t -> t
       in
-      let request = match request with Get -> `Get | Post s -> `Post s in
+      let mk_read s =
+        let buf = Buffer.create 10 in
+        Buffer.add_string buf s;
+        fun len ->
+          let len = min (Buffer.length buf) len in
+          let ret = Buffer.sub buf 0 len in
+          Utils.buffer_drop buf len;
+          ret
+      in
+      let request =
+        match request with
+          | Get -> `Get
+          | Post s -> `Post (Some (Int64.of_int (String.length s)), mk_read s)
+      in
       let url = Printf.sprintf "http://%s:%d%s" host port url in
       let data = Buffer.create 1024 in
       let x, code, y, _ =
-        Liqcurl.http_request ?headers ~follow_redirect:true
+        Liqcurl.http_request ?headers ~pos:[] ~follow_redirect:true
           ~on_body_data:(Buffer.add_string data)
-          ~timeout:(int_of_float (timeout *. 1000.))
+          ~timeout:(Some (int_of_float (timeout *. 1000.)))
           ~url ~request ()
       in
       if code <> 200 then
@@ -213,9 +226,7 @@ let init host =
       reason (Printexc.to_string e);
       -1.
   in
-  let task =
-    Duppy.Async.add ~priority:Tutils.Blocking Tutils.scheduler do_submit
-  in
+  let task = Duppy.Async.add ~priority:`Blocking Tutils.scheduler do_submit in
   { task; submit_m; submissions }
 
 let submit (user, password) task length source stype songs =
