@@ -33,18 +33,19 @@ class ['a] plug ?(register_hook = fun _ -> ()) doc insensitive duplicates =
       let plugin =
         if insensitive then String.uppercase_ascii plugin else plugin
       in
-      let doc =
+      let doc () =
         match (doc, sdoc) with
-          | Some d, _ -> d
-          | _, None -> Doc.trivial "(no doc)"
-          | _, Some s -> Doc.trivial s
+          | Some d, _ -> Lazy.force d
+          | None, None -> Doc.trivial "(no doc)"
+          | None, Some s -> Doc.trivial s
       in
       if duplicates then (
-        subsections <- (plugin, doc) :: subsections;
+        subsections <- (plugin, Lazy.from_fun doc) :: subsections;
         plugins <- (plugin, v) :: plugins)
       else (
         subsections <-
-          (plugin, doc) :: List.filter (fun (k, _) -> k <> plugin) subsections;
+          (plugin, Lazy.from_fun doc)
+          :: List.filter (fun (k, _) -> k <> plugin) subsections;
         plugins <-
           (plugin, v) :: List.filter (fun (k, _) -> k <> plugin) plugins);
       register_hook (plugin, v);
@@ -78,7 +79,7 @@ let create ?(duplicates = true) ?register_hook ?insensitive ?doc plugname =
   let insensitive = match insensitive with Some true -> true | _ -> false in
   let doc = match doc with None -> "(no doc)" | Some d -> d in
   let plug = new plug ?register_hook doc insensitive duplicates in
-  plugs#add_subsection plugname (plug :> Doc.item);
+  plugs#add_subsection plugname (Lazy.from_val (plug :> Doc.item));
   plug
 
 let list () = plugs#list_subsections
