@@ -37,7 +37,7 @@ class virtual base ~(on_start : unit -> unit) ~(on_stop : unit -> unit) =
     method state = state
     method virtual private start : unit
     method virtual private stop : unit
-    method virtual stype : Source.source_t
+    method virtual stype : [ `Fallible | `Infallible ]
 
     (* Default [reset] method. Can be overriden if necessary. *)
     method reset =
@@ -75,7 +75,7 @@ class virtual active_source ?get_clock ~name ~content_kind ~clock_safe
   object (self)
     inherit Source.active_source ~name content_kind as super
     inherit base ~on_start ~on_stop as base
-    method stype = if fallible then Source.Fallible else Source.Infallible
+    method stype = if fallible then `Fallible else `Infallible
     method private wake_up _ = if autostart then base#transition_to `Started
     method private sleep = base#transition_to `Stopped
     method is_ready = state = `Started
@@ -163,16 +163,10 @@ let meth :
         ([], fun_t [] unit_t),
         "Ask the source or output to stop.",
         fun s ->
-          val_fun [] (fun _ ->
-              if s#stype = Source.Infallible then
-                raise
-                  Term.(
-                    Runtime_error
-                      {
-                        kind = "input";
-                        msg = Some "Source is infallible and cannot be stopped";
-                        pos = [];
-                      });
+          val_fun [] (fun p ->
+              if s#stype = `Infallible then
+                Runtime_error.raise ~pos:(Lang.pos p)
+                  ~message:"Source is infallible and cannot be stopped" "input";
               s#transition_to `Stopped;
               unit) );
       ( "shutdown",
