@@ -39,12 +39,6 @@ let resolve_path ?cwd path =
   let cwd = match cwd with None -> Sys.getcwd () | Some cwd -> cwd in
   if Filename.is_relative path then Filename.concat cwd path else path
 
-(* Getenv with default value. *)
-let getenv ~default value = try Sys.getenv value with Not_found -> default
-
-(* Getenv with option result. *)
-let getenv_opt value = try Some (Sys.getenv value) with Not_found -> None
-
 (* Several list utilities *)
 
 let rec prefix p l =
@@ -75,11 +69,11 @@ let remove_one f l =
   aux [] l
 
 (* Read all data from a given filename.
- * We cannot use really_input with the 
+ * We cannot use really_input with the
  * reported length of the file because
  * some OSes such as windows may do implicit
- * conversions (file opened in text mode in 
- * win32), thus making the actual number of 
+ * conversions (file opened in text mode in
+ * win32), thus making the actual number of
  * characters that can be read from the file
  * different than its reported length.. *)
 let read_all filename =
@@ -155,7 +149,7 @@ let utf8_next s i =
     | '\192' .. '\223' -> i + 2
     | '\224' .. '\239' -> i + 3
     | '\240' .. '\247' -> i + 4
-    | _ -> i + 1
+    | _ -> failwith "invalid utf8"
 
 (* Return the utf8 char code of the first utf8 character in the given
    string. *)
@@ -329,7 +323,8 @@ let unescape_hex_char s =
 let unescape_utf8_char s =
   let b = Buffer.create 1 in
   let s = String.sub s 2 4 in
-  Buffer.add_utf_8_uchar b (Uchar.of_int (int_of_string ("0x" ^ s)));
+  let c = try Uchar.of_int (int_of_string ("0x" ^ s)) with _ -> Uchar.rep in
+  Buffer.add_utf_8_uchar b c;
   Buffer.contents b
 
 let unescape_char = function
@@ -465,10 +460,61 @@ let string_of_timezone tz =
   Printf.sprintf "%+03d%02d" (tz / 3600) (abs (tz / 60) mod 60)
 
 (** Very partial strftime clone *)
-let strftime str : string =
-  let t = Unix.localtime (Unix.gettimeofday ()) in
+let strftime ?time str : string =
+  let time =
+    match time with Some time -> time | None -> Unix.gettimeofday ()
+  in
+  let t = Unix.localtime time in
+  let weekday =
+    [|
+      "Sunday";
+      "Monday";
+      "Tuesday";
+      "Wednesday";
+      "Thursday";
+      "Friday";
+      "Saturday";
+    |]
+  in
+  let weekday_abbr = [| "Sun"; "Mon"; "Tue"; "Wed"; "Thu"; "Fri"; "Sat" |] in
+  let month =
+    [|
+      "January";
+      "Febrary";
+      "March";
+      "April";
+      "May";
+      "June";
+      "July";
+      "August";
+      "September";
+      "October";
+      "November";
+      "December";
+    |]
+  in
+  let month_abbr =
+    [|
+      "Jan";
+      "Feb";
+      "Mar";
+      "Apr";
+      "May";
+      "Jun";
+      "Jul";
+      "Aug";
+      "Sep";
+      "Oct";
+      "Nov";
+      "Dec";
+    |]
+  in
   let assoc =
     [
+      ("a", weekday_abbr.(t.Unix.tm_wday));
+      ("A", weekday.(t.Unix.tm_wday));
+      ("b", month_abbr.(t.Unix.tm_mon));
+      ("B", month.(t.Unix.tm_mon));
       ("S", Printf.sprintf "%02d" t.Unix.tm_sec);
       ("M", Printf.sprintf "%02d" t.Unix.tm_min);
       ("H", Printf.sprintf "%02d" t.Unix.tm_hour);
@@ -508,7 +554,7 @@ let rec mkdir ~perm dir =
 
 (** Expand ~ notation in filenames. *)
 let home_unrelate =
-  let home = getenv_opt "HOME" in
+  let home = Sys.getenv_opt "HOME" in
   let unrel s =
     let len = String.length s in
     if len < 2 then (match home with Some h when s = "~" -> h | _ -> s)
@@ -530,8 +576,8 @@ let home_unrelate =
   unrel
 
 let get_tempdir () =
-  if Sys.win32 then getenv ~default:"C:\\temp" "TEMP"
-  else getenv ~default:"/tmp" "TMPDIR"
+  if Sys.win32 then Option.value (Sys.getenv_opt "TEMP") ~default:"C:\\temp"
+  else Option.value (Sys.getenv_opt "TMPDIR") ~default:"/tmp"
 
 (** Decode Base64-encoded data *)
 let decode64 s =
@@ -897,8 +943,7 @@ let self_sync_type sources =
               | (`Static, None), (`Static, v) -> (`Static, Some v)
               | (`Static, Some v), (`Static, v') when v = v' -> (`Static, Some v)
               | _ -> (`Dynamic, None))
-          (`Static, None)
-          sources))
+          (`Static, None) sources))
 
 let string_of_pcre_error =
   Pcre.(
